use lib './lib';

if (@ARGV < 4) {
  print "MailCommander 1.3 - console mail checking utility\n";
  print "Copyright (C) 2006 Andy Chentsov <chentsov\@gmail.com>\n\n";
  print "  Use Mail::IMAPClient Perl library by The Kernen Group, Inc.\n";
  print "  Use Mail::POP3Client Perl library by Sean Dowd\n";
  print "  Use Email::Simple Perl library by Simon Cozens, Casey West\n\n";

  print "Usage: mailcommander.pl POP3|IMAP4 <server> <username> <password> [<limit>] SSL (optional)\n";

  exit;
}

$proto = $ARGV[0];
$server = $ARGV[1];
$username = $ARGV[2];
$password = $ARGV[3];
$limit = 0;
$ssl = false;
if (@ARGV > 4) {
  if ($ARGV[4] eq "SSL") {
    $ssl = true;
  } else {
    $limit = $ARGV[4]; # TODO: convert to int
  }
}
if (@ARGV > 5) {
  if ($ARGV[5] eq "SSL") {
    $ssl = true;
  }
}

if ($ssl eq true) {
  sub error_handler {       # 1st argument is signal name
      my($sig) = @_;
      print "<error>$sig</error>\n";
      close(LOG);
      exit(0);
  }

  $SIG{__DIE__}  = \&error_handler;
}

use Text::ParseWords;
use Mail::IMAPClient;
use Mail::POP3Client;
use Email::Simple;

%Month = ( 'Jan' => '01',
           'Feb' => '02',
           'Mar' => '03',
           'Apr' => '04',
           'May' => '05',
           'Jun' => '06',
           'Jul' => '07',
           'Aug' => '08',
           'Sep' => '09',
           'Oct' => '10',
           'Nov' => '11',
           'Dec' => '12' );

sub escape {
  my $text = shift;
  $text =~ s/&/&amp;/g;
  $text =~ s/\'/&apos;/g;
  $text =~ s/\"/&quot;/g;
  $text =~ s/\</&lt;/g;
  $text =~ s/\>/&gt;/g;
  return ($text);
}

sub MailDate2Gmail {
  my $date = shift;
  ($wday, $day, $mon, $yyyy, $h, $m, $s) = ($date =~ /([A-Za-z]+),\s(\d+)\s([A-Za-z]+)\s(\d{4})\s(\d+):(\d+):(\d+)/);
  $tdate = $yyyy . "-" . $Month{$mon} . "-" . $day . "T" . $h . ":" . $m . ":" . $s . "Z";
  return $tdate;
}

print "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";

if ($proto eq "IMAP4") {
  my $port = 143;
  $mserver = Mail::IMAPClient->new();
  $mserver->Clear(20);
  $mserver->Server($server);
  $mserver->Port($port);
  $mserver->Fast_io(1);
  $mserver->Buffer(4096);
  $mserver->Uid(1);
  $mserver->Peek(1);
  $mserver->Timeout(90);

  if ($ssl eq true) {
    if (!$mserver->connect_SSL()) {
      print "<error>\n\tCan not open imap connection on [$server] with user [$username] : $@\n</error>";
      exit();
    }
  } else {
    if (!$mserver->connect()) {
      print "<error>\n\tCan not open imap connection on [$server] with user [$username] : $@\n</error>";
      exit();
    }
  }

  $mserver->User($username);
  $mserver->Authuser("PLAIN");
  #$mserver->Authuser("CRAM-MD5");
  $mserver->Password($password);
} else {
  if ($ssl eq true) {
    $mserver = new Mail::POP3Client( USER => $username,
                         PASSWORD => $password,
                         USESSL => true,
                         PORT => 995,
                         HOST => $server,
                         TIMEOUT => 90 );
  } else {
    $mserver = new Mail::POP3Client( USER => $username,
                         PASSWORD => $password,
                         HOST => $server,
                         TIMEOUT => 90 );
  }
}

if ($mserver) {
  my $logged = false;
  if ($proto eq "IMAP4") {
    if (!$mserver->login()) {
      print "<error>\n\tError login : [$server] with user [$username] : $@\n</error>";
    } else {
      $logged = true;
    }
  } else {
    if (!$mserver->login($username, $password)) {
      print "<error>\n\tLogin failed: " . $mserver->Message() . "\n</error>";
    } else {
      $logged = true;
    }
  }
  if ($logged eq true) {
    $entries = "  <headers>\n";

    if ($proto eq "IMAP4") {
      $mserver->Select("INBOX");
      $nmessages = $mserver->message_count("INBOX");
      $unread = $nmessages - $mserver->unseen_count("INBOX");
    } else {
      $nmessages = $mserver->Count();
      $unread = $nmessages;
    }

    $st = 1;
    if ($limit > 0) {
      $st = $nmessages - $limit + 1;
      if ($st < 1) {
        $st = 1;
      }
    }
    foreach ($i = $st; $i <= $nmessages; $i++) {
      my $es;
      if ($proto eq "IMAP4") {
        $es = Email::Simple->new(join '', @{ $mserver->fetch($i, "RFC822") });
      } else {
        my $headtext = "";
        $headtext = $mserver->Head($i);
        $es = Email::Simple->new($headtext);
      }
      my $from = $es->header('From');
      @words = &shellwords($from);
      if (@words > 1) {
        $fromname = $words[0];
        $fromemail = $words[1];
        $fromemail =~ s/\<//g;
        $fromemail =~ s/\>//g;
      } else {
        $fromname = "";
        $fromemail = $from;
      }
      $entries .= "    <entry>\n" .
                  "      <author>\n" .
                  "        <email>" . escape($fromemail) . "</email>\n" .
                  "        <name>" . escape($fromname) . "</name>\n" .
                  "      </author>\n" .
                  "      <title>" . escape($es->header('Subject')) . "</title>\n" .
                  "      <issued>" . MailDate2Gmail($es->header('Date')) . "</issued>\n" .
                  "    </entry>\n";
    }
        
    $entries .= "  </headers>\n";
    
    utf8::encode($entries);

    print "<inbox>\n";
    print "  <stat>\n";
    print "    <total>" . $nmessages . "</total>\n";
    print "    <unread>" . $unread . "</unread>\n";
    print "  </stat>\n";
    print $entries;
    print "</inbox>";

    if ($proto eq "IMAP4") {
      $mserver->logout();
    } else {
      $mserver->Close();
    }
  }
} else {
  if ($proto eq "IMAP4") {
    print "<error>\n\tMail::IMAPClient::LastError\n</error>";
  } else {
    print "<error>\n\tConnection error\n</error>";
  }
}
