
DockFactory = function() {
  this.items = [];

  this.register = function(def) {
    if (this.items.indexOf(def) == -1) {
      this.items.push(def);
    }
  }
}

Dock = function(x, y, w) {
  this.left = [];
  this.center = [];
  this.right = [];
  this.sep = 10;
  this.x = x;
  this.y = y;
  this.w = w;
  this.h = mainWindow.height - this.y;
  this.locked = false;
  this.popupsEnabled = true;
  this.folders = {};

  this.aligning = false;

  this.name = "dockObj";
  this.obj = eval(this.name + "=this");

  this.hint = new Hint();

  this.updateUI = function() {};

  this.isLocked = function() {
    return (this.locked && !system.event.altKey);
  }

  this.isPopupsEnabled = function() {
    return this.popupsEnabled;
  }

  this.setSepWidth = function(value) {
    this.sep = value;
    if (this.sep < 1) {
      this.sep = 1;
    }
    this.align();
  }

  this.add = function(side, object) {
    var silent = false;
    if (arguments.length > 2) {
      silent = arguments[2];
    }
    switch(side) {
      case leftSide:
        this.left.push(object);
        break;
      case centerSide:
        this.center.push(object);
        break;
      case rightSide:
        this.right.unshift(object);
        break;
    }
    if (object instanceof DockBase) {
      object.createControls();
      object.updateColor();
      object.updatePrefs();
      object.setTop(this.y);
      if (!silent) {
        object.onAdd();
      }
    }
    this.align();
    return object;
  }

  this.insert = function(side, index, object) {
    if (index == -1) {
      this.add(side, object);
      return;
    }
    switch(side) {
      case leftSide:
        this.left.insert(index, object);
        break;
      case centerSide:
        this.center.insert(index, object);
        break;
      case rightSide:
        this.right.insert(index, object);
        break;
    }
    if (object instanceof DockBase) {
      object.createControls();
      object.updateColor();
      object.updatePrefs();
      object.setTop(this.y);
    }
    this.align();
  }

  this.find = function(o) {
    for (var i = 0; i < this.left.length; i++) {
      if (this.left[i] == o) {
        return {side: leftSide, items: this.left, index: i};
      }
    }
    for (var i = 0; i < this.center.length; i++) {
      if (this.center[i] == o) {
        return {side: centerSide, items: this.center, index: i};
      }
    }
    for (var i = 0; i < this.right.length; i++) {
      if (this.right[i] == o) {
        return {side: rightSide, items: this.right, index: i};
      }
    }
    return null;
  }

  this.remove = function(o) {
    if (o instanceof DockBase) {
      o.destroy();
    }
    var info = this.find(o);
    if (info) {
      info.items.remove(o);
      this.align();
    }
  }

  this.free = function(o) {
    this.remove(o);
    if (o instanceof DockBase) {
      o.free();
    }
    delete o;
  }

  this.align = function() {
    if (this.aligning) return;
    this.aligning = true;
    try {
      suppressUpdates();
      try {
        var x = this.x + 7;
        for (var i = 0; i < this.left.length; i++) {
          if (!(this.left[i] instanceof DockBase)) continue;
          this.left[i].setLeft(x);
          x += this.left[i].getWidth() + this.sep;
          this.left[i].setTop(this.y);
        }

        var w = 0;
        for (var i = 0; i < this.center.length; i++) {
          if (!(this.center[i] instanceof DockBase)) continue;
          w += this.center[i].getWidth() + this.sep;
        }
        var x = Math.round((this.w - w) / 2);
        for (var i = 0; i < this.center.length; i++) {
          if (!(this.center[i] instanceof DockBase)) continue;
          this.center[i].setLeft(x);
          x += this.center[i].getWidth() + this.sep;
          this.center[i].setTop(this.y);
        }

        var x = this.w - 7;
        for (var i = this.right.length - 1; i >= 0; i--) {
          if (!(this.right[i] instanceof DockBase)) continue;
          this.right[i].setLeft(x - this.right[i].getWidth());
          x -= this.right[i].getWidth() + this.sep;
          this.right[i].setTop(this.y);
        }
      } finally {
        resumeUpdates();
      }
    } finally {
      this.aligning = false;
    }
  }

  this.findFrameObject = function(f) {
    for (var i = 0; i < this.left.length; i++) {
      if (!this.left[i]) continue;
      if (this.left[i].frame == f) {
        return this.left[i];
      }
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!this.center[i]) continue;
      if (this.center[i].frame == f) {
        return this.center[i];
      }
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!this.right[i]) continue;
      if (this.right[i].frame == f) {
        return this.right[i];
      }
    }
    return null;
  }

  this.findInstances = function(objType) {
    var result = [];
    for (var i = 0; i < this.left.length; i++) {
      if (!this.left[i]) continue;
      if (typeof(this.left[i]) == "function") continue;
      //if (this.left[i] instanceof objType) {
      //print("] " + this.left[i].constructor.name);
      if (this.left[i].constructor.name == objType) {
        result.push(this.left[i]);
      }
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!this.center[i]) continue;
      if (typeof(this.center[i]) == "function") continue;
      if (this.center[i].constructor.name == objType) {
        result.push(this.center[i]);
      }
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!this.right[i]) continue;
      if (typeof(this.right[i]) == "function") continue;
      if (this.right[i].constructor.name == objType) {
        result.push(this.right[i]);
      }
    }
    return result;
  }

  this.itemAt = function(x, y) {
    for (var i = 0; i < this.left.length; i++) {
      if (!this.left[i]) continue;
      if (typeof(this.left[i]) == "function") continue;
      if (!(this.left[i] instanceof DockBase)) continue;
      if ((this.left[i].getLeft() <= x) && ((this.left[i].getLeft() + this.left[i].getWidth()) >= x)) {
        return this.left[i];
      }
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!this.center[i]) continue;
      if (typeof(this.center[i]) == "function") continue;
      if (!(this.center[i] instanceof DockBase)) continue;
      if ((this.center[i].getLeft() <= x) && ((this.center[i].getLeft() + this.center[i].getWidth()) >= x)) {
        return this.center[i];
      }
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!this.right[i]) continue;
      if (typeof(this.right[i]) == "function") continue;
      if (!(this.right[i] instanceof DockBase)) continue;
      if ((this.right[i].getLeft() <= x) && ((this.right[i].getLeft() + this.right[i].getWidth()) >= x)) {
        return this.right[i];
      }
    }
    return null;
  }

  this.setPosition = function(x, y) {
    this.x = x;
    this.y = y;
    this.h = mainWindow.height - this.y;
    this.align();
  }

  this.deleteItem = function(o) {
    if (!o) return;
    try {
      var result = alert(tr("Are you sure you want to remove sensor '%s'?").replace("%s", o.getName()), tr("Yes"), tr("No"));
      if (result == 1) {
        this.free(o);
        this.save();
        //o.destroy();
        delete o;
        o = null;
      }
    }
    catch (e) { }
  }

  this.clear = function() {
    while (this.left.length > 0) {
      try {
        var o = this.left[0];
        this.free(o);
        //o.destroy();
        delete o;
        o = null;
      } catch(e) {}
    }
    while (this.center.length > 0) {
      try {
        var o = this.center[0];
        this.free(o);
        //o.destroy();
        delete o;
        o = null;
      } catch(e) {}
    }
    while (this.right.length > 0) {
      try {
        var o = this.right[0];
        this.free(o);
        //o.destroy();
        delete o;
        o = null;
      } catch(e) {}
    }
  }

  this.importXML = function(xml) {
    if (xml == "") {
      this.loadDefault();
      return;
    }

    // load
    var doc = XMLDOM.parse(xml);

    var nodes = doc.evaluate("sensors/sensor");
    if (nodes.length <= 0) {
      this.loadDefault();
      return;
    }
    var l = [];
    var c = [];
    var r = [];
    for (var i = 0; i < nodes.length; i++) {
      var side = nodes.item(i).getAttribute("side");
      var type = nodes.item(i).getAttribute("type");
      var o = createSensor(type);
      if (o) {
        o.load(nodes.item(i));
        switch(side) {
          case "left":
              l.push(o);
              break;
          case "center":
              c.push(o);
              break;
          case "right":
              r.push(o);
              break;
        }
      }
    }
    // add sensors
    r.reverse();
    for (var i in c) {
      this.add(centerSide, c[i], true);
    }
    for (var i in l) {
      this.add(leftSide, l[i], true);
    }
    for (var i in r) {
      this.add(rightSide, r[i], true);
    }
  }

  this.exportXML = function() {
    var xml = "<?xml version=\"1.0\" ?>\n<sensors>\n";

    // save
    for (var i = 0; i < this.left.length; i++) {
      if (!(this.left[i] instanceof DockBase)) continue;
      if (this.left[i].NotSave) continue;
      xml += "  <sensor side=\"left\" type=\"" + this.left[i].className + "\">\n" +
             "    " + this.left[i].save() + "\n" +
             "  </sensor>\n";
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!(this.center[i] instanceof DockBase)) continue;
      if (this.center[i].NotSave) continue;
      xml += "  <sensor side=\"center\" type=\"" + this.center[i].className + "\">\n" +
             "    " + this.center[i].save() + "\n" +
             "  </sensor>\n";
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!(this.right[i] instanceof DockBase)) continue;
      if (this.right[i].NotSave) continue;
      xml += "  <sensor side=\"right\" type=\"" + this.right[i].className + "\">\n" +
             "    " + this.right[i].save() + "\n" +
             "  </sensor>\n";
    }

    xml += "</sensors>";

    return xml;
  }

  this.loadDefault = function() {
    // create default sensors
    this.add(centerSide, new DockImage()).textProp.value = "Informer %v";
    this.add(rightSide, new DockTime());
  }

  this.load = function() {
    var xml = preferences.sensors.value;
    this.importXML(xml);
  }

  this.save = function() {
    var always = false;
    if (arguments.length > 0) {
      always = arguments[0];
    }
    var xml = this.exportXML();
    if ((preferences.sensors.value != xml) || always) {
      preferences.sensors.value = xml;
      savePreferences();
      return true;
    }
    return false;
  }

  this.importFile = function(filename) {
    if (!filesystem.itemExists(filename)) return;
    var data = filesystem.readFile(filename);
    this.clear();
    this.importXML(data);
    this.save();
  }

  this.exportFile = function(filename) {
    var data = this.exportXML();
    if (filesystem.itemExists(filename)) {
      // remove
      filesystem.remove(filename);
    }
    filesystem.writeFile(filename, data);
  }

  this.importDlg = function() {
    var f = chooseFile(".xml");
    if (!f) return;
    this.importFile(f);
  }

  this.exportDlg = function() {
    var f = saveAs(".xml");
    if (f) {
      this.exportFile(f);
    }
  }

  this.updatePrefs = function() {
    if (this.hint) {
      this.hint.updatePrefs();
      for (var i = 0; i < this.left.length; i++) {
        if (!(this.left[i] instanceof DockBase)) continue;
        this.left[i].updatePrefs();
        if (this.left[i].hint) {
          this.left[i].hint.updatePrefs();
        }
      }
      for (var i = 0; i < this.center.length; i++) {
        if (!(this.center[i] instanceof DockBase)) continue;
        this.center[i].updatePrefs();
        if (this.center[i].hint) {
          this.center[i].hint.updatePrefs();
        }
      }
      for (var i = 0; i < this.right.length; i++) {
        if (!(this.right[i] instanceof DockBase)) continue;
        this.right[i].updatePrefs();
        if (this.right[i].hint) {
          this.right[i].hint.updatePrefs();
        }
      }
    }
  }

  this.updateSensors = function() {
    for (var i = 0; i < this.left.length; i++) {
      if (!this.left[i]) continue;
      if (this.left[i] instanceof DockBase) {
        this.left[i].onAdd();
      }
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!this.center[i]) continue;
      if (this.center[i] instanceof DockBase) {
        this.center[i].onAdd();
      }
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!this.right[i]) continue;
      if (this.right[i] instanceof DockBase) {
        this.right[i].onAdd();
      }
    }
  }

  this.wakeUp = function() {
    for (var i = 0; i < this.left.length; i++) {
      if (!(this.left[i] instanceof DockBase)) continue;
      this.left[i].wakeUp();
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!(this.center[i] instanceof DockBase)) continue;
      this.center[i].wakeUp();
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!(this.right[i] instanceof DockBase)) continue;
      this.right[i].wakeUp();
    }
  }

  this.loseFocus = function() {
    for (var i = 0; i < this.left.length; i++) {
      if (!(this.left[i] instanceof DockBase)) continue;
      this.left[i].loseFocus();
    }
    for (var i = 0; i < this.center.length; i++) {
      if (!(this.center[i] instanceof DockBase)) continue;
      this.center[i].loseFocus();
    }
    for (var i = 0; i < this.right.length; i++) {
      if (!(this.right[i] instanceof DockBase)) continue;
      this.right[i].loseFocus();
    }
  }

  this.openURL = function(url) {
    if (system.platform == "windows") {
      runCommandInBg("cmd.exe /c \"start " + url.replace(/([\&\|\(\)]+)/g, "\^$1") + "\"", "openURL");
      //openURL(url);
    } else {
      openURL(url);
    }
  },

  this.openRSS = function(url) {
    var xml = "";
    if (arguments.length > 1) {
      xml = arguments[1];
    }
    var link = "";
    if (arguments.length > 2) {
      link = arguments[2];
    }
    var webMode = (preferences.rssMode.value == "web");
    if ((link != "") && webMode) {
      this.openURL(link);
      return;
    }

    var widgetMode = (preferences.rssMode.value == "widget");
    if (widgetMode) {
      var widget_file = system.userWidgetsFolder + "/NewsStand.widget";
      if (filesystem.itemExists(widget_file)) {
        tellWidget("NewsStand", "load_feed " + url);
      } else {
        if (alert(tr("To use NewsStand widget you must download it from WidgetGallery.\n\nDownload?"), tr("Download"), tr("Cancel")) == 1) {
          this.openURL("http://widgets.yahoo.com/gallery/view.php?widget=42031");
        }
      }
    } else {
      var reader = preferences.appRSS.value;
      if (reader != "") {
        // TODO: process % macroses
        //runCommandInBg("\"" + reader + "\" " + url, "rssClient");
        runCommandInBg("\"" + reader + "\"", "rssClient");
      }
    }
  }

  this.openMail = function(username) {
    var reader = preferences.appMail.value;
    if (reader != "") {
      // TODO: process % macroses
      //runCommandInBg("\"" + reader + "\" " + username, "mailClient");
      runCommandInBg("\"" + reader + "\"", "mailClient");
    } else {
      if (system.platform == "windows") {
        try {
          var reg = COM.createObject("Wscript.Shell");
          var clientName = reg.RegRead("HKLM\\Software\\Clients\\mail\\");
          var clientExe = reg.RegRead("HKLM\\Software\\Clients\\Mail\\" + clientName + "\\shell\\open\\command\\");

          clientExe = clientExe.replace(/\%/, "\$");
          clientExe = clientExe.replace(/\%/, "");

          if (clientExe.substr(0, 1) != "\"") {
            clientExe = "\"" + clientExe + "\"";
          }
          //runCommandInBg(clientExe.replace(/\\/g, "\\\\"), "mailClient");
          runCommandInBg("cmd.exe /c " + clientExe.replace(/\\/g, "\\\\"), "mailClient");
        } catch(e) {}
      }
    }
  }

  this.openCalendar = function(_date) {
    var cal = preferences.appCalendar.value;
    var date = new Date(_date);
    if (cal != "") {
      // TODO: process % macroses
      //runCommandInBg("\"" + cal + "\" " + date, "calendarClient");
      runCommandInBg("\"" + cal + "\"", "calendarClient");
    } else {
      if (system.platform == "windows") {
        runCommandInBg("rundll32.exe shell32.dll,Control_RunDLL timedate.cpl", "datetimeApplet");
      }
    }
  }

  this.makeLink = function(link, target) {
    var fname = link.replace(/[\:\/\\\&\+\?\*\=\.]+/g, "-");
    var text = "";
    if (system.platform == "windows") {
      text = "[InternetShortcut]\nURL=" + link + "\n";
      fname += ".url";
    } else {
      text = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
             "<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n" +
             "<plist version=\"1.0\">\n" +
             "<dict>\n" +
             "        <key>URL</key>\n" +
             "        <string>" + link + "</string>\n" +
             "</dict>\n" +
             "</plist>";
      fname += ".webloc";
    }
    if (text != "") {
      filesystem.writeFile(target + "/" + fname, text);
    }
  }

  this.getShellFolder = function(folderName, common) {
    try {
      common = (common != null)?common:false;
      var result = runCommand('REG QUERY "' + (common?"HKLM":"HKCU") + '\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders" /v "' + (common?"Common ":"") + folderName + '"');
      result = result.split("\n");
      for(var i = 0; i < result.length; i++) {
        var str = result[i].trim();
        if (str.startWith((common?"Common ":"") + folderName)) {
          str = str.replace((common?"Common ":"") + folderName, "");
          str = str.replace("REG_SZ", "");
          /*var tmp = str.split("\t");
          if (tmp.length > 2) {
            print("# " + tmp[2]);
            return tmp[2];
          }
          break;*/
          return str.trim();
        }
      }
      return "";
    } catch(e) {
      return "";
    }
  }

  this.collectSystemInformation = function() {
    if (system.platform != "windows") return;
    var engine = runCommand('cmd.exe /C "FType YahooWidget"');
    engine = engine.split("=", 2);
    engine = engine.length > 1 ? engine[1].split('"')[1] : "";
    engine = engine.split("\\");
    engine.pop();
    this.folders["Konfabulator"] = engine.join("\\");

    this.folders["Programs"] = this.getShellFolder("Programs");
    this.folders["Common Programs"] = this.getShellFolder("Programs", true);
  }

}

function createSensor(className) {
  try {
    var s = eval("new " + className);
    return s;
  } catch(e) {
    return null;
  }
}

var dockFactory = new DockFactory();
