//MooTools, My Object Oriented Javascript Tools. Copyright (c) 2006 Valerio Proietti, <http://mad4milk.net>, MIT Style License.

var MooTools = {
        'version': 1.1
};

function $defined(obj){
        return (obj != undefined);
};

function $type(obj){
        if (!$defined(obj)) return false;
        if (obj.htmlElement) return 'element';
        var type = typeof obj;
        if (type == 'object' && obj.nodeName){
                switch(obj.nodeType){
                        case 1: return 'element';
                        case 3: return /\S/.test(obj.nodeValue) ? 'textnode' : 'whitespace';
                }
        }
        if (type == 'object' || type == 'function'){
                switch(obj.constructor){
                        case Array: return 'array';
                        case RegExp: return 'regexp';
                        case Class: return 'class';
                }
                if (typeof obj.length == 'number'){
                        if (obj.item) return 'collection';
                        if (obj.callee) return 'arguments';
                }
        }
        return type;
};

function $merge(){
        var mix = {};
        for (var i = 0; i < arguments.length; i++){
                for (var property in arguments[i]){
                        var ap = arguments[i][property];
                        var mp = mix[property];
                        if (mp && $type(ap) == 'object' && $type(mp) == 'object') mix[property] = $merge(mp, ap);
                        else mix[property] = ap;
                }
        }
        return mix;
};

var $extend = Object.extend = function(){
        var args = arguments;
        if (!args[1]) args = [this, args[0]];
        for (var property in args[1]) args[0][property] = args[1][property];
        return args[0];
};

var $native = Object.Native = function(){
        for (var i = 0, l = arguments.length; i < l; i++){
                arguments[i].extend = function(props){
                        for (var prop in props){
                                if (!this.prototype[prop]) this.prototype[prop] = props[prop];
                                if (!this[prop]) this[prop] = $native.generic(prop);
                        }
                };
        }
};

$native.generic = function(prop){
        return function(bind){
                return this.prototype[prop].apply(bind, Array.prototype.slice.call(arguments, 1));
        };
};

$native(Function, Array, String, Number);

var Abstract = function(obj){
        obj = obj || {};
        obj.extend = $extend;
        return obj;
};

function $chk(obj){
        return !!(obj || obj === 0);
};

function $pick(obj, picked){
        return $defined(obj) ? obj : picked;
};

function $random(min, max){
        return Math.floor(Math.random() * (max - min + 1) + min);
};

function $time(){
        return new Date().getTime();
};

function $clear(timer){
        clearTimeout(timer);
        clearInterval(timer);
        return null;
};

var Class = function(properties){
        var klass = function(){
                return (arguments[0] !== null && this.initialize && $type(this.initialize) == 'function') ? this.initialize.apply(this, arguments) : this;
        };
        $extend(klass, this);
        klass.prototype = properties;
        klass.constructor = Class;
        return klass;
};

Class.empty = function(){};

Class.prototype = {

        extend: function(properties){
                //var proto = new this(null);
                var klass = this;
                var proto = new klass(null);
                for (var property in properties){
                        var pp = proto[property];
                        proto[property] = Class.Merge(pp, properties[property]);
                }
                return new Class(proto);
        },

        implement: function(){
                for (var i = 0, l = arguments.length; i < l; i++) $extend(this.prototype, arguments[i]);
        }

};

Class.Merge = function(previous, current){
        if (previous && previous != current){
                var type = $type(current);
                if (type != $type(previous)) return current;
                switch(type){
                        case 'function':
                                var merged = function(){
                                        this.parent = arguments.callee.parent;
                                        return current.apply(this, arguments);
                                };
                                merged.parent = previous;
                                return merged;
                        case 'object': return $merge(previous, current);
                }
        }
        return current;
};

var Chain = new Class({

        chain: function(fn){
                this.chains = this.chains || [];
                this.chains.push(fn);
                return this;
        },

        callChain: function(){
                if (this.chains && this.chains.length) this.chains.shift().delay(10, this);
        },

        clearChain: function(){
                this.chains = [];
        }

});

var Events = new Class({

        addEvent: function(type, fn){
                if (fn != Class.empty){
                        this.$events = this.$events || {};
                        this.$events[type] = this.$events[type] || [];
                        this.$events[type].include(fn);
                }
                return this;
        },

        fireEvent: function(type, args, delay){
                if (this.$events && this.$events[type]){
                        this.$events[type].each(function(fn){
                                fn.create({'bind': this, 'delay': delay, 'arguments': args})();
                        }, this);
                }
                return this;
        },

        removeEvent: function(type, fn){
                if (this.$events && this.$events[type]) this.$events[type].remove(fn);
                return this;
        }

});

var Options = new Class({

        setOptions: function(){
                this.options = $merge.apply(null, [this.options].extend(arguments));
                if (!this.addEvent) return this;
                for (var option in this.options){
                        if ($type(this.options[option] == 'function') && option.test(/^on[A-Z]/)) this.addEvent(option, this.options[option]);
                }
                return this;
        }

});

Array.extend({

        forEach: function(fn, bind){
                for (var i = 0, j = this.length; i < j; i++) fn.call(bind, this[i], i, this);
        },

        filter: function(fn, bind){
                var results = [];
                for (var i = 0, j = this.length; i < j; i++){
                        if (fn.call(bind, this[i], i, this)) results.push(this[i]);
                }
                return results;
        },

        map: function(fn, bind){
                var results = [];
                for (var i = 0, j = this.length; i < j; i++) results[i] = fn.call(bind, this[i], i, this);
                return results;
        },

        every: function(fn, bind){
                for (var i = 0, j = this.length; i < j; i++){
                        if (!fn.call(bind, this[i], i, this)) return false;
                }
                return true;
        },

        some: function(fn, bind){
                for (var i = 0, j = this.length; i < j; i++){
                        if (fn.call(bind, this[i], i, this)) return true;
                }
                return false;
        },

        indexOf: function(item, from){
                var len = this.length;
                for (var i = (from < 0) ? Math.max(0, len + from) : from || 0; i < len; i++){
                        if (this[i] === item) return i;
                }
                return -1;
        },

        copy: function(start, length){
                start = start || 0;
                if (start < 0) start = this.length + start;
                length = length || (this.length - start);
                var newArray = [];
                for (var i = 0; i < length; i++) newArray[i] = this[start++];
                return newArray;
        },

        remove: function(item){
                var i = 0;
                var len = this.length;
                while (i < len){
                        if (this[i] === item){
                                this.splice(i, 1);
                                len--;
                        } else {
                                i++;
                        }
                }
                return this;
        },

        contains: function(item, from){
                return this.indexOf(item, from) != -1;
        },

        associate: function(keys){
                var obj = {}, length = Math.min(this.length, keys.length);
                for (var i = 0; i < length; i++) obj[keys[i]] = this[i];
                return obj;
        },

        extend: function(array){
                for (var i = 0, j = array.length; i < j; i++) this.push(array[i]);
                return this;
        },

        merge: function(array){
                for (var i = 0, l = array.length; i < l; i++) this.include(array[i]);
                return this;
        },

        include: function(item){
                if (!this.contains(item)) this.push(item);
                return this;
        },

        getRandom: function(){
                return this[$random(0, this.length - 1)] || false;
        },

        getLast: function(){
                return this[this.length - 1] || false;
        },

        // DUKE

        insert: function(index, value) {
                this.splice(index, 0, value);
        },

        removeByIndex: function(value) {
                this.splice(value, 1);
                return this;
        },

        countInstance: function(type) {
                var count = 0;
                for (var i = 0; i < this.length; i++) {
                  var item = this[i];
                  if (item instanceof type) count++;
                }
                return count;
        },

        equal: function(arr) {
                if (this.length != arr.length) return false;
                for (var i = 0; i < arr.length; i++) {
                  if (this[i].equal) { //likely nested array
                    if (!this[i].equal(arr[i])) return false;
                    else continue;
                  }
                  if (this[i] != arr[i]) return false;
                }
                return true;
        },

        extend: function(newArray) {
                var pos = this.length;
                for (var i = 0; i < newArray.length; i++) this[pos++] = newArray[i];
                return this;
        }

});

Array.prototype.each = Array.prototype.forEach;
Array.prototype.test = Array.prototype.contains;

function $A(array){
        return Array.copy(array);
};

function $each(iterable, fn, bind){
        if (iterable && typeof iterable.length == 'number' && $type(iterable) != 'object') Array.forEach(iterable, fn, bind);
        else for (var name in iterable) fn.call(bind || iterable, iterable[name], name);
};

String.extend({

        test: function(regex, params){
                return (($type(regex) == 'string') ? new RegExp(regex, params) : regex).test(this);
        },

        toInt: function(){
                return parseInt(this, 10);
        },

        toFloat: function(){
                return parseFloat(this);
        },

        camelCase: function(){
                return this.replace(/-\D/g, function(match){
                        return match.charAt(1).toUpperCase();
                });
        },

        hyphenate: function(){
                return this.replace(/\w[A-Z]/g, function(match){
                        return (match.charAt(0) + '-' + match.charAt(1).toLowerCase());
                });
        },

        capitalize: function(){
                return this.replace(/\b[a-z]/g, function(match){
                        return match.toUpperCase();
                });
        },

        trim: function(){
                return this.replace(/^\s+|\s+$/g, '');
        },

        clean: function(){
                return this.replace(/\s{2,}/g, ' ').trim();
        },

        rgbToHex: function(array){
                var rgb = this.match(/\d{1,3}/g);
                return (rgb) ? rgb.rgbToHex(array) : false;
        },

        hexToRgb: function(array){
                var hex = this.match(/^#?(\w{1,2})(\w{1,2})(\w{1,2})$/);
                return (hex) ? hex.slice(1).hexToRgb(array) : false;
        },

        contains: function(string, s){
                return (s) ? (s + this + s).indexOf(s + string + s) > -1 : this.indexOf(string) > -1;
        },

        escapeRegExp: function(){
                return this.replace(/([.*+?^${}()|[\]\/\\])/g, '\\$1');
        },

        // DUKE

        startWith: function(str) {
                return (this.indexOf(str) == 0);
        }

});

Array.extend({

        rgbToHex: function(array){
                if (this.length < 3) return false;
                if (this.length == 4 && this[3] == 0 && !array) return 'transparent';
                var hex = [];
                for (var i = 0; i < 3; i++){
                        var bit = (this[i] - 0).toString(16);
                        hex.push((bit.length == 1) ? '0' + bit : bit);
                }
                return array ? hex : '#' + hex.join('');
        },

        hexToRgb: function(array){
                if (this.length != 3) return false;
                var rgb = [];
                for (var i = 0; i < 3; i++){
                        rgb.push(parseInt((this[i].length == 1) ? this[i] + this[i] : this[i], 16));
                }
                return array ? rgb : 'rgb(' + rgb.join(',') + ')';
        }

});

Function.extend({

        create: function(options){
                var fn = this;
                options = $merge({
                        'bind': fn,
                        'event': false,
                        'arguments': null,
                        'delay': false,
                        'periodical': false,
                        'attempt': false
                }, options);
                if ($chk(options.arguments) && $type(options.arguments) != 'array') options.arguments = [options.arguments];
                return function(event){
                        var args;
                        if (options.event){
                                //event = event || window.event;
                                //event = null;
                                args = [(options.event === true) ? event : new options.event(event)];
                                if (options.arguments) args.extend(options.arguments);
                        }
                        else args = options.arguments || arguments;
                        var returns = function(){
                                return fn.apply($pick(options.bind, fn), args);
                        };
                        if (options.delay) return setTimeout(returns, options.delay);
                        if (options.periodical) return setInterval(returns, options.periodical);
                        if (options.attempt) try {return returns();} catch(err){return false;};
                        return returns();
                };
        },

        pass: function(args, bind){
                return this.create({'arguments': args, 'bind': bind});
        },

        attempt: function(args, bind){
                return this.create({'arguments': args, 'bind': bind, 'attempt': true})();
        },

        bind: function(bind, args){
                return this.create({'bind': bind, 'arguments': args});
        },

        bindAsEventListener: function(bind, args){
                return this.create({'bind': bind, 'event': true, 'arguments': args});
        },

        delay: function(delay, bind, args){
                return this.create({'delay': delay, 'bind': bind, 'arguments': args})();
        },

        periodical: function(interval, bind, args){
                return this.create({'periodical': interval, 'bind': bind, 'arguments': args})();
        }

});

Number.extend({

        toInt: function(){
                return parseInt(this);
        },

        toFloat: function(){
                return parseFloat(this);
        },

        limit: function(min, max){
                return Math.min(max, Math.max(min, this));
        },

        round: function(precision){
                precision = Math.pow(10, precision || 0);
                return Math.round(this * precision) / precision;
        },

        times: function(fn) {
                for (var i = 0; i < this; i++) fn(i);
        }

});

var Hash = new Class({

        length: 0,

        initialize: function(object){
                this.obj = object || {};
                this.setLength();
        },

        get: function(key){
                return (this.hasKey(key)) ? this.obj[key] : null;
        },

        hasKey: function(key){
                return (key in this.obj);
        },

        set: function(key, value){
                if (!this.hasKey(key)) this.length++;
                this.obj[key] = value;
                return this;
        },

        setLength: function(){
                this.length = 0;
                for (var p in this.obj) this.length++;
                return this;
        },

        remove: function(key){
                if (!this.hasKey(key)) return this;
                delete this.obj[key];
                this.length--;
                return this;
        },

        each: function(fn, bind){
                $each(this.obj, fn, bind);
        },

        extend: function(obj){
                $extend(this.obj, obj);
                return this.setLength();
        },

        merge: function(){
                this.obj = $merge.apply(null, [this.obj].extend(arguments));
                return this.setLength();
        },

        empty: function(){
                this.obj = {};
                this.length = 0;
                return this;
        },

        keys: function(){
                var keys = [];
                for (var property in this.obj) keys.push(property);
                return keys;
        },

        values: function(){
                var values = [];
                for (var property in this.obj) values.push(this.obj[property]);
                return values;
        }

});

function $H(obj){
        return new Hash(obj);
};

var Color = new Class({

        initialize: function(color, type){
                type = type || (color.push ? 'rgb' : 'hex');
                var rgb, hsb;
                switch(type){
                        case 'rgb':
                                rgb = color;
                                hsb = rgb.rgbToHsb();
                                break;
                        case 'hsb':
                                rgb = color.hsbToRgb();
                                hsb = color;
                                break;
                        default:
                                rgb = color.hexToRgb(true);
                                hsb = rgb.rgbToHsb();
                }
                rgb.hsb = hsb;
                rgb.hex = rgb.rgbToHex();
                return $extend(rgb, Color.prototype);
        },

        mix: function(){
                var colors = $A(arguments);
                var alpha = ($type(colors[colors.length - 1]) == 'number') ? colors.pop() : 50;
                var rgb = this.copy();
                colors.each(function(color){
                        color = new Color(color);
                        for (var i = 0; i < 3; i++) rgb[i] = Math.round((rgb[i] / 100 * (100 - alpha)) + (color[i] / 100 * alpha));
                });
                return new Color(rgb, 'rgb');
        },

        invert: function(){
                return new Color(this.map(function(value){
                        return 255 - value;
                }));
        },

        setHue: function(value){
                return new Color([value, this.hsb[1], this.hsb[2]], 'hsb');
        },

        setSaturation: function(percent){
                return new Color([this.hsb[0], percent, this.hsb[2]], 'hsb');
        },

        setBrightness: function(percent){
                return new Color([this.hsb[0], this.hsb[1], percent], 'hsb');
        }

});

function $RGB(r, g, b){
        return new Color([r, g, b], 'rgb');
};

function $HSB(h, s, b){
        return new Color([h, s, b], 'hsb');
};

Array.extend({

        rgbToHsb: function(){
                var red = this[0], green = this[1], blue = this[2];
                var hue, saturation, brightness;
                var max = Math.max(red, green, blue), min = Math.min(red, green, blue);
                var delta = max - min;
                brightness = max / 255;
                saturation = (max != 0) ? delta / max : 0;
                if (saturation == 0){
                        hue = 0;
                } else {
                        var rr = (max - red) / delta;
                        var gr = (max - green) / delta;
                        var br = (max - blue) / delta;
                        if (red == max) hue = br - gr;
                        else if (green == max) hue = 2 + rr - br;
                        else hue = 4 + gr - rr;
                        hue /= 6;
                        if (hue < 0) hue++;
                }
                return [Math.round(hue * 360), Math.round(saturation * 100), Math.round(brightness * 100)];
        },

        hsbToRgb: function(){
                var br = Math.round(this[2] / 100 * 255);
                if (this[1] == 0){
                        return [br, br, br];
                } else {
                        var hue = this[0] % 360;
                        var f = hue % 60;
                        var p = Math.round((this[2] * (100 - this[1])) / 10000 * 255);
                        var q = Math.round((this[2] * (6000 - this[1] * f)) / 600000 * 255);
                        var t = Math.round((this[2] * (6000 - this[1] * (60 - f))) / 600000 * 255);
                        switch(Math.floor(hue / 60)){
                                case 0: return [br, t, p];
                                case 1: return [q, br, p];
                                case 2: return [p, br, t];
                                case 3: return [p, q, br];
                                case 4: return [t, p, br];
                                case 5: return [br, p, q];
                        }
                }
                return false;
        }

});