/**
 * @author Andy Chentsov
 */

var NotificationManager = new Class({

  getOptions: function(){
    return {
      maxDisplay: /*4*/1,
      color: "#FFFFFF",
      titleColor: "#006CA3",
      textColor: "#000000",
      opacity: 210,
      corner: ncRight
    };
  },
   
  initialize: function(options) {
    this.notifications = [];
    this.previwed = false;
    this.setOptions(this.getOptions(), options);
    this.updatePrefs();
  },

  add: function(notify) {
    this.notifications.push(notify);
    return notify;
  },

  prepare: function(notify) {
    this.previwed = false;
    // XXX: slide previous notifies up/down
    var idx = this.notifications.indexOf(notify);
    var top = 0;
    if (idx > 0) {
      top = this.notifications[idx - 1].win.vOffset - 5 - notify.win.height;
    }
    if (top != 0) {
      notify.win.vOffset = top;
    }
  },

  next: function(notify) {
    notify.destroy();
    this.notifications.remove(notify);
    if (this.notifications.length <= 0) return;
    this.notifications[0].show();
  },

  previewNext: function(notify) {
    var idx = this.notifications.indexOf(notify);
    if (idx < (this.notifications.length - 1) && !this.previwed) {
      this.previwed = true;
      this.notifications[idx + 1].preview();
    }
  },
  
  notify: function(params) {
    /*var options = params;
    options.extend(this.options);*/
    // reset
    this.options = this.getOptions();
    var options = this.options;
    options.extend(params);
    var n = this.add(new Notify(options)).
      addEvent('onBeforeShow', this.prepare.bind(this)).
      addEvent('onShow', this.previewNext.bind(this)).
      addEvent('onHide', this.next.bind(this));
    if (this.notifications.length <= this.options.maxDisplay) {
      n.show();
    } else {
      if (!this.previwed) {
        this.previwed = true;
        n.preview();
      }
    }
  },

  closeAll: function() {
    for (var i = 0; i < this.notifications.length; i++) {
      if (typeof(this.notifications[i]) == "object") {
        this.notifications[i].destroy();
        delete this.notifications[i];
      }
    }
    this.notifications = [];
    this.previwed = false;
  },

  updatePrefs: function() {
    this.closeAll();
    this.options.color = preferences.notifyColor.value;
    this.options.titleColor = preferences.notifyTitleColor.value;
    this.options.textColor = preferences.notifyTextColor.value;
    this.options.opacity = preferences.notifyOpacity.value;
    this.options.corner = preferences.notifySide.value;
  }
   
});

NotificationManager.implement(new Events);
NotificationManager.implement(new Options);
