
var networkProvider = new Class({

  devices: [],
  current: { name: null, active: false, device: null },
  lastTime:  0,
  ReceivedPerSec: 0,
  SentPerSec: 0,
  lastIn: 0,
  lastOut: 0,
  maxIn: 1,
  maxOut: 1,
  
  initialize: function() {
    this.getList();
  },
  
  // external interface
  // -------------------------------------------------------------------------
  getList: function() {
    if (this.current.name) {
      this.setCurrentDevice(this.current.name);
    }
  },
  
  setCurrentDevice: function(name) {
    this.current = { name: name, active: false, device: null };
    for (var i = 0; i < this.devices.length; i++) {
      if (this.devices[i].name == name) {
        this.current.device = this.devices[i];
        break;
      }
    }
    this.lastTime = 0;
    this.maxIn = 1;
    this.maxOut = 1;
  },
  
  updateData: function() {
    var now = new Date();
    var timeDiff = now.getTime() - this.lastTime;
    if (timeDiff >= 1000) {
      var net = this.getIO();
      if (this.lastTime != 0) {
        this.current.active = true;
        var inDiff = net.bytesIn - this.lastIn;
        this.ReceivedPerSec = (inDiff < 0) ? 0 : Math.round((inDiff * 1000) / timeDiff);
        var outDiff = net.bytesOut - this.lastOut;
        this.SentPerSec = (outDiff < 0) ? 0 : Math.round((outDiff * 1000) / timeDiff);
        
        if (this.maxIn < this.ReceivedPerSec) {
          this.maxIn = this.ReceivedPerSec;
        }
        if (this.maxOut < this.SentPerSec) {
          this.maxOut = this.SentPerSec;
        }
      } else {
        this.current.active = false;
        this.ReceivedPerSec = 0;
        this.SentPerSec = 0;
      }
      this.lastTime = now.getTime();
      this.lastIn = net.bytesIn;
      this.lastOut = net.bytesOut;
    }
  },

  getData: function() {
    return { bytesIn: this.ReceivedPerSec, bytesOut: this.SentPerSec, 
             maxIn: this.maxIn, maxOut: this.maxOut };
  },

  // internal functions
  // -------------------------------------------------------------------------
  getIO: function() {
    this.lastTime = 0;
    return { bytesIn: 0, bytesOut: 0 };
  },

  wakeUp: function() {
    this.lastTime = 0;
  }
  
});