
// Initial german translation by Patrick Steinke <profiart (at) gmail (dot) com>
// German translation by Torsten Herz

Date.convMonthShort = ["Jan_", "Feb_", "Mar_", "Apr_", "May_", "Jun_", "Jul_", "Aug_", "Sep_", "Oct_", "Nov_", "Dec_"];
Date.convMonthLong  = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
Date.convWeekShort  = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
Date.convWeekLong   = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];

Date.monthShort = ["", "", "", "", "", "", "", "", "", "", "", ""]; 
Date.monthLong  = ["", "", "", "", "", "", "", "", "", "", "", ""]; 
Date.weekShort  = ["", "", "", "", "", "", ""]; 
Date.weekLong   = ["", "", "", "", "", "", ""];

Date.initTextResources = function() {
  for (var i = 0; i < 12; i++) {
    // Some names are identical in short and long version but we need 
    // unique short month names for translation.
    Date.monthShort[i] = tr(Date.convMonthShort[i]).replace("_", "");
    Date.monthLong[i] = tr(Date.convMonthLong[i]);
  }

  for (var i = 0; i < 7; i++) {
    Date.weekShort[i] = tr(Date.convWeekShort[i]);
    Date.weekLong[i] = tr(Date.convWeekLong[i]);
  }
}

/* Sat, 29 Jul 2006 19:45:21 GMT */
Date.dateToMonthDay = function(dateString) {
  var date = new Date(dateString); 
  return date.toString(tr("%m/%d %H:%M"));
}

Date.dateToMonthDayYear = function(dateString) {
  var date = new Date(dateString);
  var d  = date.getDate();
  if (!d) return "";

  var month = Date.monthLong[date.getMonth()];
  var wday = Date.weekShort[(date.getDay() + 6) % 7];
  var retVal;
  switch (lang) {
    case "ru":
      retVal = wday + ", " + d + " " + month + " " + date.getFullYear() + " г.";
      break;
    case "de":
      retVal = wday + ", " + d + ". " + month + " " + date.getFullYear();
      break;
    default:
      retVal = wday + ", " + month + " " + d + ", " + date.getFullYear();
      break;
  }
  return retVal;
}

/* 2006-07-29T19:45:21Z */
Date.gdateToDate = function(dateString) {
  var re = /(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+)Z/;
  var a = re.exec(dateString);
  if (!a || (a.length != 7)) return new Date(0);
  a.shift();
  
  var date = new Date();
  date.setYear(parseInt(a[0], 10));
  date.setMonth(parseInt(a[1], 10) - 1);
  date.setDate(parseInt(a[2], 10));
  date.setHours(parseInt(a[3], 10));
  date.setMinutes(parseInt(a[4], 10));
  date.setSeconds(parseInt(a[5], 10));
  date.setMilliseconds(0);
  return date;
}

/* 2006-07-29T19:45:21Z */
Date.gdateToMonthDay = function(dateString) {
  return Date.gdateToDate(dateString).toString(tr("%m/%d %H:%M"));
}

/* 2006-07-29T17:45:21+02:00 */
Date.atomDateToDate = function(dateString) {
  var re = /(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+)\+(\d+):(\d+)/;
  var a = re.exec(dateString);
  if (!a || (a.length != 9)) return "";
  a.shift();

  var date = new Date();
  date.setYear(parseInt(a[0], 10));
  date.setMonth(parseInt(a[1], 10) - 1);
  date.setDate(parseInt(a[2], 10));
  date.setHours(parseInt(a[3], 10));
  date.setMinutes(parseInt(a[4], 10));
  date.setSeconds(parseInt(a[5], 10));
  date.setMilliseconds(0);
  return date;
}

/* 2006-07-29T17:45:21+02:00 */
Date.atomDateToMonthDay = function(dateString) {
  return Date.atomDateToDate(dateString).toString(tr("%m/%d %H:%M"));
}

Date.prototype.getYMD = function() {
  return new Date(this.getFullYear(), this.getMonth(), this.getDate());
}

Date.prototype.getDateToday = function() {
  var here = new Date();
  return (this.getMonth() == here.getMonth() && this.getFullYear() == here.getFullYear()) ? here.getDate() : 0;
}

Date.prototype.isToday = function() {
  var now = new Date();
  return (this.getDate() == now.getDate() && this.getMonth() == now.getMonth() && this.getFullYear() == now.getFullYear());
}

/* First january is zero. */
Date.prototype.getYearDay = function() {
  var here = new Date(this.getTime());
  here.setMonth(0, 1);
  return Math.round((this - here) / (60 * 60 * 24 * 1000));
}

Date.prototype.getDaysInMonth = function() {
  var here = new Date(this.getTime());
  here.setDate(32);
  return 32 - here.getDate();
}

Date.prototype.getWeek = function () {
  var gregorian = true;
  if (arguments.length > 0) {
    gregorian = arguments[0];
  }
  /* By Tommy Skaue, http://www.thecodeproject.com/csharp/gregorianwknum.asp
  
     This short article reveals the javascript-way to get week-number based on 
     a certain date. Week-number normally spans from week 1 (first week of a 
     year) to week 52, but in the gregorian calendar some years have 53 weeks. 
     I have myself looked around for a javascript to find weeknumber and use 
     the gregorian calendar rules. But I could not find any. Giving up the 
     search I started the work making the function by myself. I looked up the 
     rules of the calendar, found a neat formel by Peter-Paul Koch, 
     implemented it in javascript, and here it is for you all to use free of 
     charge.
  */
  //lets calc weeknumber the cruel and hard way :D
  //Find JulianDay 
  var month = this.getMonth() + 1; //use 1-12
  var a = Math.floor((14-(month))/12);
  var y = this.getFullYear()+4800-a;
  var m = (month)+(12*a)-3;
  if (gregorian) { // (gregorian calendar)
    var jd = this.getDate() + Math.floor(((153*m)+2)/5) + 
             (365*y) + Math.floor(y/4) - Math.floor(y/100) + 
             Math.floor(y/400) - 32045;      
  } else {         // (julian calendar)
    var jd = (this.getDate()+1)+Math.round(((153*m)+2)/5)+(365+y) + 
             Math.round(y/4)-32083;    
  }
  
  //now calc weeknumber according to JD
  var d4 = (jd+31741-(jd%7))%146097%36524%1461;
  var L = Math.floor(d4/1460);
  var d1 = ((d4-L)%365)+L;
  return Math.floor(d1/7) + 1;
}

/* Most formats of C library routine strftime are supported. */
Date.prototype.toString = function(format) {
  var c;
  var result = "";
  format = format || tr("%b %d %H:%M");
  
  for (i = 0; i < format.length; i++) {
    c = format.charAt(i);
    if (c == '%') {
      c = format.charAt(++i);
      switch (c) {
      case '%':
        result += '%';
        break;
      case 'a': /* short name of weekday: 'Mon' */
        result += Date.weekShort[this.getDay()];
        break;
      case 'A': /* long name of weekday: 'Monday' */
        result += Date.weekLong[this.getDay()];
        break;
      case 'b': /* short name of month: 'Jan' */
        result += Date.monthShort[this.getMonth()];
        break;
      case 'B': /* long name of the month: 'January' */
        result += Date.monthLong[this.getMonth()];
        break;
      case 'd': /* numeric day of the month, leading zeros: '01' */
        result += String(this.getDate()).pad(2, "0");
        break;
      case 'e': /* numeric day of the month, no leading zeros: '1' */
        result += this.getDate();
        break;
      case 'H': /* hour, 24 hour clock, leading zeros: '01' */
        result += String(this.getHours()).pad(2, "0");
        break;
      case 'I': /* hour, 12 hour clock, leading zeros: '01' */
        result += String((this.getHours() + 11) % 12 + 1).pad(2, "0");;
        break;
      case 'j': /* day of the year: '210' */
        result += this.getYearDay() + 1;
        break;
      case 'k': /* hour, 24 hour clock, no leading zeros: '1' */
        result += this.getHours();
        break;
      case 'l': /* hour, 12 hour clock, no leading zeros: '1' */
        result += String((this.getHours() + 11) % 12 + 1);
        break;
      case 'L': /* month number, no leading zeros: '1' */
        result += (this.getMonth() + 1);
        break;
      case 'm': /* month number, leading zeros: '01' */
        result += String(this.getMonth() + 1).pad(2, "0");
        break;
      case 'M': /* minutes, leading zeros: '00' */
        result += String(this.getMinutes()).pad(2, "0");
        break;
      case 'n': /* newline */
        result += '\n';
        break;
      case 'p': /* AM or PM */
        result += this.getHours() < 12 ? 'AM' : 'PM';
        break;
      case 'P': /* am or pm */
        result += this.getHours() < 12 ? 'am' : 'pm';
        break;
      case 'S': /* seconds, leading zeros: '00' */
        result += String(this.getSeconds()).pad(2, "0");
        break;
      case 't': /* tab */
        result += '\t';
        break;
      case 'y': /* year, 2 digits: '06' */
        result += String(this.getFullYear() % 100).pad(2, "0");
        break;
      case 'Y': /* year, 4 digits: '2006' */
        result += this.getFullYear();
        break;
      case 'z': /*  timezone in numeric format: '+0100' */
        var t = this.getTimezoneOffset();
        result += (t < 0 ? '-' : '+') 
                + String(Math.floor(t / 60)).pad(2, "0")
                + String(t % 60).pad(2, "0");
        break;
      }
    } else {
      result += c;
    }
  }
  return result;
}

Date.prototype.getCalendar = function() {
  var here = new Date(this.getTime());
  here.setDate(1);
  var fday = here.getDay();
  if (!fday) {
          fday = 7;
  }

  var gdim = this.getDaysInMonth();
  var lday = gdim + fday;
  var wcnt = Math.ceil((gdim + fday) / 7);

  var weeks = new Array(wcnt);
  for (var i = 0; i < wcnt; i++) {
          weeks[i] = new Array(8);
  }
  for (var i = fday; i < lday; i++) {
          weeks[Math.floor(i / 7)][i % 7] = i - fday + 1;
  }
  for (var i = 0; i < wcnt - 1; i++) {
          weeks[i][7] = weeks[i + 1][0];
  }
  return weeks;
}

Date.prototype.getCalendar2 = function(weekstart) {
  var here = new Date(this.getTime());
  here.setDate(1);
  var fday = (here.getDay() + 7 - weekstart) % 7;
  var gdim = this.getDaysInMonth();
  var lday = gdim + fday;
  var wcnt = Math.ceil(lday / 7);

  var weeks = new Array(wcnt);
  for (var i = 0; i < wcnt; i++) {
          weeks[i] = new Array(7);
  }
  for (var i = fday; i < lday; i++) {
          weeks[Math.floor(i / 7)][i % 7] = i - fday + 1;
  }
  return weeks;
}

Date.prototype.print = function(tr) {
  var weekstart = 1;
  if (arguments.length > 1) {
    weekstart = arguments[1];
  }
  var dbl = false;
  if (arguments.length > 2) {
    dbl = arguments[2];
  }
  var w = Date.weekShort;
  var padding = function (x, y) { return !x ? '     ' : (x == y ? '[' : ' ') + (x < 10 ? ' ' : '') + x +  (x == y ? '] ' : '  '); }

  var cal = this.getCalendar();
  var today = this.getDateToday();
  var nl = '\n' + (dbl?'\n':'');
  var result = '';
  if (tr) {
    for (var j = 0; j < 7; j++) {
      result += '  ' + w[(j + weekstart) % 7];
      for (var i = 0; i < cal.length; i++) { result += padding(cal[i][j + weekstart], today); }
      result += nl
    }
  } else {
    if (w[0].substr(2, 1) == '.') {
      for (var j = 0; j < 7; j++) { result += ' ' + w[(j + weekstart) % 7] + ' '; }
    } else {
      for (var j = 0; j < 7; j++) { result += w[(j + weekstart) % 7] + '  '; }
    }
    result += nl;
    for (var i = 0; i < cal.length; i++) {
      for (var j = 0; j < 7; j++) { result += padding(cal[i][j + weekstart], today); }
      result += nl;
    }
  }
  return result;
}

Date.initTextResources();
