
/**
 *  dynWindow kit for Yahoo! Widget Engine
 *  Copyright (C) 2006 Andy Chentsov <chentsov@gmail.com>
 *
 *  Create Window from xml
 *
 *  version 1.1
 *
 */

String.prototype.isBoolean = function() {
  var s = this.toLowerCase();
  return ((s == "true") || (s == "false"));
}

String.prototype.isNumber = function() {
  var s = this.trim();
  return /^([\d\+\-\.\,]*)$/.test(s);
}

String.prototype.convertType = function() {
  if (this.isBoolean()) {
    return (this.toLowerCase() == "true");
  }
  if (this.isNumber()) {
    return Number(this);
  }
  var translate = (arguments.length == 1) && arguments[0];
  return translate ? tr(this) : String(this);
}

if ("".trim == null) {
  String.prototype.trim = function() {
    return this.replace(/(^\s*)|(\s*$)/g, "");
  }
}

if ("".extractFilePath == null) {
  String.prototype.extractFilePath = function() {
    url = this;
    i = url.length
    while (i >= 0 && (url.substring(i, i + 1) != "\\")) {
      i--;
    }
    if (i > 0) {
      return url.substring(0, i);
    } else return "";
  }
}

if (Array.prototype.indexOf == null) {
  Array.prototype.indexOf = function(element) { 
    for (var i = 0; i < this.length; i ++) 
      if (this[i] == element) return i; 
    return -1; 
  }
}

Object.prototype.initializedPrototypes = [];

Object.prototype.inherits = function (superClass) { 
  this.prototype = new superClass; 
  this.prototype.constructor = this; 
  if (Object.initializedPrototypes.indexOf(this.constructor.name) == -1) {
    // this is the first time the object calls 'inherits', I must therefore add the superClass to the prototype. 
    // if I understood things correctly, this should add a "pointer" to superClass for all instances of this object. 
    this.prototype.superClass = new superClass; 
    Object.initializedPrototypes.push(this.constructor.name);
  } 
  // the line below proves me wrong ! this.prototype.superClass is undefined (when called for variable c2) 
  //else print(this.constructor.name + ' should already have a superClass: ' + this.prototype.superClass);
  superClass.call(this);
  return this; 
} 

Object.prototype.isInstanceOf = function(classOrSuperClass) { 
  var ptr = this; 
  do { 
    if (ptr.constructor == classOrSuperClass) return true; 
    ptr = ptr.prototype.superClass; 
  } while (ptr != null);
  return false; 
}

createFrame = function(parentElement, parent, node) {
  var fr = new dynFrame();
  fr.init(parentElement, parent, node);
  return fr;
}

createText = function(parentElement, parent, node) {
  var txt = new dynText();
  txt.init(parentElement, parent, node);
  return txt;
}

createTextArea = function(parentElement, parent, node) {
  var txt = new dynTextArea();
  txt.init(parentElement, parent, node);
  return txt;
}

createImage = function(parentElement, parent, node) {
  var img = new dynImage();
  img.init(parentElement, parent, node);
  return img;
}

createScrollbar = function(parentElement, parent, node) {
  var sb = new dynScrollbar();
  sb.init(parentElement, parent, node);
  return sb;
}

var elements = 0;

function dynElement() {
  this.container = false;

  this.handleScript = function(script) {
    var owner = this.owner.name;
    return function() { eval("var owner = " + owner + "; " + script); };
  }

  this.setElementProperty = function(element, property, value) {
    switch(property) {
      case "font":
        element.style.fontFamily = $font(value);
        break;
      case "size":
        element.style.fontSize = value + "px";
        break;
      case "style":
        element.style.fontStyle = value.test("italic")?"italic":"normal";
        element.style.fontWeight = value.test("bold")?"bold":"normal";
        break;
      case "color":
        element.style.color = value;
        break;
      default:
        element[property] = value;
    }
  },

  this.init = function(parentElement, window, node) {
    if (this.container) {
      this.items = [];
    }
    this.visible = true;
    this.parentElement = parentElement;
    this.window = window;
    this.node = node;
    this.element = this.createElement();
    this.parent = this.window?this.window:this.element;
    this.isWindow = (this.constructor.name == "dynWindow");
    if (this.isWindow) {
      try {
        this.visible = this["autoVisible"];
        this.element.visible = false;
      } catch(e) {}
    }
    if (this.constructor.name == "dynFrame") { this.parent = this.frame; }
    this.name = String(this.node.evaluate("string(@name)"));
    if (this.name.trim() == "") {
      this.name = String(this.node.evaluate("string(name)"));
    }
    if (this.name.trim() == "") {
      this.name = "element" + elements++;
    }

    this.prefix = "";
    this.owner = this.getOwner();
    if (this.owner != null) {
      if (this.owner.name == null) {
        this.prefix = "o" + (new Date().valueOf()) + "_";
      } else {
        this.prefix = this.owner.name + "_";
      }
    }
    this.fieldName = this.prefix + this.name;
    eval(this.fieldName + "=this.element");
    if (this.owner != null) {
      this.owner[this.name] = this.element;
    }

    if ((this.constructor.name != "dynMenuItem") &&
        (this.constructor.name != "dynWindow")) {
      if (this.parent.addSubview) {
        this.parent.addSubview(this.element);
      } else {
        this.element.window = this.window;
      }
    }

    try {

      var nodes = this.node.evaluate("@*");
      for (var i = 0; i < nodes.length; i++) {
        var a = nodes.item(i);
        if (a.name == "name") continue;
        try {
          /*if (a.name.toLowerCase() == "data") {
            // support automatic translation of text elements
            a.value = tr(a.value);
          }*/
          if ((a.name.toLowerCase() == "vscrollbar") ||
              (a.name.toLowerCase() == "hscrollbar")) {
            sbName = a.value.convertType();
            if (this.owner != null) {
              sbName = this.prefix + sbName;
            }
            eval(this.prefix + this.name + "." + a.name + "=" + sbName);
            continue;
          }
          if (this.isWindow && (a.name.toLowerCase() == "visible")) {
            this.visible = a.value.convertType();
            continue;
          }
          if (a.name.substr(0, 2).toLowerCase() == "on") {
            this.element[a.name] = this.handleScript(a.value.convertType());
          } else {
            this.setElementProperty(this.element, a.name, a.value.convertType(true));
          }
        } catch(e) {}
      }

      var nodes = this.node.evaluate("*");
      for (var i = 0; i < nodes.length; i++) {
        var e = nodes.item(i);
        if (e.tagName == "name") continue;

        if (this.container) {
          if (e.tagName == "frame") {
            this.items.push(createFrame(this, this.parent, e));
            continue;
          }
          if (e.tagName == "text") {
            this.items.push(createText(this, this.parent, e));
            continue;
          }
          if (e.tagName == "textarea") {
            this.items.push(createTextArea(this, this.parent, e));
            continue;
          }
          if (e.tagName == "image") {
            this.items.push(createImage(this, this.parent, e));
            continue;
          }
          if (e.tagName == "scrollbar") {
            this.items.push(createScrollbar(this, this.parent, e));
            continue;
          }
        } else {
          if (e.tagName == "shadow") {
            var sh = new dynShadow();
            sh.init(this, this.parent, e);
            this.element.shadow = sh.shadow;
            continue;
          }
        }
        if (e.tagName == "contextMenuItems") {
          var subnodes = e.evaluate("*");
          var menu = [];
          for (var j = 0; j < subnodes.length; j++) {
            var mi = subnodes.item(j);
            var miO = new dynMenuItem();
            miO.init(this, this.parent, mi);
            menu.push(miO.menuitem);
          }
          this.element.contextMenuItems = menu;
          continue;
        }

        try {
          if ((e.tagName.toLowerCase() == "vscrollbar") ||
              (e.tagName.toLowerCase() == "hscrollbar")) {
            eval(this.name + "." + e.tagName + "=" + String(e.evaluate("string(text())")).convertType());
            continue;
          }
          if (this.isWindow && (e.tagName.toLowerCase() == "visible")) {
            this.visible = String(e.evaluate("string(text())")).convertType();
            continue;
          }
          if (e.tagName.substr(0, 2).toLowerCase() == "on") {
            this.element[e.tagName] = this.handleScript(String(e.evaluate("string(text())")).convertType());
          } else {
            this.setElementProperty(this.element, e.tagName, String(e.evaluate("string(text())")).convertType(true));
          }
        } catch(e) {}
      }

    } catch(e) {}
    if (this.isWindow) {
      this.window.visible = this.visible;
    }
  }

  this.getWindow = function() {
    if (this.constructor.name == "dynWindow") {
      return this.window;
    }
    var p = this.parentElement;
    while (p) {
      if (p.constructor.name == "dynWindow") {
        return p.window;
      }
      p = p.parentElement;
    }
  }

  this.getOwner = function() {
    if (this["owner"]) {
      return this["owner"];
    }
    var p = this.parentElement;
    while (p) {
      if (p.constructor.name == "dynWindow") {
        return p["owner"];
      }
      p = p.parentElement;
    }
  }

  this.createElement = function() {
    
  }

  this.clear = function() {
    if (!this.container) return;
    for (var i = 0; i < this.items.length; i++) {
      try {
        this.items[i].destroy();
      } catch(e) {}
      var o = this.items[i];
      delete o;
      o = null;
      this.items[i] = null;
    }
    this.items = [];
  }

  this.destroy = function() {
    if (this.container) this.clear();
    if (this.element) {
      try {
        this.element.visible = false;
      } catch(e) {}
      delete this.element;
      this.element = null;
    }
  }
}

function dynText() {
  this.container = false;
  this.inherits(dynElement);
  this.createElement = function() {
    this.prototype.createElement();
    this.text = new Text(this.getWindow());
    return this.text;
  }
}

function dynTextArea() {
  this.container = false;
  this.inherits(dynElement);
  this.createElement = function() {
    this.prototype.createElement();
    this.text = new TextArea(this.getWindow());
    return this.text;
  }
}

function dynImage() {
  this.container = false;
  this.inherits(dynElement);
  this.createElement = function() {
    this.prototype.createElement();
    this.image = new Image(this.getWindow());
    return this.image;
  }
}

function dynScrollbar() {
  this.container = false;
  this.inherits(dynElement);
  this.createElement = function() {
    this.prototype.createElement();
    this.scrollbar = new ScrollBar(this.getWindow());
    return this.scrollbar;
  }
}

function dynShadow() {
  this.container = false;
  this.inherits(dynElement);
  this.createElement = function() {
    this.prototype.createElement();
    this.shadow = new Shadow(this.getWindow());
    return this.shadow;
  }
}

function dynMenuItem() {
  this.container = false;
  this.inherits(dynElement);
  this.createElement = function() {
    this.prototype.createElement();
    this.menuitem = new MenuItem();
    return this.menuitem;
  }
}

function dynFrame() {
  this.inherits(dynElement);
  this.container = true;

  this.createElement = function() {
    this.prototype.createElement();
    this.frame = new Frame(this.getWindow());
    return this.frame;
  }
}

/**
 * dynWindow constructor
 *
 * @param  string  xml          file name with window definition
 * @param  object  owner        pointer to owner object (optional)
 * @param  boolean autoVisible  auto set window.visible to true (optional)
 * @return dynWindow            instance of dynWindow class
 */

function dynWindow(xml) {
  this.inherits(dynElement);
  this.items = [];
  this.xmlfile = xml;
  this.xmldata = "";
  this.path = "";
  this.parseError = false;
  this.container = true;
  this.owner = ((arguments.length > 1)?arguments[1]:null);
  this.autoVisible = ((arguments.length > 2)?arguments[2]:true);

  this.createElement = function() {
    this.prototype.createElement();
    this.window = new Window();
    return this.window;
  }

  this.load = function() {
    if (this.xmlfile == "") return;
    this.clear();
    this.path = this.xmlfile.extractFilePath() + "\\";
    this.xmldata = filesystem.readFile(this.xmlfile);
    //try {
      this.xml = XMLDOM.parse(this.xmldata);
      var tmp = this.xml.evaluate("window");
      if (tmp.length > 0) {
        this.init(null, null, tmp.item(0));
      }
    /*} catch(e) {
      this.parseError = true;
    }*/
  }

  this.show = function() {
    if (this.element) {
      try {
        this.element.visible = true;
      } catch(e) {}
    }
  }

  this.close = this.destroy;

  this.load();
}
