
// Base Class
include("Scripts/Base/MooTools.js");

include("Scripts/Utils/MiniHTML.js");
include("Scripts/Windows/dynWindow.js");
include("Scripts/Windows/Selector.js");
include("Scripts/Windows/Readme.js");
include("Scripts/Handlers/contextMenus.js");

// Utilities
include("Scripts/Utils/Utils.js");
include("Scripts/Utils/ColorUtils.js");
include("Scripts/Utils/DateExtension.js");
include("Scripts/Utils/SwapUtils.js");

include("Scripts/Monitor/mm.js");

include("Scripts/Network/Network.js");
include("Scripts/Network/NetworkMacOS.js");
include("Scripts/Network/NetworkWin.js");

// Base Dock Classes
include("Scripts/Base/DockBase.js");
include("Scripts/Base/Dock.js");
include("Scripts/Base/Hint.js");
include("Scripts/Base/FloatingHint.js");
include("Scripts/Base/Property.js");

include("Scripts/Skins.js");

// Base Sensor Classes
include("Sensors/BaseTimer.js");
include("Sensors/BaseMeter.js");
include("Sensors/BaseGraph.js");
include("Sensors/BaseSoloGraph.js");
include("Sensors/BaseDualGraph.js");
include("Sensors/BaseMemGraph.js");
include("Sensors/Text.js");
include("Sensors/BaseText.js");
include("Sensors/BaseShortcut.js");

// Sensors
include("Sensors/Spacer.js");
include("Sensors/Date.js");
include("Sensors/Time.js");
include("Sensors/DiskMeter.js");
include("Sensors/Disk.js");
include("Sensors/CPUMeter.js");
include("Sensors/CPUGraph.js");
include("Sensors/CPUGraphDual.js");
include("Sensors/MemMeter.js");
include("Sensors/MemGraph.js");
include("Sensors/SwapMeter.js");
include("Sensors/SwapGraph.js");
include("Sensors/SpeedFan.js");
include("Sensors/Trash.js");
include("Sensors/Uptime.js");
include("Sensors/NetworkGraph.js");
include("Sensors/WiFi.js");
include("Sensors/BatteryLife.js");
include("Sensors/RSSFeed.js");
include("Sensors/Mail.js");
include("Sensors/GMail.js");
include("Sensors/GMailForDomains.js");
include("Sensors/YMail.js");
include("Sensors/StdMail.js");
include("Sensors/InternetRadio.js");
include("Sensors/Image.js");
include("Sensors/Shortcut.js");
include("Sensors/Power.js");
include("Sensors/USB.js");
include("Sensors/TurboFolder.js");
include("Sensors/TurboFolderStartMenu.js");

include("Scripts/Upgrade.js");

// Notification system
include("Scripts/Base/Notify.js");
include("Scripts/Base/NotificationManager.js");

var notification = new NotificationManager();
var oldPos;
var oldMM = -1;
var oldMMPos = -1;
var dock = new Dock(screen.availLeft, dockPane.vOffset, dockPane.width);
var workareaBackupFile = system.widgetDataFolder + "/workarea.bak";

if (system.platform == "macintosh") {
  preferences.speedFanLog.hidden = true;
}

mainWindow.opacity = 0;
if (parseInt(preferences.docked.value) == 1) {
  if (parseInt(preferences.position.value) == 1) {
    mainWindow.vOffset -= mainWindow.height;
  } else {
    mainWindow.vOffset = screen.availHeight + mainWindow.height;
  }
}

function moveWidget() {
  var percent = (animator.milliseconds - this.startTime) / this.duration;
  var ease = animator.ease(this.startValue, this.endValue, percent, animator.kEaseOut);
  var alpha = animator.ease(this.startAlpha, this.endAlpha, percent, animator.kEaseOut);

  mainWindow.vOffset = ease;
  mainWindow.opacity = alpha;

  return !(animator.milliseconds >= (this.startTime + this.duration));
}

function finishRisingWidget() {
  if (parseInt(preferences.docked.value) == 1) {
    if (parseInt(preferences.position.value) == 1) {
      mainWindow.vOffset = screen.availTop;
    } else {
      mainWindow.vOffset = screen.availHeight - mainWindow.height;
    }
  }
  mainWindow.opacity = 255;

  dock.collectSystemInformation();
  dock.load();
  dock.updateSensors();

  initWorkarea();
}

function riseWidget() {
  var loadAtFinish = true;
  if (arguments.length > 0) {
    loadAtFinish = arguments[0];
  }
  if (loadAtFinish) {
    a = new CustomAnimation(30, moveWidget, finishRisingWidget);
  } else {
    a = new CustomAnimation(30, moveWidget);
  }
  a.duration = 300;
  if (parseInt(preferences.position.value) == 1) {
    a.startValue = screen.availTop - background.height;
    a.endValue = screen.availTop;
  } else {
    a.startValue = screen.availHeight;
    a.endValue = screen.availHeight - mainWindow.height;
  }
  a.startAlpha = 0;
  a.endAlpha = 255;
  animator.start(a);
}

function closeWidget() {
  var p = parseInt(preferences.position.value);
  if (arguments.length > 0) {
    p = arguments[0];
  }
  a = new CustomAnimation(30, moveWidget);
  a.duration = 300;
  if (p == 1) {
    a.startValue = screen.availTop;
    a.endValue = screen.availTop - background.height;
  } else {
    a.startValue = screen.availHeight - mainWindow.height;
    a.endValue = screen.availHeight;
  }
  a.startAlpha = 255;
  a.endAlpha = 0;
  animator.start(a);
}

function alignControls() {
  suppressUpdates();
  try {
    mainWindow.height = background.srcHeight - 1;
    dockPane.height = mainWindow.height;
    background.height = mainWindow.height;

    dock.align();
  } finally {
    resumeUpdates();
  }
}

function resizeWidget() {
  var isChanged = false;
  if (mainWindow.width != screen.availWidth) {
    mainWindow.width = screen.availWidth;
    dockPane.width = mainWindow.width;
    background.width = dockPane.width;
    dock.w = dockPane.width;
    isChanged = true;
  }
  if (parseInt(preferences.position.value) == 1) {
    if (mainWindow.vOffset != screen.availTop) {
      mainWindow.vOffset = screen.availTop;
      //isChanged = true;
    }
  } else {
    if (mainWindow.vOffset != (screen.availHeight - mainWindow.height)) {
      mainWindow.vOffset = screen.availHeight - mainWindow.height;
      //isChanged = true;
    }
  }
  if (mainWindow.hOffset != screen.availLeft) {
    mainWindow.hOffset = screen.availLeft;
  }
  if (isChanged) {
    //dock.align();
    alignControls();
  }
}

function setLocked(value) {
  if (dock) {
    dock.locked = value;
    preferences.lockedState.value = dock.locked?1:0;
    savePreferences();
  }
}

function setPopups(value) {
  if (dock) {
    dock.popupsEnabled = value;
    preferences.popupsEnabled.value = dock.popupsEnabled?1:0;
    savePreferences();
  }
}

function dragDropped(x, y, data) {
  //print("Drop (" + x + "x" + y + "): " + data);
  if (data.length > 0) {
    if ((data[0] == "string") && !data[1].match(/^(http|https|ftp):\/\//i)) return;
    var o = dock.itemAt(x - mainWindow.hOffset, y - mainWindow.vOffset);
    if (!o) {
      var newSide = leftSide;
      var w3 = (background.width / 3);
      if (system.event.hOffset < w3) {
        newSide = leftSide;
      } else if ((system.event.hOffset >= w3) && (system.event.hOffset <= (w3 * 2))) {
        newSide = centerSide;
      } else {
        newSide = rightSide;
      }
      // Create RSS Feed sensor if dropped .xml or .rss file
      if (data[1].match(/(\.xml|\.rss)$/i)) {
        var sensor = newSensorEx(newSide, "DockRSSSensor", null, -1, "after", true);
        sensor.setURL(data[1]);        
      } else {
        if (data[1].match(/^[a-z]{1}\:$/i)) {
          var sensor = newSensorEx(newSide, "DockDisk", null, -1, "after", true);
          sensor.setDisk(data[1] + "\\");
        } else if (filesystem.isDirectory(data[1])) {
          var sensor = newSensorEx(newSide, "TurboFolder", null, -1, "after", true);
          sensor.setFolder(data[1]);
        } else if (data[1].trim() != "") {
          var sensor = newSensorEx(newSide, "DockShortcut", null, -1, "after", true);
          sensor.setLink(data[1]);
        }
      }
      dock.save();
    } else {
      var w2 = (background.width / 2);
      if (data[1].match(/^[a-z]{1}\:$/i)) {
        var sensor = newSensorEx(o.getSide(), "DockDisk", o, o.getIndex(), ((x <= w2)?"before":"after"), true);
        sensor.setDisk(data[1] + "\\");
      } else if (filesystem.isDirectory(data[1])) {
        if (o.className != "TurboFolder") {
          var sensor = newSensorEx(o.getSide(), "TurboFolder", o, o.getIndex(), ((x <= w2)?"before":"after"), true);
          sensor.setFolder(data[1]);
        }
      } else if (data[1].trim() != "") {
        var sensor = newSensorEx(o.getSide(), "DockShortcut", o, o.getIndex(), ((x <= w2)?"before":"after"), true);
        sensor.setLink(data[1]);
      }
    }
  }
}

function updateHotKey() {
  hkStr = preferences.shortcutKey.value;
  focusShortcut.modifier = hkStr.substr(0, hkStr.lastIndexOf('+'));
  focusShortcut.key = hkStr.substr(hkStr.lastIndexOf('+') + 1);
}

// Workarea routines

function initWorkarea() {
  if (system.platform == "macintosh") return;
  if (filesystem.itemExists(workareaBackupFile)) {
    if (alert("Informer is not closed properly, restore workarea?", "Yes", "No") == 1) {
      restoreWorkarea();
    }
  }
  if ((parseInt(preferences.docked.value) == 1) &&
      (parseInt(preferences.clipWorkarea.value) == 1)) {
    setWorkarea();
  }
}

function setWorkarea() {
  if (parseInt(preferences.docked.value) != 1) return;
  if (system.platform == "macintosh") return;
  if (filesystem.itemExists(workareaBackupFile)) filesystem.remove(workareaBackupFile);

  // Make Backup
  filesystem.writeFile(workareaBackupFile, "/l=" + screen.availLeft + " /t=" + screen.availTop + " /r=" + (screen.availLeft + screen.availWidth) + " /b=" + (screen.availTop + screen.availHeight));

  // Set new workarea size
  var data = "";
  if (parseInt(preferences.position.value) == 1) {
    // top
    data = "/t=" + (screen.availTop + 25);
  } else {
    // bottom
    data = "/b=" + (screen.availTop + screen.availHeight - 25);
  }
  if (data != "") {
    runCommandInBg("Applications\\workarea.exe " + data, "workarea");
  }
}

function restoreWorkarea(force) {
  force = (force == null)?false:force;
  if (!force) {
    if (parseInt(preferences.docked.value) != 1) return;
  }
  if (system.platform == "macintosh") return;
  if (!filesystem.itemExists(workareaBackupFile)) return;
  var data = filesystem.readFile(workareaBackupFile);
  runCommandInBg("Applications\\workarea.exe " + data, "workarea");
  if (filesystem.itemExists(workareaBackupFile)) filesystem.remove(workareaBackupFile);
}
