
var DockBaseDualGraph = DockBaseGraph.extend({
  initialize: function() {
    this.parent();
    this.className = "DockBaseDualGraph";

    this.graphTopColorizeProp = new MultiColorProp("graph-top-color", 
                                                   "Top graph color:",
                                                   "#00FF00",
                                                   coActive);
    this.props.push(this.graphTopColorizeProp);
    this.graphTopOpacityProp = new OpacityProp("graph-top-opacity", 
                                               "Top graph:", 140, 50);
    this.props.push(this.graphTopOpacityProp);
    
    this.graphBottomColorizeProp = new MultiColorProp("graph-bottom-color", 
                                                      "Bottom graph color:",
                                                      "#00FF00",
                                                      coActive);
    this.props.push(this.graphBottomColorizeProp);
    this.graphBottomOpacityProp = new OpacityProp("graph-bottom-opacity", 
                                                  "Bottom graph:", 140, 50);
    this.props.push(this.graphBottomOpacityProp);
    this.graphBottomFlipProp = new BoolProp("graph-bottom-flip", 
                                            "Flip bottom graph", true);
    this.props.push(this.graphBottomFlipProp);
    
    this.graphTop = [];
    this.graphBottom = [];
    this.index = 0;
  },

  clear: function() {
    for (var i = 0; i < this.graphTop.length; i++) {
      this.graphTop[i].height = 0;
      this.graphBottom[i].height = 0;
    }
  },

  flipGraph: function() {
    if (this.graphBottomFlipProp.value) {
      this.graphTopOffset = 12;
      this.graphBottomOffset = 13;
      this.graphBottomAlign = "top";
    } else {
      this.graphTopOffset = 13;
      this.graphBottomOffset = 23;
      this.graphBottomAlign = "bottom";
    }
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    if (!this.graphTop) return;

    this.flipGraph(); 
    for (var i = 0; i < this.graphTop.length; i++) {
      var value = this.graphTop[i].height * 0.1;
      this.graphTop[i].colorize = this.graphTopColorizeProp.colorize(value);
      this.graphTop[i].opacity = this.graphTopOpacityProp.value;
      this.graphTop[i].vOffset = this.graphTopOffset;
      
      var value = this.graphBottom[i].height * 0.1;
      this.graphBottom[i].colorize = this.graphBottomColorizeProp.colorize(value);
      this.graphBottom[i].opacity = this.graphBottomOpacityProp.value;
      this.graphBottom[i].vOffset = this.graphBottomOffset;
      this.graphBottom[i].vAlign = this.graphBottomAlign;
    }
  },

  createGraphControls: function() {
    this.flipGraph();
    for (var i = 0; i < this.widthProp.value; i++) {
      var g = new Image(mainWindow);
      this.frame.addSubview(g);
      g.src = "Resources/Sensors/pixel.png";
      g.hOffset = i;
      g.vOffset = this.graphTopOffset;
      g.vAlign = "bottom";
      g.width = 1;
      g.height = 0;
      g.opacity = this.graphTopOpacityProp.value;
      g.zOrder = 3;

      this.graphTop.push(g);

      var g = new Image(mainWindow);
      this.frame.addSubview(g);
      g.src = "Resources/Sensors/pixel.png";
      g.hOffset = i;
      g.vOffset = this.graphBottomOffset;
      g.vAlign = this.graphBottomAlign;
      g.width = 1;
      g.height = 0;
      g.opacity = this.graphBottomOpacityProp.value;
      g.zOrder = 3;

      this.graphBottom.push(g);
    }
  },

  updateValue: function() {
    if (this.value) {
      this.value.data = this.getValueText();
    }

    var valueTop = this.getValueTop() * 0.01;
    this.graphTop[this.index].height = Math.round(valueTop * 10);
    this.graphTop[this.index].colorize = this.graphTopColorizeProp.colorize(valueTop);

    var valueBottom = this.getValueBottom() * 0.01;
    this.graphBottom[this.index].height = Math.round(valueBottom * 10);
    this.graphBottom[this.index].colorize = this.graphBottomColorizeProp.colorize(valueBottom);

    var last = this.widthProp.value - 1;
    if (--this.index < 0) this.index = last;

    var x = this.index;
    for (i = 0; i <= last; i++) {
      this.graphTop[i].hOffset = x;
      this.graphBottom[i].hOffset = x;
      if (--x < 0) x = last;
    }
  },

  getValue: function() {
    return 0;
  },

  getValueTop: function() {
    return 0;
  },

  getValueBottom: function() {
    return 0;
  }

});
