
/* ------------------------------------------------------------------------ */

var WidthProp = BaseProp.extend({
  initialize: function() {
    this.parent("width", 50);
    this.className = "UnitProp";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Width:");
    f.type = "slider";
    f.minLength = 50;
    f.maxLength = 150;
    f.ticks = 20;
    f.tickLabel = ["50" + tr(" pix."), 
                   "100" + tr(" pix."),
                   "150" + tr(" pix.")];
    f.defaultValue = 50;
    f.value = this.value;
    ff.push(f);
  },
  
  set: function(value) {
    this.value = !isNaN(value) && (value >= 50) ? parseInt(value) : 50;
  }
});

/* ------------------------------------------------------------------------ */

var DockBaseGraph = DockBaseTimer.extend({
  initialize: function() {
    this.parent();
    this.className = "DockBaseGraph";

    this.bk = null;
    this.grid = null;
    this.label = null;
    this.value = null;

    this.widthProp = new WidthProp();
    this.bkColorizeProp = new ColorizeProp("bk-color", "Background color:");
    this.bkOpacityProp = new OpacityProp("bk-opacity", "Background:", 0);
    this.gridColorizeProp = new ColorizeProp("grid-color", "Grid color:");
    this.gridOpacityProp = new OpacityProp("grid-opacity", "Grid:", 0);
    
    this.props = [this.intervalProp, this.widthProp, 
                  this.bkColorizeProp, this.bkOpacityProp,
                  this.gridColorizeProp, this.gridOpacityProp];
  },

  destroyControls: function() {
    if (this.parent()) {
      this.grid = null;
      this.bk = null;
      this.label = null;
      this.value = null;

      this.clear();
    }
  },

  clear: function() {
    
  },

  clearHistory: function() {

  },

  updateProps: function(loaded) {
    this.parent(loaded);
    this.setWidth();
    this.setBkColorize();
    this.setBkOpacity();
    this.setGridColorize();
    this.setGridOpacity();
  },

  minInterval: function() {
    return 1.0;
  },

  getName: function() {
    return "Base Graph";
  },

  graphName: function() {
    return "Base";
  },

  updatePrefs: function() {
    this.parent();
    if (this.label) { $$font(this.label, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.value) { $$font(this.value, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    this.updateValue();
    dockObj.align();
  },

  updateColor: function() {
    this.parent();
    if (this.label) { this.label.style.color = this.color; }
    if (this.value) { this.value.style.color = this.color; }
  },

  createGraphControls: function() {

  },

  createControls: function() {
    if (this.parent()) {
      this.bk = new Image(mainWindow);
      this.frame.addSubview(this.bk);
      this.bk.src = "Resources/Sensors/Fill.png";
      this.bk.hOffset = 0;
      this.bk.vOffset = 2;
      this.bk.height = 21;
      this.bk.width = this.widthProp.value;
      this.bk.colorize = this.bkColorizeProp.isActive
                       ? this.bkColorizeProp.value : "#000000";
      this.bk.opacity = this.bkOpacityProp.value;
      this.bk.zOrder = 1;

      this.grid = new Image(mainWindow);
      this.frame.addSubview(this.grid);
      this.grid.src = "Resources/Sensors/GraphGrid.png";
      this.grid.fillMode = "tile";
      this.grid.hOffset = 0;
      this.grid.vOffset = 2;
      this.grid.height = 21;
      this.grid.width = this.widthProp.value + 1; // +1 for right border
      this.grid.colorize = this.gridColorizeProp.isActive
                         ? this.gridColorizeProp.value : "#A0FF9B";
      this.grid.opacity = this.gridOpacityProp.value;
      this.grid.zOrder = 2;

      this.createGraphControls();

      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.hOffset = this.bk.hOffset + this.bk.width + 5;
      this.label.vOffset = 11;
      this.label.data = this.graphName();
      this.label.style.fontFamily = "Arial, Helvetica";
      this.label.style.fontSize = "9px";
      this.label.style.fontWeight = "bold";
      this.label.style.color = this.color;

      this.value = new Text(mainWindow);
      this.frame.addSubview(this.value);
      this.value.hOffset = this.bk.hOffset + this.bk.width + 29;
      this.value.vOffset = 21;
      this.value.data = "0%";
      this.value.style.fontFamily = "Arial, Helvetica";
      this.value.style.fontSize = "9px";
      this.value.style.fontWeight = "bold";
      this.value.style.color = this.color;
      this.value.hAlign = "right";
    }
    return (this.frame != null);
  },

  updateValue: function() {
    if (this.value) {
      this.value.data = this.getValueText();
    }
  },

  getValue: function() {
    return 0;
  },

  getValueText: function() {
    return this.getValue() + "%";
  },

  setWidth: function() {
    // adapt it to the width of the grid image
    this.widthProp.value -= this.widthProp.value % 5;
    if (!this.bk || (this.widthProp.value == this.bk.width)) return;

    this.clearHistory();
    if (this.timer) this.timer.ticking = false;
    this.recreateControls();
  },

  setBkColorize: function() {
    if (this.bk) {
      this.bk.colorize = this.bkColorizeProp.isActive 
                       ? this.bkColorizeProp.value : "#000000";
    }
  },

  setBkOpacity: function(value) {
    if (this.bk) {
      this.bk.opacity = this.bkOpacityProp.value;
    }
  },

  setGridColorize: function(value) {
    if (this.grid) {
      this.grid.colorize = this.gridColorizeProp.isActive 
                         ? this.gridColorizeProp.value : "#A0FF9B";
    }
  },

  setGridOpacity: function(value) {
    if (this.grid) {
      this.grid.opacity = this.gridOpacityProp.value;
    }
  },

  getTitle: function() {
    return "Base graph";
  },

  getHint: function() {
    return "";
  }
});
