
var FilePropEx = FileProp.extend({
  initialize: function(name, title, description, extension) {
    this.parent(name, title, description, extension);
    this.className = "FilePropEx";
  },
  
  set: function(value) {
    this.value = value ? value : "";
  }
});

var DockBaseShortcut = DockBase.extend({
  initialize: function() {
    this.parent();
    this.className = "DockBaseShortcut";
    this.image = null;
    
    this.titleProp = new TextProp("title", "Title:", "",
                                  "Leave blank for automatic detection.");
    this.imageProp = new FileProp("image", "Image:", 
                                  "Image to display, maximum height 24 pix.\nLeave blank to application icon.",
                                  [".ico", ".png", ".gif", ".jpg", ".jpeg"]);
    this.colorizeProp = new ColorizeProp("colorize", "Color:");
    this.opacityProp = new OpacityProp("opacity", "Opacity:", 255, 50);
    
    this.props = [this.titleProp, this.imageProp, this.colorizeProp, 
                  this.opacityProp];
  },

  destroy: function() {
    if (this.parent()) {
      this.image = null;
    }
  },

  updateProps: function(loaded) {
    this.setImage();
    if (this.image) {
      this.image.opacity = this.opacityProp.value;
      this.image.colorize = this.colorizeProp.colorize();
    }
  },

  getName: function() {
    return tr("Shortcut");
  },

  createControls: function() {
    if (this.parent()) {
      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.missingSrc = "Resources/Sensors/App.png";
      this.image.hOffset = 0;
      this.image.tracking = "rectangle";
      this.updateProps();
    }
    return (this.frame != null);
  },

  getTarget: function() {

  },

  doSetImage: function() {

  },
  
  setImage: function(value) {
    if (this.image) {
      this.doSetImage(value);
      this.image.height = this.image.useFileIcon 
                        ? 24 : Math.min(this.image.srcHeight, 24);
      this.image.width = this.image.height;
      this.image.vOffset = Math.round((25 - this.image.height) / 2);
    }
  },

  doGetTitle: function() {

  },

  getTitle: function() {
    if (this.titleProp.value != "") {
      return this.titleProp.value;
    }
    return this.doGetTitle();
  },

  getHint: function() {
    var t = this.getTitle();
    var s = t;
    if ((this.titleProp.value == "") && (t != "")) {
      if (dockObj.locked) {
        s = "%s";
      } else {
        s = "Open '%s'";
      }
      if (this.getTarget().match(/\.widget$/i)) {
        s += " widget";
      }
      s = tr(s).replace(/%s/, t);
    }
    return s;
  }

});
