
var bcmNoBattery = 0;
var bcmACPower   = 1;
var bcmDraining  = 2;
var bcmCharging  = 3;

/* ------------------------------------------------------------------------ */

var BatteryProp = ListProp.extend({
  initialize: function() {
    var o = [];
    var ov = [];
    try {
      for (var i = 0; i < system.batteryCount; i++) {
        o.push((i + 1) + ". " + system.battery[i].name);
        ov.push(i);
      }
    } catch(e) {}

    if (o.length <= 0) {
      o.push(tr("None"));
    }
    if (ov.length <= 0) {
      ov.push(-1);
    }
    this.parent("battery", "Battery:", 0, o, ov);
    this.className = "BatteryProp";
  }
});

/* ------------------------------------------------------------------------ */

var BatteryDisplayProp = BoolProp.extend({
  initialize: function(name, title) {
    this.parent(name, title, true, 
              "Hint: Click on the sensor to switch between both settings.");
    this.className = "BatteryDisplayProp";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "popup";
    f.option = [tr("Remaining Time [h:mm]"), tr("Battery Charge [%]")];
    f.optionValue = [0, 1];
    f.description = this.description;
    f.value = this.value ? 1 : 0;
    f.defaultValue = 1;
    ff.push(f);
  }
});

/* ------------------------------------------------------------------------ */

var BatteryTimeProp = BoolProp.extend({
  initialize: function() {
    this.parent("calculate-time-to-full", "Remaining time display:", false, 
              "When displaying the remaining time until fully charged.");
    this.className = "BatteryTimeProp";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "popup";
    f.option = [tr("display a calculated time"), 
                tr("display what the OS reports")];
    f.optionValue = [1, 0];
    f.description = this.description;
    f.value = this.value ? 1 : 0;
    f.defaultValue = 1;
    ff.push(f);
  }
});

/* ------------------------------------------------------------------------ */

var BatteryChargedProp = ListProp.extend({
  initialize: function() {
    var o = [tr("when the OS reports that it is"),
                tr("when the battery charge reaches 95%"), 
                tr("when the battery charge reaches 96%"), 
                tr("when the battery charge reaches 97%"), 
                tr("when the battery charge reaches 98%"), 
                tr("when the battery charge reaches 99%"), 
                tr("when the battery charge reaches 100%")];
    var ov = [-1, 95, 96, 97, 98, 99, 100];
    this.parent("assume-charged", "Assume battery is charged:", -1, o, ov);
    this.className = "BatteryChargedProp";
  }
});

/* ------------------------------------------------------------------------ */

var BatteryAlertProp = ListProp.extend({
  initialize: function() {
    var o = [tr("never"),
                tr("when the battery charge is over 90%"), 
                tr("when the battery charge is over 91%"), 
                tr("when the battery charge is over 92%"), 
                tr("when the battery charge is over 93%"), 
                tr("when the battery charge is over 94%"), 
                tr("when the battery charge is over 95%")];
    var ov = [-1, 90, 91, 92, 93, 94, 95];
    this.parent("alert-level", "Display an alert:", -1, o, ov);
    this.className = "BatteryAlertProp";
  }
});

/* ------------------------------------------------------------------------ */

var DockBatteryLife = DockBaseTimer.extend({
  initialize: function(disk) {
    this.parent();
    this.className = "DockBatteryLife";
    
    this.emptyTime = -1;
    this.batteryTime = -1;
    this.beginChargeRate = 0;
    this.beginTime = -1;
    this.timeToFull = -1;
    this.hintText = "";
    this.alertShown = false;
    
    this.batteryProp = new BatteryProp();
    this.whenInUseShowPercentProp = new BatteryDisplayProp("when-in-use-display-percent", 
                                                           "When battery is in use show:");
    this.smoothTimeToEmptyProp = new BoolProp("smooth-time-to-empty", 
                                              "Smooth out remaining time",
                                              false, "");
    this.whenChargingShowPercentProp = new BatteryDisplayProp("when-charging-display-percent", 
                                                              "When battery is charging show:");
    this.calculateTimeToFullProp = new BatteryTimeProp();
    this.assumeChargedProp = new BatteryChargedProp();
    this.alertLevelProp = new BatteryAlertProp();
    
    this.props = [this.batteryProp, this.whenInUseShowPercentProp,
                  this.smoothTimeToEmptyProp, this.whenChargingShowPercentProp,
                  this.calculateTimeToFullProp, this.assumeChargedProp,
                  this.alertLevelProp];
    
  },

  destroy: function() {
    if (this.parent()) {
      this.icon = null;
      this.label = null;
      this.back = null;
      this.state = null;
      this.top = null;
      this.front = null;
      this.value = null;
    }
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    if (!loaded) {
      this.updateValue();
    }
  },

  getName: function() {
    return tr("Battery Life");
  },

  updatePrefs: function() {
    this.parent();
    if (this.label) { $$font(this.label, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.value) { $$font(this.value, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    dockObj.align();
  },

  updateColor: function() {
    this.parent();
    if (this.label) { this.label.style.color = this.color; }
    if (this.value) { this.value.style.color = this.color; }
  },

  getTitle: function() {
    if (this.batteryProp.value == -1) {
      return tr("No battery");
    }
    try {
      return system.battery[this.batteryProp.value].name;
    } catch(e) {
      return tr("Battery ") + (this.batteryProp.value + 1);
    }
  },

  createControls: function() {
    if (this.parent()) {
      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.hOffset = 22;
      this.label.vOffset = 11;
      this.label.data = this.getTitle();
      this.label.style.fontFamily = "Arial, Helvetica";
      this.label.style.fontSize = "9px";
      this.label.style.fontWeight = "bold";
      this.label.style.color = this.color;

      this.back = new Image(mainWindow);
      this.frame.addSubview(this.back);
      this.back.hOffset = 0;
      this.back.vOffset = 1;
      this.back.width = 19;
      this.back.height = 23;
      this.back.src = "Resources/Sensors/Battery/Battery-Back.png";
      this.back.zOrder = 1;

      this.state = new Image(mainWindow);
      this.frame.addSubview(this.state);
      this.state.hOffset = 0;
      this.state.vOffset = 1;
      this.state.width = 19;
      this.state.height = 23;
      this.state.src = "Resources/Sensors/Battery/Level.png";
      this.state.clipRect = "0,21,19,21";
      this.state.zOrder = 2;
      this.state.visible = false;

      this.top = new Image(mainWindow);
      this.frame.addSubview(this.top);
      this.top.hOffset = 0;
      this.top.vOffset = 1;
      this.top.width = 19;
      this.top.height = 4;
      this.top.src = "Resources/Sensors/Battery/Level-Top.png";
      this.top.zOrder = 3;
      this.top.visible = false;

      this.front = new Image(mainWindow);
      this.frame.addSubview(this.front);
      this.front.hOffset = 0;
      this.front.vOffset = 1;
      this.front.width = 19;
      this.front.height = 23;
      this.front.src = "Resources/Sensors/Battery/Battery-Front.png";
      this.front.zOrder = 4;
      this.front.visible = true;

      this.icon = new Image(mainWindow);
      this.frame.addSubview(this.icon);
      this.icon.src = "Resources/Sensors/Battery/No-Battery.png";
      this.icon.hOffset = Math.round((19 - this.icon.srcWidth) / 2);
      this.icon.vOffset = Math.round((25 - this.icon.srcHeight) / 2);
      this.icon.zOrder = 5;
      this.icon.colorize = "#FFFFFF";

      this.value = new Text(mainWindow);
      this.frame.addSubview(this.value);
      this.value.hOffset = 22;
      this.value.vOffset = 21;
      this.value.data = "";
      this.value.style.fontFamily = "Arial, Helvetica";
      this.value.style.fontSize = "9px";
      this.value.style.fontWeight = "bold";
      this.value.style.color = this.color;

      this.updateValue();
    }
    return (this.frame != null);
  },

  convertLevel: function(percent, lowIn, highIn, lowOut, highOut) {
    return (percent - lowIn) / (highIn - lowIn) * (highOut - lowOut) + lowOut;
  },

  setState: function(state, status, percent) {
    var icon = "";
    var label = tr("In Use");

    if (percent < 0) { percent = 0; }
    if (percent > 100) { percent = 100; }
    var adjustBase = 3;
    var level;
    if (percent < 15) {
      level = this.convertLevel(percent, 0, 15, 0, 30);
    } else if (percent < 40) {
      level = this.convertLevel(percent, 15, 40, 30, 70);
    } else {
      level = this.convertLevel(percent, 40, 100, 70, 100);
    }

    switch (state) {
    case bcmNoBattery:
      icon = "No-Battery";
      label = tr("No Battery");
      break;
    case bcmACPower:
      icon = "Charged";
      label = tr("Full");
      break;
    case bcmDraining:
      var level;
      if (percent < 15) {
        label = tr("Low Battery");
      }
      adjustBase = 23 - (Math.round(level * (20/100)));
      break;
    case bcmCharging:
      icon = "Charging";
      label = tr("Charging");
      adjustBase = 23 - (Math.round(level * (20/100)));
      break;
    }

    if (percent < 50) {
      var g = Math.round(percent * 5.10);
      var color = "#FF" + g.toString(16).pad(2, "0") + "00";
    } else {
      var r = Math.round((100 - percent) * 5.10);
      var color = "#" + r.toString(16).pad(2, "0") + "FF00";
    }
    this.top.vOffset = adjustBase - 1;
    this.top.visible = (state != bcmNoBattery) && (percent <= 88);
    this.top.colorize = color;
    this.state.clipRect = "0, " + adjustBase + ", 19, 21";
    this.state.visible = (state != bcmNoBattery) && (percent > 0);
    this.state.colorize = color;

    if (this.value.data != status)  this.value.data = status;
    if (this.label.data != label)   this.label.data = label;
    if (this.icon) {
      if (icon) {
         this.icon.src = "Resources/Sensors/Battery/" + icon + ".png";
         this.icon.hOffset = Math.round((19 - this.icon.srcWidth) / 2);
         this.icon.vOffset = Math.round((25 - this.icon.srcHeight) / 2);
      }
      this.icon.visible = icon != "";
    }
  },

  minutesToHours: function(m) {
    return String(Math.floor(m / 60)) + ":" + String(m % 60).pad(2, "0");
  },

  updateValue: function() {
    if (!this.back) return;
    if (system.batteryCount <= 0) {  // note: this will fail when there is more than one battery installed
      if (this.beginTime > 0) this.beginTime = -1;
      if (this.batteryTime > 0) this.batteryTime = -1;
      this.setState(bcmNoBattery, "", 0);
      return;
    }
      try {
        var maxValue = system.battery[this.batteryProp.value].maximumCapacity;
        var curValue = system.battery[this.batteryProp.value].currentCapacity;
        var value = (curValue / maxValue) * 100;
        var percent = Math.round(value);
        var isCharging = (system.battery[this.batteryProp.value].isCharging && this.assumeChargedProp.value==-1)
                          || percent < this.assumeChargedProp.value;
        status = percent + "%";
        if (system.battery[this.batteryProp.value].powerSourceState == "AC Power") {
        if (this.batteryTime > 0) this.batteryTime = -1;
          if (isCharging) {
            if (this.alertLevelProp.value != -1 && percent>this.alertLevelProp.value && !this.alertShown) {
              this.showAlert(tr("  \n  Alert triggered\n  remove your battery\n  "));
              this.alertShown = true;
            }
            if (this.beginTime <= 0) {  // init
              this.timeToFull = -1;
              this.beginTime = animator.milliseconds;
              this.beginChargeRate = percent;       
            }
            var fullyCharged = this.assumeChargedProp.value==-1?100:this.assumeChargedProp.value;
            // formula: elapsed_time[sec] / percent_delta [%] * percent_tofill [%]
            if (percent>this.beginChargeRate) {
              var minutesLeft = ((animator.milliseconds-this.beginTime)/1000) / (percent - this.beginChargeRate) * (fullyCharged - percent) / 60;
              if (this.timeToFull<=0 || minutesLeft > this.timeToFull) this.timeToFull = minutesLeft;   
              this.timeToFull += (minutesLeft - this.timeToFull) / 30;                  
            } 

            var result = this.calculateTimeToFullProp.value?this.timeToFull:system.battery[this.batteryProp.value].timeToFullCharge;
            
            if (result == "-1") {
              this.hintText = tr("    Capacity is ") + percent + "%\n    " + tr("Calculating Time Until Full");
              if (!this.whenChargingShowPercentProp.value) status = "*";
            } else {
              var timeStr = this.minutesToHours(parseInt(result));
              this.hintText = tr("    Capacity is ") + percent + "%\n    " + timeStr + tr(" Until Full");
              if (!this.whenChargingShowPercentProp.value) status = timeStr;
            }
            this.setState(bcmCharging, status, value);
          } else {                   // if not charging
            this.hintText = tr("    Capacity is ") + percent +"%\n    " + tr("Battery Is Charged");
            if (this.assumeChargedProp.value!=-1) status = "";
            this.setState(bcmACPower, status, value);
          }
        } else {                                                 // battery is in use
          this.alertShown = false;
          if (this.beginTime > 0) this.beginTime = -1;
        if ((this.batteryTime < 0) && (percent == 100)) this.batteryTime = animator.milliseconds;
          if (system.battery[this.batteryProp.value].timeToEmpty != "-1") {
            if (this.emptyTime <= 0 || !this.smoothTimeToEmptyProp.value) {this.emptyTime = system.battery[this.batteryProp.value].timeToEmpty;} // init
            this.emptyTime += (system.battery[this.batteryProp.value].timeToEmpty-this.emptyTime)/10;
          } else {this.emptyTime=-1;}
          if (this.emptyTime == "-1") {
            this.hintText = tr("    Capacity is ") + percent + "%\n    " + tr("Calculating Time Remaining");
            if (!this.whenInUseShowPercentProp.value) status = "*";
          } else {
            var timeStr = this.minutesToHours(parseInt(this.emptyTime));
            this.hintText = tr("    Capacity is ") + percent + "%\n    " + timeStr + tr(" Remaining");
            if (!this.whenInUseShowPercentProp.value) status = "~ " + timeStr;
          }
        if (this.batteryTime > 0) {
          var timeStr = this.minutesToHours(Math.floor((animator.milliseconds - this.batteryTime) / 60000));
          this.hintText += "\n    " + timeStr + tr(" Already In Use"); 
        }
          this.setState(bcmDraining, status, value);
        }
      } catch(e) {}
  },
  
  showAlert: function(text) {
    dockObj.hint.show(this.frame.hOffset + Math.round(this.frame.width / 2)-6, 200, text); // y=200 will be ignored atm
  },

  doClick: function() {
    this.parent();
    try {
      if (system.battery[this.batteryProp.value].powerSourceState == "Battery Power") {
        this.whenInUseShowPercentProp.value = !this.whenInUseShowPercentProp.value;
      } else {
        this.whenChargingShowPercentProp.value = !this.whenChargingShowPercentProp.value;
      }
      dockObj.save();
      this.updateValue();
    } catch (e) {}
  },

  getHint: function() {
    return this.getTitle() +
           ((this.hintText != "")?("\n" + this.hintText):"");
  }
});

dockFactory.register({name: tr("Battery Life"),
                      className: "DockBatteryLife",
                      classDef: DockBatteryLife,
                      category: tr("System"),
                      description: tr("Show battery charge and state")
                      });
