
include("Sensors/CPUHint.js");

var DockCPUGraphDual = DockBaseDualGraph.extend({
  initialize: function() {
    this.parent();
    this.className = "DockCPUGraphDual";

    this.graphTopColorizeProp.reset("#20B7FF");
    this.graphBottomColorizeProp.reset("#FFAD5B");
    this.graphTopOpacityProp.reset(220);
    this.graphBottomOpacityProp.reset(220);

    this.WBEM = null;
    this.wmi = null;
    this.prev = [
      {cpu: 0,
       timestamp: 0},
      {cpu: 0,
       timestamp: 0}
    ];

    if (system.platform == "windows") {
      try {
        if (!this.wmi) {
          this.WBEM = COM.createObject("WbemScripting.SWbemLocator.1");
          this.wmi = this.WBEM.ConnectServer(".", "root/cimv2");
        }
      } catch(e) {}
    } else {

    }

    this.showAlert = true;
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    if (!loaded && this.showAlert && (system.cpu.numProcessors == 1)) {
      alert(tr("You have only one processor."));
      this.showAlert = false;
    }
  },

  getName: function() {
    return tr("CPU Graph (dual)");
  },

  graphName: function() {
    return tr("CPUs");
  },

  getValue: function() {
    return system.cpu.activity;
  },

  getCPUActivity: function(CPUNum) {
    if (this.wmi) {
      var value = this.wmi.Get('Win32_PerfRawData_PerfOS_Processor=' + CPUNum).Properties_.Item("PercentIdleTime").Value;
      //var value = this.wmi.Get('Win32_PerfRawData_PerfOS_Processor=' + CPUNum).Properties_.Item("PercentProcessorTime").Value;
      var time = this.wmi.Get('Win32_PerfRawData_PerfOS_Processor=' + CPUNum).Properties_.Item("TimeStamp_Sys100NS").Value;

      if (time > this.prev[CPUNum].timestamp) {
        var activity = Math.round((1 - ((value - this.prev[CPUNum].cpu) / (time - this.prev[CPUNum].timestamp))) * 100);
        this.prev[CPUNum].cpu = value;
        this.prev[CPUNum].timestamp = time;
        return activity;
      }
    }
    return system.cpu.activity;
  },

  getValueTop: function() {
    if (system.cpu.numProcessors == 1) {
      return system.cpu.activity;
    } else {
      return this.getCPUActivity(0);
    }
  },

  getValueBottom: function() {
    if (system.cpu.numProcessors == 1) {
      return system.cpu.activity;
    } else {
      return this.getCPUActivity(1);
    }
  },

  getTitle: function() {
    return "Dual core CPU usage history";
  },

  getHint: function() {
    return getCPUHint();
  },

  doClick: function() {
    this.parent();
    if (system.platform == "windows") {
      runCommandInBg("%SystemRoot%/system32/taskmgr.exe", "taskManagerRun");
    }
    if (system.platform == "macintosh") {
      filesystem.open("/Applications/Utilities/Activity Monitor.app");
    }
  }
});

if (system.platform == "windows") {
  dockFactory.register({name: tr("CPU Graph (dual)"),
                        className: "DockCPUGraphDual",
                        classDef: DockCPUGraphDual,
                        category: tr("System"),
                        description: tr("Show dual core CPU usage history")
                        });
}
