
var ddmNone   = 0;
var ddmShort  = 1;
var ddmLong   = 2;
var ddmNumber = 3;
var ddmTop    = 4;
var ddmBottom = 5;

var ddcNo     = 0;
var ddcMonday = 1;
var ddcSunday = 2;

/* ------------------------------------------------------------------------ */

var DayNameProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("No"), tr("Short"), tr("Long")]; 
    var optValues = [ddmNone, ddmShort, ddmLong];
    this.parent("dayname", "Display weekday:", 2, optTexts, optValues,
              "Display the weekday name in front of the day.");
    this.className = "DayNameProp";
  }
});

/* ------------------------------------------------------------------------ */

var MonthProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("No"), tr("Number"), tr("Short"), tr("Long")]; 
    var optValues = [ddmNone, ddmNumber, ddmShort, ddmLong];
    this.parent("month-format", "Display month:", ddmLong, optTexts, optValues);
    this.className = "MonthProp";
  }
});

/* ------------------------------------------------------------------------ */

var YearProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("No"), tr("Top"), tr("Bottom")]; 
    var optValues = [ddmNone, ddmTop, ddmBottom];
    this.parent("year-format", "Display year:", ddmTop, optTexts, optValues);
    this.className = "YearProp";
  }
});

/* ------------------------------------------------------------------------ */

var CalendarProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("No"), tr("Yes (start at monday)"), tr("Yes (start at sunday)")];
    var optValues = [ddcNo, ddcMonday, ddcSunday];
    var comment = tr("Show a calendar of the current month in the popup.");
    this.parent("show-calendar", "Show popup calendar:", 1, optTexts, optValues,
              comment);
    this.className = "CalendarProp";
  }
});

/* ------------------------------------------------------------------------ */

include("Sensors/DateHint.js");

// Based on Date sensor by Matthew R. Baerbock, www.BlueWebStudios.com

var DockDate = DockBaseTimer.extend({
  initialize: function() {
    this.parent(1.0);
    this.className = "DockDate";
    this.lastUpdate = 0;
    
    this.dayNameProp = new DayNameProp();
    this.monthProp = new MonthProp();
    this.yearProp = new YearProp();
    this.shadowProp = new BoolProp("shadow", "Display shadow", false, "");
    this.calendarProp = new CalendarProp();

    this.props = [this.dayNameProp, this.monthProp, this.yearProp, 
                  this.shadowProp, this.calendarProp];

    this.hint = null;
  },

  getName: function() {
    return tr("Date");
  },

  updateProps: function(loaded) {
    this.updateVisual();
    if (this.yearProp.value == ddmNone) {
      var format = [["%d", "%b, %d", "%B, %d", "%m/%d"], 
                    ["%a, %d", "%a, %b %d", "%a, %B %d", "%a, %m/%d"],
                    ["%A, %d", "%A, %b %d", "%A, %B %d", "%A, %m/%d"]];
      this.dayFormat = tr(format[this.dayNameProp.value][this.monthProp.value]);
    } else {
      var format = ["%d", "%a, %d", "%A, %d"];
      this.dayFormat = tr(format[this.dayNameProp.value]);

      var format = ["%b", "%b", "%B", "%m"];
      this.monthFormat = tr(format[this.monthProp.value]);
    }
    this.updateValue(true);
  },

  updateColor: function() {
    this.parent();
    if (this.month) { this.month.style.color = this.color; }
    if (this.day) { this.day.style.color = this.color; }
    if (this.year) { this.year.style.color = this.color; }
    this.updateValue(true);
  },

  updatePrefs: function() {
    this.parent();
    if (this.day) { $$font(this.day, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.month) { $$font(this.month, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.year) { $$font(this.year, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    this.updateVisual();
  },

  createControls: function() {
    if (this.parent()) {
      this.day = new Text(mainWindow);
      this.frame.addSubview(this.day);
      this.day.data = "00";
      this.day.vOffset = 18;
      this.day.style.fontFamily = "Arial, Helvetica";
      this.day.style.fontSize = "15px";
      this.day.style.fontWeight = "bold";
      this.day.style.color = "#FFFFFF";

      this.month = new Text(mainWindow);
      this.frame.addSubview(this.month);
      this.month.vOffset = 21;
      this.month.style.fontFamily = "Arial, Helvetica";
      this.month.style.fontSize = "11px";
      this.month.style.color = "#FFFFFF";

      this.year = new Text(mainWindow);
      this.frame.addSubview(this.year);
      this.year.data = "0000";
      this.year.vOffset = 13;
      this.year.style.fontFamily = "Arial, Helvetica";
      this.year.style.fontSize = "11px";
      this.year.style.color = "#FFFFFF";

      this.updateProps();
    }
    return (this.frame != null);
  },

  updateVisual: function() {
    if (!this.day) return;

    var s = null;
    if (this.shadowProp.value) {
      s = new Shadow();
      s.hOffset = 1;
      s.vOffset = 1;
      s.color = Color.isLightColor(Color.HexToRGB(this.color)) ? "#000000" : "#FFFFFF";
    }
    this.day.hOffset = 0;
    this.day.shadow = s;

    if (this.yearProp.value != ddmNone) {
      this.year.hOffset = this.day.width + 3;
      this.year.vOffset = (this.yearProp.value == ddmTop) ? 11 : 22;
      this.year.shadow = s;
      this.year.visible = true;
      
      this.month.hOffset = this.day.width + 3;
      this.month.vOffset = (this.yearProp.value == ddmTop) ? 21 : 10;
      this.month.style.fontSize = "11px";
      this.month.shadow = s;
      this.month.visible = true;
      this.frame.width = this.year.hOffset + 
                         ((this.year.width > this.month.width) 
                            ? this.year.width : this.month.width);
    } else {
      this.year.visible = false;
      this.month.visible = false;
      this.frame.width = this.day.width;
    }
    dockObj.align();
  },

  updateValue: function() {
    if (!this.day) return;
    var force = false;
    if (arguments.length > 0) {
      force = arguments[0];
    }
    var time = new Date();
    if (force || (this.lastUpdate != time.getDate())) {
      this.day.data = time.toString(this.dayFormat);
      if (this.yearProp.value != ddmNone) {
        this.month.data = time.toString(this.monthFormat).toLowerCase();
        this.year.data = time.getFullYear();
      }
      this.updateVisual();
      
      if (this.timer) {
        this.timer.onTimerFired = this.updateValue.bind(this);
      }
      this.lastUpdate = time.getDate();
    }
  },

  createHint: function() {
    this.hint = new DateHint();
  },

  showHint: function(x, y) {
    if (!this.hint) this.createHint();
    if (this.hint) {
      dockObj.hint.hide();
      this.hint.setShowCalendar(this.calendarProp.value != ddcNo,
                                this.calendarProp.value == ddcSunday ? 0 : 1);
      this.hint.show(x, y, this.getHint());
    }
    return false;
  },

  hideHint: function() {
    if (this.hint) {
      this.hint.hide();
    }
    return true;
  },

  getHint: function() {
    var time = new Date();
    return tr(" Week ") + time.getWeek() +
           tr(", Day ") + (time.getYearDay() + 1);
  },

  doClick: function() {
    this.parent();
    var date = new Date();
    dockObj.openCalendar(Number(date));
  }
});

dockFactory.register({name: tr("Date"),
                      className: "DockDate",
                      classDef: DockDate,
                      category: tr("Info"),
                      description: tr("Show current date")
                      });
