
var dmOccupied      = 0;
var dmFree          = 1;
var dmOccupiedBytes = 2;
var dmFreeBytes     = 3;

/* ------------------------------------------------------------------------ */

var DiskModeProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("occupied space [%]"), tr("free space [%]"),
                    tr("occupied space [bytes]"), tr("free space [bytes]")];
    var optValues = [dmOccupied, dmFree, dmOccupiedBytes, dmFreeBytes];
    this.parent("mode", "Show:", dmOccupied, optTexts, optValues);
    this.className = "DiskModeProp";
  }
});

/* ------------------------------------------------------------------------ */

var DockDiskMeter = DockBaseMeter.extend({
  initialize: function() {
    this.parent();
    this.className = "DockDiskMeter";

    this.percent = 0;
    this.total = 0;
    this.free = 0;
    
    this.titleProp = new TextProp("title", tr("Title:"), "",
                                  tr("Sensor's title (leave blank for default)"));
    this.diskProp  = new DiskProp(tr("Drive to monitor free space"));
    this.modeProp  = new DiskModeProp();

    this.props = [this.titleProp, this.diskProp, this.modeProp, 
                  this.intervalProp, this.colorizeProp];
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    this.setDisk();
  },

  getName: function() {
    return tr("Disk Meter for ") + this.diskProp.value;
  },

  meterName: function() {
    return (this.titleProp.value == "")?(((system.platform == "windows")?tr("HD: "):"") + this.diskProp.value):this.titleProp.value;
  },

  getHint: function() {
    return tr("Disk Meter for ") + this.diskProp.value +
      tr("\nTotal Space: ") + trBytesToUI(this.total) +
      tr("\nFree Space: ") + trBytesToUI(this.free) +
      tr("\nDisk Space Used: ") + (100 - this.percent) + "%";
  },

  getValue: function() {
    var vols = filesystem.volumes;
    for (vol in vols) {
      if (vols[vol].path !== this.diskProp.value) continue;
      var p = Math.round((100 * vols[vol].freeBytes) / vols[vol].totalBytes);
      this.percent = p;
      this.total = vols[vol].totalBytes;
      this.free = vols[vol].freeBytes;
      return 100.0 - p;
    }
    return 0;
  },

  updateValue: function() {
    this.parent();
    if (this.value) {
      if ((this.modeProp.value == dmFree) || (this.modeProp.value == dmOccupied)) {
        var v = this.getValue();
        if (this.modeProp.value == dmFree) {
          v = 100.0 - v;
        }
        this.value.hAlign = "right";
        this.value.data = (v + "%");
        this.value.hOffset = 32;
      } else {
        this.value.hAlign = "left";
        this.value.hOffset = 8;
        if (this.modeProp.value == dmFreeBytes) {
          this.value.data = trBytesToUI(this.free);
        } else {
          this.value.data = trBytesToUI(this.total - this.free);
        }
        /*if ((this.value.width + 10) > this.frame.width) {
          this.value.hOffset = this.value.width + 10;
        }*/
      }
      dockObj.align();
    }
  },

  setTitle: function() {
    if (this.label) {
      this.label.data = ((this.titleProp.value == "")?(tr((system.platform == "windows")?"HD: ":"") + this.diskProp.value):this.titleProp.value);
      dockObj.align();
    }
  },

  setDisk: function() {
    this.setTitle();
    this.updateValue();
  },

  doClick: function() {
    this.parent();
    filesystem.open(this.diskProp.value);
  }
});

dockFactory.register({name: tr("Disk Meter"),
                      className: "DockDiskMeter",
                      classDef: DockDiskMeter,
                      category: tr("System"),
                      description: tr("Show used/free hard disk space")
                      });
