
include("Sensors/GMailHint.js");

var DockGMail = DockGenericMail.extend({
  initialize: function() {
    this.parent(tr("GMail"));
    this.className = "DockGMail";
    
    this.autoLoginProp = new BoolProp("autoLogin", "Auto Login", true,
                                      "Check if you would like sensor to automatically login when you click on it.");

    this.usernameProp.description = tr("Please enter username without domain (@gmail.com, @googlemail.com, etc.).");
    this.props = [this.titleProp, this.usernameProp, this.passwordProp, 
                  this.autoLoginProp, this.rowLimitProp, this.widthLimitProp, 
                  this.intervalProp, this.customIconProp, this.notifyProp];
    this.reverseOrderProp.value = true;
  },

  getName: function() {
    return tr("GMail Checker");
  },

  doOpenClient: function() {
    if (this.autoLoginProp.value) {
      this.openURL("https://www.google.com/accounts/ServiceLoginAuth?service=mail&Email=" + this.filterUsername(this.usernameProp.value) + "&Passwd=" + this.passwordProp.value + "&continue=https://mail.google.com/mail?ui=html&amp;zy=l");
    } else {
      this.openURL("http://mail.google.com/mail/");
    }
  },

  createHint: function() {
    this.hint = new GMailHint(this);
    this.hint.setOptions(this.rowLimitProp.value, this.widthLimitProp.value);
  },

  getIcon: function() {
    return "Resources/Sensors/Gmail-Rotated.png";
  },

  getNodes: function(xml, totals) {
    var setTotal = (totals != null)?totals:true;
    var nodes = null;
    try {
      var doc = XMLDOM.parse(xml);
      nodes = doc.evaluate("/feed/entry");
      if (setTotal) {
        this.total = parseInt(String(doc.evaluate("string(/feed/fullcount)")));
      }
    } catch (e) { }
    return nodes;
  },

  /*updateFeed: function(url) {
    url["caller"].finishChecking();
    var data = url.result;
    if (url.response == 200 && data.length) {
      url["caller"].data = data;
      url["caller"].lastUpdate = new Date();
      // save cache
      if (url["caller"].cache != "") {
        filesystem.writeFile(url["caller"].cache, data);
      }
      url["caller"].makeElementInactive(url["caller"].icon, false);
      url["caller"].updateValue();
    } else {
      url["caller"].setError("Connection time-out.");
    }
  },*/

  updateFeed: function(req) {
    if (req.readyState != 4) return;
    this.finishChecking();
    var data = req.responseText;
    if (data.length && !(data.match(new RegExp("\<HTML\>", "ig") ))) {
      this.data = data;
      this.lastUpdate = new Date();
      // save cache
      if (this.cache != "") {
        filesystem.writeFile(this.cache, data);
      }
      this.makeElementInactive(this.icon, false);
      this.updateValue();
    } else {
      if (data.length) {
        var msg = "Unknown error.";
        var re = new RegExp("\<title\>(.*)\<\/title\>", "ig");
        var match = re.exec(data);
        if (match != null) {
          msg = match[1];
        }
        this.setError(msg);
      } else {
        this.setError("Connection time-out.");
      }
    }
  },

  getUrl: function() {
    return "https://mail.google.com/mail/feed/atom/";
  },

  basicAuth: function(username, password) {
    var strMain = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    var strEncode = username + ":" + password;
    var result = "";
    var chr1, chr2, chr3;
    var enc1, enc2, enc3, enc4;
    var i = 0;
    while (i < strEncode.length) {
      chr1 = strEncode.charCodeAt(i++);
      chr2 = strEncode.charCodeAt(i++);
      chr3 = strEncode.charCodeAt(i++);

      enc1 = chr1 >> 2;
      enc2 = ( ( chr1 & 3 ) << 4 ) | ( chr2 >> 4 );
      enc3 = ( ( chr2 & 15 ) << 2 ) | ( chr3 >> 6 );
      enc4 = chr3 & 63;

      if ( isNaN( chr2 ) ) {
        enc3 = enc4 = 64;
      } else if ( isNaN( chr3 ) ) {
        enc4 = 64;
      }
      result += strMain.charAt(enc1) + strMain.charAt(enc2) + 
                strMain.charAt(enc3) + strMain.charAt(enc4);
    }
    return result;
  },

  getUserName: function() {
    return this.filterUsername(this.usernameProp.value);
  },

  getPassword: function() {
    return this.passwordProp.value;
  },

  doFetch: function() {
    this.parent();
    /*var u = new URL();
    u.location = "https://" + this.filterUsername(this.usernameProp.value) + ":" + this.passwordProp.value + "@mail.google.com/mail/feed/atom/";
    u["caller"] = this;
    u.fetchAsync(this.obj.updateFeed);*/

    if (this.req) {
      this.req.abort();
      this.req = null;
    }

    this.requestBody = null;
    this.req = new XMLHttpRequest()
    this.req.onreadystatechange = this.updateFeed.pass(this.req, this);
    this.setupRequest();
    this.req.send(this.requestBody);
  },

  setupRequest: function() {
    this.req.open("GET", this.getUrl(), true);
    this.req.setRequestHeader("Authorization", "Basic " + this.basicAuth(this.getUserName(), this.getPassword()));
  },

  genCacheName: function() {
    return "gmail-" + this.filterUsername(this.usernameProp.value) + "@gmail.com.xml";
  },

  setError: function() {
    if (arguments.length > 0) {
      this.parent(arguments[0]);
    } else {
      this.parent();
    }
    if (this.status) {
      if (this.oldData != "") {
        try {
          var old = XMLDOM.parse(this.oldData);
          try {
            count = Number(old.evaluate("string(feed/fullcount)"));
            this.setStatus(count);
          } finally {
            old = null;
          }
        } catch(e) {
          this.setStatus("n/a");
        }
      }
    }
  },

  filterUsername: function(value) {
    if (!value) return "";
    var a = value.split("@");
    if (a.length > 0) {
      return a[0];
    }
    return "";
  }
});

dockFactory.register({name: tr("GMail Checker"),
                      className: "DockGMail",
                      classDef: DockGMail,
                      category: tr("Internet"),
                      description: tr("Check mail on GMail")
                      });
