/*
FYI - the PlayState property tells you what's currently happening.

0 - Undefined
1 - Stopped
2 - Paused
3 - Playing
4 - ScanForward
5 - ScanReverse
6 - Buffering
7 - Waiting
8 - MediaEnded
9 - Transitioning
10 - Ready
11 - Reconnecting
*/
var PlayState = [tr("Undefined"), tr("Stopped"), tr("Paused"), tr("Playing"), 
                 tr("ScanForward"), tr("ScanReverse"), tr("Buffering"), 
                 tr("Waiting"), tr("MediaEnded"), tr("Transitioning"), 
                 tr("Ready"), tr("Reconnecting")];

var CurrentRadio = "";

var DockRadio = DockBase.extend({
  initialize: function() {
    this.parent();
    this.className = "DockRadio";
    
    this.image = null;
    this.radioStatus = 0;

    this.radioNameProp = new TextProp("radioName", "Radio Name:", "", "");
    this.radioUrlProp = new TextProp("radioUrl", "Radio URL:", "", "");
    this.radioAutostartProp = new BoolProp("radioAutostart", 
                                           "Autoplay on Informer start", false, "");
    this.imageProp = new FileProp("image", "Radio logo:", 
                                   "Radio logo, maximum height 24 px.\nLeave blank for default logo.");
    this.colorizeProp = new ColorizeProp("colorize", "Color:");
    this.opacityProp = new OpacityProp("opacity", "Opacity:", 255, 50);
    
    this.props = [this.radioNameProp, this.radioUrlProp, 
                  this.radioAutostartProp, this.imageProp, this.colorizeProp,
                  this.opacityProp];

    this.credits = "\n" + tr("Internet Radio sensor by Fabrice Deshayes aka Xtream");
    
    // Windows Media Player initialisation
    try {
      this.wmpPlayer = COM.createObject("WMPlayer.OCX");
      COM.connectObject( this.wmpPlayer, "wmpPlayer_" );
      this.wmpPlayer.controls.stop();
    } catch(e) {
      print("Internet Radio sensor initialize: " + e.message);
    }          
  },

  destroy: function() {
    if (this.parent()) {
      this.image = null;
      this.wmpPlayer.controls.stop();
      this.wmpPlayer = null;
    }
  },

  updateProps: function(loaded) {
    this.setImage();
    this.setOpacity();
    this.setColorize();
    this.makeElementInactive(this.image, this.radioStatus == 0);
    if (loaded) {
      if (this.radioAutostartProp.value) {
        this.playStop();
      } else {
        this.wmpPlayer.URL = this.radioUrlProp.value;
        this.wmpPlayer.controls.stop();
      }
    }
  },

  showProps: function() {
    var oldURL = this.radioUrlProp.value;
    if (this.parent()) {
      if (oldURL != this.radioUrlProp.value) {
        if (this.radioStatus == 0) {
          this.wmpPlayer.URL = this.radioUrlProp.value;
        } else {
          this.playStop();
          this.playStop();
        }
      }
      return true;
    }
    return false;
  },

  getName: function() {
    return tr("Internet Radio");
  },

  makeElementInactive: function(element, value) {
    if (!element) return;
    switch (preferences.inactiveSensorMode.value) {
      case "colorize":
        element.colorize = value?preferences.inactiveSensorColor.value:this.colorizeProp.colorize();
        break;
      case "opacity":
        element.opacity = value?parseInt(preferences.inactiveSensorOpacity.value):this.opacityProp.value;
        break;
    }
  },

  resetElementInactive: function(element) {
    if (!element) return;
    element.colorize = this.colorizeProp.colorize();
    element.opacity = this.opacityProp.value;
  },

  getHint: function() {
    var data = tr("Player status: ") + PlayState[this.wmpPlayer.playState] + 
              tr("\nRadio name: ") + this.radioNameProp.value;
    var click_text;
    if (this.radioStatus == 0) {
      click_text = tr("\n\nClick to play...");
    } else {
      data += tr("\nSong name: ") + tr("not available yet");
      click_text = tr("\n\nClick to stop...");
    }
    if (!dockObj.locked) {
      data += "\n" + click_text + "\n";
    }
    return data;
  },

  playStop: function() {
    try {
      if (this.radioStatus==0) {
        if (CurrentRadio) {
          eval(CurrentRadio);
        }
        CurrentRadio = this.obj.name + ".playStop();";
        this.wmpPlayer.URL = this.radioUrlProp.value;
        this.wmpPlayer.controls.play();
        this.radioStatus = 1;
      } else {
        CurrentRadio = "";
        this.wmpPlayer.controls.stop();
        this.radioStatus = 0;
      }
    } catch(e) {
      print(e.message);
    }      
    if (this.image) {
      this.makeElementInactive(this.image, this.radioStatus == 0);
    }
  },

  createControls: function() {
    if (this.parent()) {
      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.missingSrc = "Resources/Sensors/Radio.png";
      this.image.hOffset = 0;
      this.updateProps();
    }
    return (this.frame != null);
  },

  setImage: function(value) {
    if (this.image) {
      this.image.src = (this.imageProp.value != "") 
                     ? this.imageProp.value : "Resources/Sensors/Radio.png";
      this.image.vOffset = Math.round((25 - this.image.srcHeight) / 2);
    }
  },

  setOpacity: function(value) {
    if (this.image) {
      this.image.opacity = this.opacityProp.value;
    }
  },

  setColorize: function(value) {
    if (this.image) {
      this.image.colorize = this.colorizeProp.colorize();
    }
  },
  
  doClick: function() {
    this.parent();
    this.playStop();
  }
});

dockFactory.register({name: tr("Internet Radio"),
                      className: "DockRadio",
                      classDef: DockRadio,
                      category: tr("Media"),
                      description: tr("Listen internet-radio")
                      });
