
Informer.importclass("Sensors.MailHint");

var DockGenericMail = DockBaseTimer.extend({
  initialize: function(title) {
    this.startTimer = null;
    this.checking = null;
    this.oldData = "";
    this.data = "";
    this.lastUpdate = null;
    this.lastNewCount = 0;
    this.cache = "";
    this.isFetchHeaders = true;
    //this.parent(15 * 60.0, ".doFetch();");
    this.parent(15 * 60.0, this.doFetch);
    this.className = "DockGenericMail";
    this.errorMsg = "";
    this.total = null;

    this.error = false;

    this.headers = [];
    this.lastHeaderDate = 0;

    this.titleProp = new TextProp("title", "Title:", title, "");
    this.usernameProp = new TextProp("username", "Username:", "", "");
    this.passwordProp = new TextProp("password", "Password:", "", "", true);
    this.customIconProp = new FileProp("icon", "Icon:", 
                                       "Leave blank for default icon");
    this.rowLimitProp = new RowLimitProp("Max numbers of mails:",
                                         "How many mail messages display in Hint?");
    this.widthLimitProp = new WidthLimitProp();
    this.reverseOrderProp = new BoolProp("reverse", "Reverse order", false, "");
    this.notifyProp = new BoolProp("notify", "Display notification", true, "");
    
    if (this.isFetchHeaders) {
      this.props = [this.titleProp, this.usernameProp, this.passwordProp, 
                    this.intervalProp, this.customIconProp, 
                    this.rowLimitProp, this.widthLimitProp,
                    this.notifyProp];
    } else {
      this.props = [this.titleProp, this.usernameProp, this.passwordProp, 
                    this.intervalProp, this.customIconProp,
                    this.notifyProp];
    }
    
    // Only used by derived DockStandardMail, but makes showProps() easier.
    this.serverProp = new TextProp("server", "Server:", "", "");
    
    this.hint = null;
  },

  destroy: function() {
    if (this.startTimer) {
      this.startTimer.ticking = false;
    }
    if (this.parent()) {
      this.checking = null;
      this.label = null;
      this.status = null;
      this.icon = null;
      this.badge = null;
    }
  },

  getNotifyIcon: function() {
    return "Resources/Icons/Mail.png";
  },

  onAdd: function() {
    if (this.startTimer) {
      delete this.startTimer;
      this.startTimer = null;
    }
    this.startTimer = new Timer();
    this.startTimer.interval = 2.0;
    this.startTimer.onTimerFired = this.startUp.bind(this);
    this.startTimer.ticking = true;
  },

  startUp: function() {
    this.startTimer.ticking = false;
    //delete this.startTimer;
    this.startTimer = null;
    this.doFetch();
  },

  load: function(xml) {
    this.parent(xml);
    if (filesystem.itemExists(this.cache)) {
      this.oldData = filesystem.readFile(this.cache);
    }
  },
  
  updateProps: function(loaded) {
    this.setUsername();
    this.setTitle();
    this.setCustomIcon();  
    this.updateValue();
  },

  getName: function() {
    return tr("Mail Checker");
  },

  getHint: function() {
    return tr("Click to open Mail client");
  },

  showHint: function(x, y) {
    if (this.hint) {
      if ((this.headers.length <= 0) && (this.oldData != "")) {
        this.parseHeaders(this.oldData);
      }
      dockObj.hint.hide();
      this.hint.setOptions(this.rowLimitProp.value, this.widthLimitProp.value);
      if (this.errorMsg != "") {
        this.hint.setErrorText(this.errorMsg);
      } else {
        this.hint.render();
      }
      this.hint.show(x, y);
    }
    return false;
  },

  hideHint: function() {
    if (this.hint) {
      if (this.hint.win && this.hint.win.visible && !this.error) {
        this.setHasNews(false);
        this.oldData = this.data;
        if (this.total) this.setStatus(this.total);
        this.resetNew();
      }
      this.hint.hide();
    }
    return true;
  },

  resetNew: function() {
    for (var i = 0; i < this.headers.length; i++) {
      this.headers[i].isNew = false;
    }
  },

  minInterval: function() {
    return 60.0;
  },

  doOpenClient: function() {
    print("Open Mail Client");
  },

  getIcon: function() {
    return "Resources/Sensors/Mail-Rotated.png";
  },

  showProps: function() {
    var oldServer   = this.serverProp.value;
    var oldUsername = this.usernameProp.value;
    var oldPassword = this.passwordProp.value;
    if (this.parent()) {
      if ((oldServer   != this.serverProp.value)   ||
          (oldUsername != this.usernameProp.value) || 
          (oldPassword != this.passwordProp.value)) {
        this.doFetch();
      }
      return true;
    }
    return false;
  },

  doClick: function() {
    this.parent();
    this.setHasNews(false);
    this.oldData = this.data;
    this.resetNew();
    this.doOpenClient();
  },

  updateColor: function() {
    this.parent();
    if (this.label) { this.label.style.color = this.color; }
    if (this.status) { this.status.style.color = this.color; }
    if (this.checking) {
      if (this.checking.visible) {
        var isDark = !Color.isLightColor(Color.HexToRGB(this.color));
        this.checking.src = "Resources/Sensors/Checking" + (isDark?"-Dark":"") + ".gif";
      }
    }
  },

  updatePrefs: function() {
    this.parent();
    if (this.label) { $$font(this.label, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.status) { $$font(this.status, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    dockObj.align();
  },

  createHint: function() {
    this.hint = new MailHint(this);
  },

  createControls: function() {
    if (this.parent()) {
      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.hOffset = 30;
      this.label.vOffset = 11;
      this.label.data = this.titleProp.value;
      this.label.style.fontFamily = "Arial, Helvetica";
      this.label.style.fontSize = "9px";
      this.label.style.fontWeight = "bold";
      this.label.style.color = this.color;

      this.status = new Text(mainWindow);
      this.frame.addSubview(this.status);
      this.status.hOffset = 30;
      this.status.vOffset = 21;
      this.status.data = tr("n/a");
      this.status.style.fontFamily = "Arial, Helvetica";
      this.status.style.fontSize = "9px";
      this.status.style.fontWeight = "bold";
      this.status.style.color = this.color;

      this.icon = new Image(mainWindow);
      this.frame.addSubview(this.icon);
      this.icon.hOffset = 4;
      this.icon.vOffset = 3;
      this.icon.width = 21;
      this.icon.height = 20;
      this.icon.missingSrc = this.getIcon();
      this.icon.tracking = "rectangle";
      this.setCustomIcon();

      this.checking = new Image(mainWindow);
      this.frame.addSubview(this.checking);
      this.checking.hOffset = 31;
      this.checking.vOffset = 15;
      this.checking.src = "Resources/Sensors/Fill.png";
      this.checking.opacity = 150;
      this.checking.visible = false;

      this.badge = new Image(mainWindow);
      this.frame.addSubview(this.badge);
      this.badge.hOffset = 0;
      this.badge.vOffset = 13;
      this.badge.src = "Resources/Sensors/NewBadge.png";
      this.badge.visible = false;

      this.setHasNews(false);
      this.updateValue();

      this.createHint();
    }
    return (this.frame != null);
  },

  getNodes: function(xml, totals) {
    var setTotal = (totals != null)?totals:true;
    var nodes = null;
    try {
      var doc = XMLDOM.parse(xml);
      //nodes = doc.evaluate("/inbox/headers/entry/issued");
      // SpeedUp!
      //nodes = doc.evaluate("/inbox/headers/entry[position() > (count(node()) - " + (this.rowLimitProp.value + 1) + ")]/issued");
      if (this.rowLimitProp.value != -1) {
        nodes = doc.evaluate("/inbox/headers/entry[position() > (count(node()) - " + (this.rowLimitProp.value /*+ 1*/) + ")]");
      } else {
        nodes = doc.evaluate("/inbox/headers/entry");
      }
      if (setTotal) {
        this.total = parseInt(String(doc.evaluate("string(/inbox/stat/total)")));
      }
    } catch (e) { }
    return nodes;
  },

  parseDate: function(str) {
    return Date.gdateToDate(str);
  },

  sortByDate: function(item1, item2) {
    if (item1.date < item2.date) {
      return -1;
    }
    return (item1.date > item2.date) ? 1 : 0;
  },

  parseHeaders: function(xml) {
    this.headers = [];
    var nodes = this.getNodes(xml);
    if (nodes == null) {
      return false;
    }
    this.lastHeaderDate = 0;
    if (this.oldData != "") {
      var oldNodes = this.getNodes(this.oldData, false);
      if (oldNodes == null) {
        //return false;
        return true;
      }
      var node = oldNodes.item(this.reverseOrderProp.value?0:(oldNodes.length - 1));
      if (node) {
        var s = String(node.evaluate("string(issued/text())"));
        this.lastHeaderDate = this.parseDate(s);
      }
    }
    var start = nodes.length - ((this.rowLimitProp.value != -1)?this.rowLimitProp.value:0);
    var end = nodes.length;
    if (start < 0) {
      start = 0;
    }
    if (this.reverseOrderProp.value) {
      start = 0;
      end = ((this.rowLimitProp.value != -1)?this.rowLimitProp.value:nodes.length);
    }
    for (var i = start; i < end; i++) {
      try {
        var header = {};
        header.subject = String(nodes.item(i).evaluate("string(title/text())"));
        header.from = {};
        header.from.addr = String(nodes.item(i).evaluate("string(author/email/text())"));
        header.from.name = String(nodes.item(i).evaluate("string(author/name/text())"));
        header.date = this.parseDate(String(nodes.item(i).evaluate("string(issued/text())")));
        header.isNew = (header.date > this.lastHeaderDate);
        this.headers.push(header);
      } catch(e) {}
    }
    this.headers.sort(this.sortByDate);
    return true;
  },
  
  updateValue: function() {
    if (!this.label) return;

    try {
      if (this.parseHeaders(this.data)) {
        var newCount = 0;
        for (var i = 0; i < this.headers.length; i++) {
          if (this.headers[i].isNew) {
            newCount++;
          }
        }
        var count = (this.total)?this.total:this.headers.length;
        this.setStatus(newCount ? (newCount + '/' + count) : count);
        this.setHasNews(newCount);
        if (!arguments.length && this.hint && this.hint.win && this.hint.win.visible) {
          this.hint.render();
        }
        this.error = false;

        if (newCount && this.notifyProp.value) {
          notification.notify({title: this.titleProp.value, text: tr("You have %s messages.").sprintf(newCount), icon: this.getNotifyIcon()});
        }
      } else {
        this.setError("Format error");
      }
    } catch(e) {
      this.setError("Format error (" + e + ")");
    }
  },

  sensorMenuItems: function(items) {
    this.parent();

    var item = new MenuItem();
    item.title = (system.platform == "windows")?"-":"(-";
    items.push(item);

    var item = new MenuItem();
    item.title = tr("Check for Mail");
    item.onSelect = this.doFetch.bind(this);
    items.push(item);
  },

  beforeFetch: function() {
    this.error = false;
    if (this.usernameProp.value == "") return false;
    if (this.passwordProp.value == "") return false;
    return true;
  },

  doFetch: function() {
    if (!this.beforeFetch()) return;

    if (this.timer) {
      this.timer.ticking = false;
      this.timer.ticking = true;
    }
    
    this.startChecking();
  },

  startChecking: function() {
    if (this.badge) {
      this.badge.visible = false;
    }
    this.errorMsg = "";
    this.setStatus("");
    var isDark = !Color.isLightColor(Color.HexToRGB(this.color));
    if (!this.checking) return;
    this.checking.src = "Resources/Sensors/Checking" + (isDark?"-Dark":"") + ".gif";
    this.checking.opacity = 150;
    this.checking.visible = true;
  },

  finishChecking: function() {
    this.checking.visible = false;
    this.checking.src = "Resources/Sensors/Fill.png";
    this.setStatus("");
  },

  setError: function() {
    this.error = true;
    if (arguments.length > 0) {
      this.errorMsg = tr(arguments[0]);
    }
    this.setHasNews(false);
    if (this.badge) {
      this.badge.src = "Resources/Sensors/ErrorBadge.png";
      this.badge.visible = true;
    }
    this.setStatus("n/a");
    this.makeElementInactive(this.icon, true);
    if (this.hint) {
      this.hideHint();
    }
  },

  setHasNews: function(value) {
    if (this.badge) {
      this.badge.src = "Resources/Sensors/NewBadge.png";
      this.badge.visible = value;
      this.makeElementInactive(this.icon, !value);
    }
    if (value) {
      if (value > this.lastNewCount) {
        this.lastNewCount = value;
        this.doNotify();
      }
    } else {
      this.lastNewCount = 0;
    }
  },

  setStatus: function(value) {
    if (this.status) {
      value = tr(value);
      if ((this.status.data != value) && !this.checking.visible) {
        this.status.data = value;
        dockObj.align();
      }
    }
  },

  genCacheName: function() {
    return "mail-" + this.usernameProp.value + ".xml";
  },

  setUsername: function() {
    this.setHasNews(false);

    var path = system.widgetDataFolder;
    if (path != "") {
      this.cache = path + "/" + this.genCacheName();
    }
  },

  setTitle: function() {
    if (this.label && (this.label.data != this.titleProp.value)) {
      this.label.data = this.titleProp.value;
      dockObj.align();
    }
  },

  setCustomIcon: function() {
    if (this.icon) {
      this.icon.src = (this.customIconProp.value != "")
                    ? this.customIconProp.value : this.getIcon();
    }
  }
});
