
/* ------------------------------------------------------------------------ */

// Don't use ListProp here. We want always up-to-date adapter names.
var NetAdapterProp = StringProp.extend({
  initialize: function() {
    this.parent("net-adapter", "");
    this.className = "NetAdapterProp";
    
    if (system.platform == "windows") {
      this.net = new networkProviderWin();
    } else {
      this.net = new networkProviderMacOS();
    }
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = "Adapter:";
    f.type = "popup";
    f.option = [];
    try {
      //this.net.getList();
      if (this.net.current.name) {
        f.option.push(this.net.current.name);
      }
      for (var i = 0; i < this.net.devices.length; i++) {
        if (this.net.devices[i].name != this.net.current.name) {
          f.option.push(this.net.devices[i].name);
        }
      }
      f.defaultvalue = this.net.devices[0].name;
    } catch (e) {}
    f.value = this.value;
    ff.push(f);
  }
});

/* ------------------------------------------------------------------------ */

var DockNetwork = DockBaseDualGraph.extend({
  initialize: function() {
    this.parent();
    this.className = "DockNetwork";

    this.graphTopColorizeProp.reset("#20B7FF");
    this.graphBottomColorizeProp.reset("#FFAD5B");
    this.graphTopOpacityProp.reset(220);
    this.graphBottomOpacityProp.reset(220);
    
    this.adapterProp = new NetAdapterProp();
    this.bandwidthProp = new TextProp("bandwidth", "Bandwidth:", "", 
                                      "Max. bytes/sec, two comma separated values for different input/output speeds. Leave blank for automatic detection.");
    
    this.props = [this.adapterProp, this.bandwidthProp].concat(this.props);
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    this.net = this.adapterProp.net;
    var name = this.net.current.name;
    this.net.setCurrentDevice(this.adapterProp.value);
    if (name != this.net.current.name) {
      this.clear();
    }
    if (this.bandwidthProp.value) {
      var bw = this.bandwidthProp.value.match(/(\d+),(\d+)/);
      if (!bw) {
        bw = this.bandwidthProp.value.match(/\d+/);
        if (bw) {
          bw.push(bw[1]);
        }
      }
      if (bw) {
        if (this.net.maxIn < bw[1]) {
          this.scaleGraph(this.graphTop, this.net.maxIn, bw[1]);
          this.net.maxIn = bw[1];
        }
        if (this.net.maxOut < bw[2]) {
          this.scaleGraph(this.graphBottom, this.net.maxOut, bw[2]);
          this.net.maxOut = bw[2];
        }
      }
    }
  },
  
  scaleGraph: function(graph, oldValue, newValue) {
    var f = oldValue / newValue;
    for (var i = 0; i < graph.length; i++) {
      graph[i].height = Math.round(graph[i].height * f);
    }
  },

  createControls: function() {
    if (this.parent()) {
      this.label.hAlign = "right";
      this.label.data = "1023 MB/s";
      this.label.hOffset = this.bk.hOffset + this.bk.width + 5 + this.label.width;
      this.value.hOffset = this.label.hOffset;
    }
    return (this.frame != null);
  },
  
  getName: function() {
    return tr("Network Graph");
  },

  graphName: function() {
    return tr("Net");
  },

  updateValue: function() {
    this.parent();
    var net = this.net.getData();
    this.label.data = trBytesPerSecToUI(net.bytesIn);
    this.value.data = trBytesPerSecToUI(net.bytesOut);
  },
  
  getValue: function() {
    var old = this.net.getData();
    this.net.updateData();
    var net = this.net.getData();
    if (old.maxIn < net.maxIn) {
      this.scaleGraph(this.graphTop, old.maxIn, net.maxIn);
    }
    if (old.maxOut < net.maxIn) {
      this.scaleGraph(this.graphBottom, old.maxOut, net.maxOut);
    }
    var percentIn = (net.bytesIn * 100) / net.maxIn;
    var percentOut = (net.bytesOut * 100) / net.maxOut;
    return Math.round(percentIn > percentOut ? percentIn : percentOut);
  },

  getValueTop: function() {
    var net = this.net.getData();
    return Math.round((net.bytesIn * 100) / net.maxIn);
  },

  getValueBottom: function() {
    var net = this.net.getData();
    return Math.round((net.bytesOut * 100) / net.maxOut);
  },

  getTitle: function() {
    return tr("Network usage history");
  },

  getHint: function() {
    var net = this.net.getData();
    var text = tr("Adapter:") + " " + this.net.current.name +
               "\n   " + tr("Status: " + (this.net.current.active ? tr("active") : tr("inactive")))  +
               "\n   " + tr("Receive: ") + trBytesPerSecToUI(net.bytesIn) + 
               tr(" (max. ") + trBytesPerSecToUI(net.maxIn) + ")" +
               "\n   " + tr("Send: ") + trBytesPerSecToUI(net.bytesOut) + 
               tr(" (max. ") + trBytesPerSecToUI(net.maxOut) + ")";
    if ((system.platform == "windows") && !dockObj.locked) {
      text += tr("\n\nClick to open Network Connections");
    }
    return text;
  },

  wakeUp: function() {
    this.net.wakeUp();
  },

  doClick: function() {
    this.parent();
    if (system.platform == "windows") {
      runCommandInBg("%SystemRoot%/system32/control.exe netconnections", "netConnectionsRun");
    }
  }
});

dockFactory.register({name: tr("Network Graph"),
                      className: "DockNetwork",
                      classDef: DockNetwork,
                      category: tr("Network"),
                      description: tr("Show network usage history")
                      });
