
var pmLock      = 0;
var pmLogout    = 1;
var pmFUS       = 2;
var pmSuspend   = 3;
var pmHibernate = 4;
var pmShutdown  = 5;
var pmRestart   = 6;

/* ------------------------------------------------------------------------ */

var PowerModeProp = ListProp.extend({
  initialize: function() {
    if (system.platform == "macintosh") {
      var optTexts = [tr("Logout"), tr("Suspend"), tr("Shutdown"), 
                      tr("Restart")];
      var optValues = [pmLogout, pmSuspend, pmShutdown, pmRestart];
    } else {
      var optTexts = [tr("Lock"), tr("Logout"), tr("Fast User Switching"),
                      tr("Suspend"), tr("Hibernate"), tr("Shutdown"), 
                      tr("Restart")];
      var optValues = [pmLock, pmLogout, pmFUS, pmSuspend, pmHibernate,
                       pmShutdown, pmRestart];
    }
    this.parent("mode", "Mode:", pmSuspend, optTexts, optValues);
    this.className = "PowerModeProp";
  }
});

/* ------------------------------------------------------------------------ */

var DockPower = DockBase.extend({
  initialize: function() {
    this.parent();
    this.className = "DockPower";
    this.image = null;
    this.query = "";
    
    this.modeProp = new PowerModeProp();
    this.safetyProp = new BoolProp("safety-dialog", "Safety Dialog", true,
                                   "Check this if a safety dialog should be displayed before execution of the choosen function.");
    this.opacityProp = new OpacityProp("opacity", "Opacity:", 255, 50);
    
    if (system.platform == "macintosh") {
      this.props = [this.modeProp, this.safetyProp, this.opacityProp];
    } else {
      this.poweroffPathProp = new FileProp("path", "PowerOff:",
                                           "If specified use poweroff.exe instead of default functions.\n\nThe download link for the utility by Jorgen Bosman is:\nhttp://users.pandora.be/jbosman/pwroff30.zip.",
                                          [".exe"]);
      this.props = [this.modeProp, this.safetyProp, this.poweroffPathProp, 
                    this.opacityProp];
    }
  },

  destroy: function() {
    if (this.parent()) {
      this.image = null;
    }
  },

  updateProps: function(loaded) {
    if (!this.image) return;
    
    if (this.modeProp.value == pmLock) {
      this.image.src = "Resources/Sensors/Power/Lock.png";
      this.query = "Do you really want to lock this computer?";
    } else if (this.modeProp.value == pmLogout) {
      this.image.src = "Resources/Sensors/Power/Logout.png";
      this.query = "Do you really want to log out of this session?";
    } else if (this.modeProp.value == pmFUS) {
      this.image.src = "Resources/Sensors/Power/FUS.png";
      this.query = "Do you really want to switch to another account?";
    } else if (this.modeProp.value == pmHibernate) {
      this.image.src = "Resources/Sensors/Power/Hibernate.png";
      this.query = "Do you really want to hibernate this computer?";
    } else if (this.modeProp.value == pmShutdown) {
      this.image.src = "Resources/Sensors/Power/Shutdown.png";
      this.query = "Do you really want to shutdown this computer?";
    } else if (this.modeProp.value == pmRestart) {
      this.image.src = "Resources/Sensors/Power/Restart.png";
      this.query = "Do you really want to restart this computer?";
    } else {
      this.image.src = "Resources/Sensors/Power/Suspend.png";
      this.query = "Do you really want to suspend this computer?";
    }
    this.image.opacity = this.opacityProp.value;
  },

  getName: function() {
    return tr("Power");
  },

  createControls: function() {
    if (this.parent()) {
      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.src = "Resources/Sensors/Power/Suspend.png";
      this.image.hOffset = 0;
      this.image.vOffset = 2;
      this.image.height = 21;
      this.image.width = 21;
      this.image.tracking = "rectangle";
      this.image.useFileIcon = false;

      this.updateProps();
    }
    return (this.frame != null);
  },

  getTitle: function() {
  },

  getHint: function() {
    //if ((!dockObj.locked || (system.event.modifiers.indexOf("option") != -1) )) {
      return this.modeProp.optTexts[this.modeProp.value];
    //}
    //return "";
  },

  doClick: function() {
    this.parent();

    var cmd;
    if (system.platform == "macintosh") {
      if (this.modeProp.value == pmLogout) {
        cmd = "osascript -e 'tell application\"System Events\" to log out'";
      } else if (this.modeProp.value == pmShutdown) {
        cmd = "osascript -e 'tell application \"System Events\" to shut down'";
      } else if (this.modeProp.value == pmRestart) {
        cmd = "osascript -e 'tell application \"System Events\" to restart'";
      } else {
        cmd = "osascript -e 'tell application \"System Events\" to sleep'";
      }
    } else if (this.poweroffPathProp.value == "") {
      if (this.modeProp.value == pmLock) {
        cmd = "RunDll32.exe user32.dll,LockWorkStation";
      } else if (this.modeProp.value == pmLogout) {
        cmd = "shutdown.exe -l -f";
      } else if (this.modeProp.value == pmFUS) {
        cmd = "RunDll32.exe user32.dll,LockWorkStation";
      } else if (this.modeProp.value == pmHibernate) {
        cmd = "RunDll32.exe powrprof.dll SetSuspendState 1,1,0";
      } else if (this.modeProp.value == pmShutdown) {
        cmd = "shutdown.exe -s -f -t 0";
      } else if (this.modeProp.value == pmRestart) {
        cmd = "shutdown.exe -r -f -t 0";
      } else {
        cmd = "RunDll32.exe powrprof.dll SetSuspendState 0,1,0";
      }
    } else {
      cmd = this.poweroffPathProp.value;
      if (this.modeProp.value == pmLock) {
        cmd += " lock";
      } else if (this.modeProp.value == pmLogout) {
        cmd += " logoff";
      } else if (this.modeProp.value == pmFUS) {
        cmd = "RunDll32.exe user32.dll,LockWorkStation";
      } else if (this.modeProp.value == pmHibernate) {
        cmd += " hibernate";
      } else if (this.modeProp.value == pmShutdown) {
        cmd += " shutdown -force";
      } else if (this.modeProp.value == pmRestart) {
        cmd += " reboot -force";
      } else {
        cmd += " standby -force";
      }
    }
    if ((!this.safetyProp.value || (alert(tr(this.query), tr("Yes"), tr("No")) == 1) )) {
      //runCommand(cmd);
      runCommandInBg(cmd, "PowerTools");
    }
  }
});

dockFactory.register({name: tr("Power"),
                      className: "DockPower",
                      classDef: DockPower,
                      category: tr("System"),
                      description: tr("Power off, reboot, lock, ... computer")
                      });
