
include("Sensors/RSSHint.js");

var DockRSSSensor = DockBaseTimer.extend({
  initialize: function() {
    this.link = "";
    this.checking = null;
    this.oldData = "";
    this.data = "";
    this.rss = null;
    this.parseDate = Date.dateToMonthDay;
    this.lastUpdate = null;
    this.lastNewCount = 0;
    this.cache = "";
    //this.parent(15 * 60.0, ".doFetch();");
    this.parent(15 * 60.0, this.doFetch);
    this.className = "DockRSSSensor";
    
    this.errorMsg = "";
    this.error = false;

    this.titleProp = new TextProp("title", "Title:", "RSS", "");
    this.urlProp = new TextProp("url", "URL:", "", "");
    this.customIconProp = new FileProp("icon", "Icon:", 
                                       "Leave blank for default icon");
    this.reverseOrderProp = new BoolProp("reverse", "Reverse order", false, "");
    this.sortDateProp = new BoolProp("sort", "Sort by date", false, "");
    this.rowLimitProp = new RowLimitProp("Max numbers of news:",
                                         "How many news display in Hint?");
    this.widthLimitProp = new WidthLimitProp();
    this.notifyProp = new BoolProp("notify", "Display notification", true, "");
    
    this.props = [this.titleProp, this.urlProp, this.intervalProp, 
                  this.customIconProp, this.rowLimitProp, this.widthLimitProp, this.reverseOrderProp, 
                  this.sortDateProp, this.notifyProp];

    this.hint = null;
  },

  destroy: function() {
    if (this.parent()) {
      this.checking = null;
      this.label = null;
      this.status = null;
      this.icon = null;
      this.badge = null;
    }
  },

  onAdd: function() {
    this.doFetch();
  },

  load: function(xml) {
    this.parent(xml);
    if (filesystem.itemExists(this.cache)) {
      this.oldData = filesystem.readFile(this.cache);
    }
  },
  
  updateProps: function(loaded) {
    if (loaded) {
      this.setURL(this.urlProp.value);
    }
    this.setTitle();
    this.setCustomIcon();
    this.updateValue();
  },

  getName: function() {
    return tr("RSS Feed");
  },

  getHint: function() {
    return (!dockObj.locked?tr("Click to open RSS Reader"):"");
  },

  showHint: function(x, y) {
    if (this.hint) {
      dockObj.hint.hide();
      this.hint.setOptions(this.widthLimitProp.value);
      if (this.errorMsg != "") {
        this.hint.setErrorText(this.errorMsg);
      } else {
        this.hint.render();
      }
      this.hint.show(x, y);
    }
    return false;
  },

  hideHint: function() {
    if (this.hint) {
      if (this.hint.win && this.hint.win.visible && !this.error) {
        this.setHasNews(false);
        this.oldData = this.data;
        this.updateValue(false);
      }
      this.hint.hide();
    }
    return true;
  },

  minInterval: function() {
    return 60.0;
  },

  showProps: function() {
    var oldURL = this.urlProp.value;
    if (this.parent()) {
      if (oldURL != this.urlProp.value) {
        this.setURL();
      }
      return true;
    }
    return false;
  },

  doClick: function() {
    this.parent();
    this.setHasNews(false);
    this.oldData = this.data;
    // open RSS Reader
    if (this.data != "") {
      dockObj.openRSS(this.urlProp.value, this.data, this.link);
    }
  },

  updatePrefs: function() {
    this.parent();
    if (this.label) { $$font(this.label, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.status) { $$font(this.status, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    dockObj.align();
  },

  updateColor: function() {
    this.parent();
    if (this.label) { this.label.style.color = this.color; }
    if (this.status) { this.status.style.color = this.color; }
    if (this.checking) {
      if (this.checking.visible) {
        var isDark = !Color.isLightColor(Color.HexToRGB(this.color));
        this.checking.src = "Resources/Sensors/Checking" + (isDark?"-Dark":"") + ".gif";
      }
    }
  },

  createControls: function() {
    if (this.parent()) {
      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.hOffset = 27;
      this.label.vOffset = 11;
      this.label.data = this.titleProp.value;
      this.label.style.fontFamily = "Arial, Helvetica";
      this.label.style.fontSize = "9px";
      this.label.style.fontWeight = "bold";
      this.label.style.color = this.color;

      this.status = new Text(mainWindow);
      this.frame.addSubview(this.status);
      this.status.hOffset = 27;
      this.status.vOffset = 21;
      this.status.data = "no news";
      this.status.style.fontFamily = "Arial, Helvetica";
      this.status.style.fontSize = "9px";
      this.status.style.fontWeight = "bold";
      this.status.style.color = this.color;

      this.icon = new Image(mainWindow);
      this.frame.addSubview(this.icon);
      this.icon.hOffset = 3;
      this.icon.vOffset = 3;
      this.icon.width = 20;
      this.icon.height = 20;
      this.icon.missingSrc = "Resources/Sensors/RSS.png";
      this.setCustomIcon();

      this.checking = new Image(mainWindow);
      this.frame.addSubview(this.checking);
      this.checking.hOffset = 27;
      this.checking.vOffset = 15;
      this.checking.src = "Resources/Sensors/Fill.png";
      this.checking.opacity = 150;
      this.checking.visible = false;

      this.badge = new Image(mainWindow);
      this.frame.addSubview(this.badge);
      this.badge.hOffset = 0;
      this.badge.vOffset = 13;
      this.badge.src = "Resources/Sensors/NewBadge.png";
      this.badge.visible = false;

      this.setHasNews(false);
      this.updateValue();

      this.hint = new RSSHint(this);
    }
    return (this.frame != null);
  },

  getNodes: function(xml) {
    var doc;
    var nodes;
    try {
      doc = XMLDOM.parse(xml);
      
      if (this.rss) {
        return doc.evaluate(this.rss.item);
      }
      // Test RSS format
      nodes = doc.evaluate("/rss/channel/item");
      if (nodes.length > 0) {
        this.rss = { item: "/rss/channel/item", id: "link", date: "pubDate" };
        this.link = String(doc.evaluate("string(/rss/channel/link)"));
        this.parseDate = Date.dateToMonthDay;
        return nodes;
      }
      // Test RDF format
      nodes = doc.evaluate("/rdf:RDF/item");
      if (nodes.length > 0) {
        this.rss = { item: "/rdf:RDF/item", id: "link", date: null };
        this.link = String(doc.evaluate("string(/rdf:RDF/channel/link)"));
        return nodes;
      }
      // Test Atom format
      nodes = doc.evaluate("/feed/entry");
      if (nodes.length > 0) {
        this.rss = { item: "/feed/entry", id: "id", date: "updated" };
        this.link = String(doc.evaluate("string(/feed/id)"));
        this.parseDate = Date.atomDateToMonthDay;
        return nodes;
      }
    } catch (e) { }
    return null;
  },
  
  sortByDate: function(item1, item2) {
    if (item1.date > item2.date) {
      return -1;
    }
    return (item1.date < item2.date) ? 1 : 0;
  },

  parseNode: function(node) {
    var header = { title: "", id: "", date:"", isNew:true };
    try {
      header.title = String(node.evaluate("string(title/text())"));
      header.id = String(node.evaluate("string("+this.rss.id+"/text())"));
      if (this.rss.date) {
        header.date = this.parseDate(String(node.evaluate("string("+this.rss.date+"/text())")));
      }
    } catch (e) { }
    return header;
  },

  parseHeaders: function() {
    var headers = [];
      
    // Get all entries
    if (this.data != "") {
      var nodes = this.getNodes(this.data);
      if (nodes != null) {
        for (var i = 0; i < nodes.length; i++) {
          var header = this.parseNode(nodes.item(i));
          if (header) {
            headers.push(header);
          }
        }
      }
    }

    // Sort and order all entries
    if (this.sortDateProp.value) {
      headers.sort(this.sortByDate);
    }
    this.total = headers.length;
    this.newCount = headers.length;
    this.headers = [];

    if (this.reverseOrderProp.value) {
      var last = headers.length - 1;
      for (var i = 0; i <= last; i++) {
        this.headers.push(headers[last - i]);
      }
    } else {
      for (var i = 0; i < headers.length; i++) {
        this.headers.push(headers[i]);
      }
    }

    // Mark already known entries as old
    if ((this.total > 0) && (this.oldData != "")) {
      var nodes = this.getNodes(this.oldData);
      if (nodes != null) {
        for (var i = 0; i < nodes.length; i++) {
          var node = nodes.item(i); // ugly hack to avoid Y!WE crashes!!!
          var header = this.parseNode(nodes.item(i));
          if (header) {
            for (var h = 0; h < this.total; h++) {
              if (this.headers[h].isNew && (this.headers[h].id == header.id) &&
                  (this.headers[h].date == header.date)) {
                this.headers[h].isNew = false;
                this.newCount--;
                break;
              }
            }
          }
        }
      }
    }

    // Check row limits
    var count = this.rowLimitProp.value;
    if ((count != -1) && (count < this.total)) {
      // Remove the last entries: old entries and if necessary some new one
      var remove_new = 0;
      var remove_old = this.total - count;
      if (this.newCount > count) {
        remove_new = this.newCount - count;
        remove_old = this.total - this.newCount;
      }
      for (var h = this.total - 1; remove_old + remove_new > 0; h--) {
        if (this.headers[h].isNew) {
          if (remove_new) {
            this.headers.removeByIndex(h);
            remove_new--;
          }
      } else {
          if (remove_old) {
            this.headers.removeByIndex(h);
            remove_old--;
          }
        }
      }
    }
  },
  
  updateValue: function() {
    if (!this.label) return;

    try {
      if (this.errorMsg == "") {
        this.parseHeaders();
        var count = (this.total)?this.total:this.headers.length;
        this.setStatus(this.newCount ? (this.newCount + '/' + count) : count);
        this.setHasNews(this.newCount);
        if (!arguments.length && this.hint && this.hint.win && this.hint.win.visible) {
          this.hint.render();
        }

        if (this.newCount && this.notifyProp.value) {
          notification.notify({title: this.titleProp.value, text: tr("%s new headlines.").sprintf(this.newCount), icon: "Resources/Icons/RSS.png"});
        }
      }
    } catch(e) {
      this.setError("Unknown RSS format");
    }
  },

  sensorMenuItems: function(items) {
    this.parent();

    var item = new MenuItem();
    item.title = (system.platform == "windows")?"-":"(-";
    items.push(item);

    var item = new MenuItem();
    item.title = tr("Check for news");
    item.onSelect = this.doFetch.bind(this);
    items.push(item);
  },

  updateFeed: function(url) {
    var t = url["caller"];
    t.finishChecking();
    var data = url.result;
    // fix encoding <?... encoding= to UTF-8
    if (data.indexOf("<?xml") != -1) {
      var idx = data.indexOf("?>");
      if (idx != -1) {
        data = data.substr(idx + 2, data.length - (idx + 2));
      }
    }
    data = "<?xml version='1.0' encoding='UTF-8'?>\n" + data;
    if (url.response == 200) {
      t.data = data;
      t.lastUpdate = new Date();
      // save cache
      if (t.cache != "") {
        filesystem.writeFile(t.cache, data);
      }
      t.makeElementInactive(t.icon, false);
      this.error = false;
      t.updateValue();
    } else {
      t.setError("Connection time-out.");
    }
  },

  doFetch: function() {
    if (this.urlProp.value == "") return;
    this.startChecking();
    var u = new URL();
    u.location = this.urlProp.value;
    u["caller"] = this;
    u.fetchAsync(this.obj.updateFeed);
  },

  startChecking: function() {
    this.error = false;
    if (this.badge) {
      this.badge.visible = false;
    }
    //this.setStatus("loading...");
    this.setStatus("");
    this.errorMsg = "";
    var isDark = !Color.isLightColor(Color.HexToRGB(this.color));
    this.checking.src = "Resources/Sensors/Checking" + (isDark?"-Dark":"") + ".gif";
    this.checking.opacity = 150;
    this.checking.visible = true;
  },

  finishChecking: function() {
    this.checking.visible = false;
    this.checking.src = "Resources/Sensors/Fill.png";
    this.setStatus("");
  },

  setError: function() {
    this.error = true;
    if (arguments.length > 0) {
      this.errorMsg = tr(arguments[0]);
    }
    this.setHasNews(false);
    if (this.badge) {
      this.badge.src = "Resources/Sensors/ErrorBadge.png";
      this.badge.visible = true;
    }
    this.makeElementInactive(this.icon, true);
    this.setStatus("n/a");
  },

  setHasNews: function(value) {
    if (this.badge) {
      this.badge.src = "Resources/Sensors/NewBadge.png";
      this.badge.visible = value;
      this.makeElementInactive(this.icon, !value);
    }
    if (value) {
      if (value > this.lastNewCount) {
        this.lastNewCount = value;
      this.doNotify();
    }
    } else {
      this.lastNewCount = 0;
    }
  },

  setStatus: function(value) {
    if (this.status) {
      value = tr(value);
      if ((this.status.data != value) && !this.checking.visible) {
        this.status.data = value;
        dockObj.align();
      }
    }
  },

  setURL: function(url) {
    if (url) {
      this.urlProp.value = url;
      this.rss = null;
    }
    if (this.urlProp.value.length > 0) {
      if (this.urlProp.value.indexOf("://") == -1) {
        this.urlProp.value = "http://" + this.urlProp.value;
      }
    }
    this.setHasNews(false);

    // calculate this.cache here
    var uri = this.urlProp.value.split("/");
    uri.shift();
    uri.shift();
    if (uri.length > 0) {
      var host = uri[0];
      var doc = uri[uri.length - 1];
      if (doc.indexOf(".") != -1) {
        doc = doc.split(".")[0];
      }

      var path = system.widgetDataFolder;
      if (path != "") {
        this.cache = path + "/rss-" + host + "-" + doc + ".xml";
      }
    }

    if (this.frame) {
      this.doFetch();
    }
  },

  setTitle: function() {
    if (this.label && (this.label.data != this.titleProp.value)) {
      this.label.data = this.titleProp.value;
      dockObj.align();
    }
  },

  setCustomIcon: function(value) {
    if (this.icon) {
      this.icon.src = (this.customIconProp.value != "")
                    ? this.customIconProp.value : "Resources/Sensors/RSS.png";
    }
  }
});

dockFactory.register({name: tr("RSS/Atom Feed"),
                      className: "DockRSSSensor",
                      classDef: DockRSSSensor,
                      category: tr("Internet"),
                      description: tr("RSS/Atom Feed monitoring")
                      });
