
var DockShortcut = DockBaseShortcut.extend({
  initialize: function() {
    this.parent();
    this.className = "DockShortcut";
    
    this.linkProp = new FilePropEx("link", "Application or Widget:",
                                    "Application, Widget, Document or Folder to open.",
                                    [".exe", ".cmd", ".lnk", ".widget"]);
    this.paramProp = new TextProp("param", "Parameters:", "", 
                                  "Parameters for application");
    this.urlProp = new TextProp("url", "... or Internet Link:", "", 
                                    "Internet Link to open, must start with http://, https:// or ftp://");
    
    this.props.unshift(this.linkProp, this.paramProp, this.urlProp);
  },

  updateProps: function(loaded) {
    if (loaded && (this.linkProp.value != "")) {
      this.setLink(this.linkProp.value);
    }
    this.parent();
  },

  getName: function() {
    return tr("Shortcut");
  },

  getTarget: function() {
    if (this.urlProp.value != "") {
      return this.urlProp.value;
    } else {
      return this.linkProp.value;
    }
  },

  setLink: function(value) {
    if (value.match(/^(http|https|ftp):\/\//i)) {
      this.linkProp.value = "";
      var d = value.split("\n");
      this.urlProp.set(d[0]);
      this.titleProp.set(d[1]);
    } else {
      this.linkProp.value = value;
      this.urlProp.value = "";
    }
    this.setImage();
  },

  doSetImage: function(value) {
    if (this.imageProp.value == "") {
      if (this.linkProp.value.match(/\.url$/i) || (this.urlProp.value != "")) {
        this.image.src = "Resources/Sensors/Web.png";
        this.image.useFileIcon = false;
      } else {
        this.image.src = (this.linkProp.value != "") 
                       ? this.linkProp.value : "Resources/Sensors/App.png";
        this.image.useFileIcon = (this.linkProp.value != "");
      }
    } else {
      this.image.src = this.imageProp.value;
      this.image.useFileIcon = false;
    }
  },

  doGetTitle: function() {
    if (this.urlProp.value != "") {
      s = this.urlProp.value;
      if (s.length > 150) {
        s = s.substr(0, 75) + "..." + s.substr(s.length - 75);
      }
      return s;
    }
    var fname = this.linkProp.value.extractFileName();
    if (fname != "") {
      var idx = fname.lastIndexOf(".");
      if (idx != -1) {
        fname = fname.substr(0, idx);
      }
    }
    return fname;
  },

  doClick: function() {
    this.parent();
    if (this.urlProp.value != "") {
      this.openURL(this.urlProp.value);
    } else {
      var app = this.linkProp.value;
      if (filesystem.isDirectory(app)) {
        filesystem.open(app);
        return;
      }
      if (system.platform == "windows") {
        if (app.match(/\.widget$/i)) {
          app = "cmd.exe /c \"" + app + "\"";
        } else {
          app = "\"" + app + "\"";
          if (this.paramProp.value != "") {
            app += " " + this.paramProp.value.replace("?", "\\?");
          }
        }
      }
      runCommandInBg(app, "shortcutRun_" + this.name);
    }
  }
});

dockFactory.register({name: tr("Shortcut"),
                      className: "DockShortcut",
                      classDef: DockShortcut,
                      category: tr("General"),
                      description: tr("... to application, widget, folder or web-site")
                      });
