
/* ------------------------------------------------------------------------ */

var SpacerWidthProp = BaseProp.extend({
  initialize: function() {
    this.parent("width", 9);
    this.className = "SpacerWidthProp";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Spacer width in pix.:");
    f.type = "slider";
    f.minLength = 3;
    f.maxLength = 145;
    f.tickLabel = ["3 " + tr("pix."), "145 " + tr("pix.")];
    f.defaultValue = 9;
    f.value = this.value;
    ff.push(f);
  },
  
  set: function(value) {
    value = parseInt(value);
    if (!isNaN(value)) {
      this.value = value;
    }
  }
});

/* ------------------------------------------------------------------------ */

var DockSpacer = DockBase.extend({
  initialize: function(disk) {
    this.parent();
    this.className = "DockSpacer";
    
    this.showDividerProp = new BoolProp("show", "Show Divider", false, "");
    this.dividerWidthProp = new SpacerWidthProp();
    
    this.props = [this.showDividerProp, this.dividerWidthProp];
  },

  destroy: function() {
    if (this.parent()) {
      this.image = null;
    }
  },

  updateProps: function(loaded) {
    if (this.image) {
      this.image.opacity = this.showDividerProp.value ? 100 : 0;
      this.image.hOffset = Math.round((this.dividerWidthProp.value - 1) / 2);
    }
    if (this.bk) {
      this.bk.hOffset = this.dividerWidthProp.value;
    }
    dockObj.align();
  },

  getName: function() {
    return tr("Spacer");
  },

  updateColor: function() {
    this.parent();
    if (this.image) { this.image.colorize = this.color; }
  },

  getWidth: function() {
    return this.dividerWidthProp.value;
  },

  createControls: function() {
    if (this.parent()) {
      this.bk = new Image(mainWindow);
      this.frame.addSubview(this.bk);
      this.bk.hOffset = 0;
      this.bk.vOffset = 0;
      this.bk.width = this.dividerWidthProp.value;
      this.bk.height = this.frame.height;
      this.bk.src = "Resources/Sensors/Fill.png";
      this.bk.opacity = 0;
      this.bk.tracking = "rectangle";

      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.hOffset = Math.round((this.dividerWidthProp.value - 1) / 2);
      this.image.vOffset = 3;
      this.image.width = 1;
      this.image.height = 19;
      this.image.src = "Resources/Sensors/Fill.png";
      this.image.opacity = this.showDividerProp.value?100:0;
    }
    return (this.frame != null);
  },

  doMouseEnter: function() {
    this.parent();
    if (!this.showDividerProp.value) {
      animator.start(new FadeAnimation(this.image, 100, 200, animator.kEaseIn));
    }
  },

  doMouseExit: function() {
    this.parent();
    if (!this.showDividerProp.value) {
      animator.start(new FadeAnimation(this.image, 0, 300, animator.kEaseOut));
    }
  }
});

dockFactory.register({name: tr("Spacer"),
                      className: "DockSpacer",
                      classDef: DockSpacer,
                      category: tr("General"),
                      description: tr("Spacer and/or delimiter")
                     });
