﻿
var speedFan =  {date: new Date(0), columns: null, values: null};

var sfuCelsius    = 0;
var sfuFahrenheit = 1;
var sfuRPM        = 2;
var sfuVolt       = 3;

/* ------------------------------------------------------------------------ */

// Don't use ListProp here. We want always up-tp-date column headers.
var ColNameProp = StringProp.extend({
  initialize: function() {
    this.parent("col-name", "");
    this.className = "ColNameProp";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = "Column:";
    f.type = "popup";
    f.option = [];
    try {
      for (var i = 1; i < speedFan.columns.length; i++) {
        f.option.push(speedFan.columns[i]);
      }
      f.defaultvalue = speedFan.columns[1];
    } catch (e) {}
    f.value = this.value;
    ff.push(f);
  }
});

/* ------------------------------------------------------------------------ */

var UnitProp = ListProp.extend({
  initialize: function() {
    var optTexts = ["°C", "°F", tr("rpm"), "V"];
    var optValues = [sfuCelsius, sfuFahrenheit, sfuRPM, sfuVolt];
    this.parent("unit", "Unit:", sfuCelsius, optTexts, optValues);
    this.className = "UnitProp";
  }
});

/* ------------------------------------------------------------------------ */

var AlertProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("No"), tr("30 sec."), tr("1 min."), tr("2 min."), tr("3 min."), tr("4 min."), tr("5 min.")];
    var optValues = [0, 0.5, 1, 2, 3, 4, 5];
    this.parent("alert", "Display alert every:", 1, optTexts, optValues, "");
    this.className = "AlertProp";
  },
  
  set: function(value) {
    value = parseFloat(value);
    this.value = !isNaN(value) ? value : this.defaultValue;
  }  
});

/* ------------------------------------------------------------------------ */

var DockSpeedFan = DockBaseMeter.extend({
  initialize: function(disk) {
    this.colIdx = -1;
    this.unitText = "°C";
    this.parent();
    this.className = "DockSpeedFan";
    this.timeStamp = -1;
    this.timeStampChanged = true;
    this.valueMin = 1000000;
    this.valueMax = -1000000;
    this.alertTime = 0;
    
    this.titleProp = new TextProp("title", "Title:", "",
                                  "Sensor's title (leave blank for SpeedFan header title)");
    this.colNameProp = new ColNameProp();
    this.limitMinProp = new TextProp("val-min", "Min. Value:", "20.0", "");
    this.limitMaxProp = new TextProp("val-max", "Max. Value:", "80.0", "");
    this.unitProp = new UnitProp();
    this.colorizeProp = new MultiColorProp("colorize", "Color:");
    this.alertProp = new AlertProp();
    
    this.props = [this.titleProp, this.colNameProp, 
                  this.limitMinProp, this.limitMaxProp, this.unitProp, 
                  this.intervalProp, this.colorizeProp, this.alertProp];
  },

  updateProps: function(loaded) {
    this.setUnit();
    this.parent(loaded);
    this.refreshLogs();
    this.updateValue();
  },

  minInterval: function() {
    return 3.0;
  },
  
  getName: function() {
    return "SpeedFan " + this.meterName();
  },

  meterName: function() {
    if (this.titleProp.value != "") {
      return this.titleProp.value;
    }
    if (this.colNameProp.value) {
      return this.colNameProp.value;
    }
    if (speedFan.columns && (speedFan.columns.length > 1)) {
      return speedFan.columns[1];
    }
    return "SpeedFan";
  },
  
  getFileName: function() {
    var now = new Date();
    return preferences.speedFanLog.value + "/" + now.toString("SFLog%Y%m%d.csv");
  },
  
//  getFile: function(tail) {
//    if (tail) {
//      //return runCommand('tail -1 "' + this.getFileName() + '"');
//      // Speed up!
//      var lines = filesystem.readFile(this.getFileName(), true);
//      return (lines.length > 0)?lines[lines.length - 1]:"";
//    }
//    return filesystem.readFile(this.getFileName(), true);
//  },

  getFile: function(tail) {
    var fileName = this.getFileName();
    if (tail) {
      var lines = filesystem.readFile(fileName, true);
      if (lines && lines.length > 1) {
        lines = [ lines[0], lines[lines.length - 1] ];
        filesystem.writeFile(fileName, lines); // Speed up!
        return lines[1];
      }
      return "";
    }
    return filesystem.readFile(fileName, true);
  },

  showProps: function() {
    if (!speedFan.columns) {
      this.refreshLogs();
      if (!speedFan.columns) {
        var msg = tr("SpeedFan log file '%s' not found.\n\nChoose another folder in Informer Preferences and/or make sure that\nSpeedFan is running, logging is enabled and headers are added.")
        msg = msg.replace("%s", this.getFileName());
        alert(msg);
        showWidgetPreferences();
        return;
      }
    }
    return this.parent();
  },

  getHint: function() {
    var text;
    var v = "--";
    try {
      var vMin = this.valueMin;
      var vMax = this.valueMax;
      var v = speedFan.values[this.colIdx];
      if (this.unitProp.value == sfuFahrenheit) {
        try {
          vMin = 32 + Math.round(Number(vMin) * 1.8);
          vMax = 32 + Math.round(Number(vMax) * 1.8);
          v    = 32 + Math.round(Number(v) * 1.8);
        } catch (e) {};
      }
      if (this.colIdx != -1 && speedFan.columns.length != 0) {
        text = speedFan.columns[this.colIdx] + ": " + v + this.unitText 
             + "   (" + vMin + this.unitText + ".." + vMax + this.unitText + ")";
      } else {
        text = "-- N/A --";
      }
    } catch (e) {
      log('Error: ' + e);
      text = "n/a";
    }
    return text;
  },

  getValue: function() {
    if (this.colIdx < 1) {
      this.refreshLogs();
    }
    
    // Global caching of the last values for all SpeedFan sensors.
    var now = new Date();
    if (now.getTime() - speedFan.date.getTime() >= 1000 * this.intervalProp.value) {
      try {
        var lf = this.getFile(true);
        if (lf) {
          var values = lf.split("\t");
          if (values.length == speedFan.columns.length) {
            var ts = Number(values[0]);
            if (isNaN(ts) || (ts == this.timeStamp)) {
              if (!this.timeStampChanged) {
                speedFan.values = null;
                return "--";
              }
              this.timeStampChanged = false;
            } else {
              speedFan.date = now;
              this.timeStamp = ts;
              this.timeStampChanged = true;
              speedFan.values = values;
            }
          }
        }
      } catch (e) {
        log("  Couldn't get new values from log file: " + e);
        speedFan.values = null;
      }
    }

    // Process current value
    try {
      var v = parseFloat(speedFan.values[this.colIdx]);
      if (v < this.valueMin) {
        this.valueMin = v;
      }
      if (v > this.valueMax) {
        this.valueMax = v;
      }
      var vLimit = (this.unitProp.value == sfuFahrenheit) ? 32 + Math.round(Number(v) * 1.8) : v;
      // Swap
      var vMin = parseFloat(this.limitMinProp.value);
      var vMax = parseFloat(this.limitMaxProp.value);
      if (vMin > vMax) {
        var tmp = vMax;
        vMax = vMin;
        vMin = tmp;
      }
      if ((vLimit < vMin) /*|| (vLimit > vMax)*/) {
        return "--";
      }
      return v;
    } catch (e) {
      log("  Couldn't process new value: " + e);
    }
    return "--";
  },

  updateValue: function() {
    if (this.value) {
      var v = this.getValue();
      if (v == "--") {
        this.value.data = "--";
        dockObj.align();
	return;
      }  
      if (!isNaN(v) && (this.unitProp.value == sfuFahrenheit)) {
        v = 32 + Math.round(Number(v) * 1.8);
      }
      // Swap
      var vMin = parseFloat(this.limitMinProp.value);
      var vMax = parseFloat(this.limitMaxProp.value);
      if (vMin > vMax) {
        var tmp = vMax;
        vMax = vMin;
        vMin = tmp;
      }
      var v2 = v;
      if (v2 > vMax) v2 = vMax;
      v2 = (isNaN(v2) ? 0 : Number(v2) - vMin) / (vMax - vMin);
      
      this.bar.height = Math.round(v2 * this.barBk.height);
      this.bar.vOffset = this.barBk.vOffset + this.barBk.height - this.bar.height;
      if (this.colorizeProp.isDynamic) {
        this.bar.colorize = this.colorizeProp.colorize(Math.abs(v2));
      }
      
      var oldWidth = this.value.width;
      if (isNaN(v)) {
        v = "--";
      }
      this.value.data = v + this.unitText;
      if ((this.value.width + 8) > oldWidth) {
        this.value.hOffset = this.value.width + 8;
        oldWidth = this.value.width;
      }
      
      //print(new Date().valueOf() + " - " + this.alertTime.valueOf() + " = " + (new Date().valueOf() - this.alertTime.valueOf()));
      
      if ((this.alertProp.value > 0) && (v >= vMax) && ((new Date().valueOf() - this.alertTime.valueOf()) > (this.alertProp.value * 60000) ) ) {
        notification.notify({title: "SpeedFan: " + this.titleProp.value, text: "Critical value %s!".sprintf(v + this.unitText), color: "#FF0000", titleColor: "#FFFFFF", textColor: /*"#FFE5D2"*/"#FFF4EC"});
        this.alertTime = new Date();
      }
      
      dockObj.align();
    }
  },

  refreshLogs: function() {
    try {
      var lf = this.getFile(false);
      speedFan.columns = lf[0].split("\t");
      this.setColName();
    } catch (e) {
      this.colIdx = -1;
      speedFan.columns = null;
    }
  },

  setColName: function() {
    this.colIdx = -1;
    if (speedFan.columns) {
      for (var i = 1; i < speedFan.columns.length; i++) {
        if (speedFan.columns[i] == this.colNameProp.value) {
          this.colIdx = i;
          break;
        }
      }
    }
    this.setTitle();
  },

  setUnit: function() {
    if (this.unitProp.value == sfuFahrenheit) {
      this.unitText = "°F";
    } else if (this.unitProp.value == sfuRPM) {
      this.unitText = tr("rpm");
    } else if (this.unitProp.value == sfuVolt) {
      this.unitText = "V";
    } else {
      this.unitText = "°C";
    }
  },

  setTitle: function() {
    if (this.label) {
      this.label.data = this.meterName();
      dockObj.align();
    }
  }
});

if (system.platform == "windows") {
  dockFactory.register({name: tr("SpeedFan Meter"),
                        className: "DockSpeedFan",
                        classDef: DockSpeedFan,
                        category: tr("System"),
                        description: tr("Show SpeedFan measurements")
                        });
}