
var DockStandardMail = DockGenericMail.extend({
  initialize: function() {
    this.parent(tr("IMAP4/POP3 Mail"));
    this.className = "DockStandardMail";
    
    this.imapProp = new ListProp("imap4", "Protocol:", true, 
                                 ["IMAP4", "POP3"], [1, 0]);
    this.sslProp = new BoolProp("ssl", "Use IMAP4 or POP3 over SSL connection",
                                false, "");

    this.agentLimitProp = new BoolProp("limit", "Retrieve only displayed number of message headers.", true, "");
                                      
    this.props = [this.titleProp, this.serverProp, this.imapProp, this.sslProp, 
                  this.usernameProp, this.passwordProp, this.intervalProp, 
                  this.customIconProp, this.rowLimitProp, this.agentLimitProp,
                  this.widthLimitProp, this.notifyProp];
  },

  getName: function() {
    return tr("IMAP4/POP3 Mail Checker");
  },

  doOpenClient: function() {
    // Open Default Mail Client
    dockObj.openMail(this.usernameProp.value);
  },

  getIcon: function() {
    return "Resources/Sensors/Mail-Rotated.png";
  },

  parseMailResult: function(data) {
    this.finishChecking();
    if (data.length && (data.indexOf('<') != -1)) {
      var a = data.match(/<error><\!\[CDATA\[\s*([a-zA-Z0-9_#\., \r\n]*)\s*\]\]><\/error>/mi);
      if (a) {
        var msg = a[1];
        if (msg.match(/Connection refused/i)) {
          msg = "Connection time-out.";
        }
        this.setError(msg);
        return;
      }
      
      this.data = data;
      this.lastUpdate = new Date();
      // save cache
      if (this.cache != "") {
        filesystem.writeFile(this.cache, data);
      }
      this.makeElementInactive(this.icon, false);
      this.updateValue();
    } else {
      this.setError("No data received.");
    }
  },

  doFetch: function() {
    this.parent();
    var limit = 0;
    if (this.agentLimitProp.value) {
      limit = this.rowLimitProp.value;
    }
    if (system.platform == "windows") {
      //runCommandInBg(("Applications/MailCommander.exe -p:" + (this.imapProp.value?"IMAP":"POP") + " -s:" + this.serverProp.value + " -u:" + this.usernameProp.value + " -pw:" + this.passwordProp.value + ((limit > 0)?(" -l:" + limit):"") + " -q" + (this.sslProp.value?" -SSL":"")).replace(/([\&\|\(\)]+)/g, "\^$1"), "mail_" + this.name);
      runCommandInBg("cmd.exe /c start /B /D\"" + dockObj.folders["Konfabulator"] + "\" " + ("Applications/MailCommander.exe -p:" + (this.imapProp.value?"IMAP":"POP") + " -s:" + this.serverProp.value + " -u:" + this.usernameProp.value + " -pw:" + this.passwordProp.value + ((limit > 0)?(" -l:" + limit):"") + " -q" + (this.sslProp.value?" -SSL":"")).replace(/([\&\|\(\)]+)/g, "\^$1"), "mail_" + this.name);
    } else {
      runCommandInBg("Applications/MailCommander.pl " + (this.imapProp.value?"IMAP4":"POP3") + " " + this.serverProp.value + " " + this.usernameProp.value + " " + this.passwordProp.value + ((limit > 0)?(" " + limit):"") + (this.sslProp.value?" SSL":""), "mail_" + this.name);
    }
  },

  genCacheName: function() {
    return "stdmail-" + this.usernameProp.value + ".xml";
  },

  setError: function() {
    if (arguments.length > 0) {
      this.parent(arguments[0]);
    } else {
      this.parent();
    }
    if (this.errorMsg == "") {
      if (this.oldData != "") {
        try {
          /*var old = XMLDOM.parse(this.oldData);
          try {
            var total = old.evaluate("string(inbox/stat/total)");
            this.setStatus(total ? Number(total) : "n/a");
          } finally {
            old = null;
          }*/
          this.setStatus(this.total ? Number(this.total) : "n/a");
        } catch(e) {
          this.setStatus("n/a");
        }
      }
    }
  }
});

dockFactory.register({name: tr("IMAP4/POP3 Mail Checker"),
                      className: "DockStandardMail",
                      classDef: DockStandardMail,
                      category: tr("Internet"),
                      description: tr("Check mail via IMAP4/POP3")
                      });
