
/* ------------------------------------------------------------------------ */

var FontProp = StringProp.extend({
  initialize: function() {
    this.parent("font");
    this.value = "Arial";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Font:");
    f.type = "font";
    f.value = this.value;
    f.defaultValue = "Arial, Helvetica";
    f.description = tr("Font used to display sensor");
    ff.push(f);
  },
  
  set: function(value) {
    if (value) {
      this.value = value;
    }
  }
});

/* ------------------------------------------------------------------------ */

var FontStyleProp = BaseProp.extend({
  initialize: function() {
    this.parent("fontstyle");
    this.value = "";
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Font Style:");
    f.type = "popup";
    f.option = [tr("Normal"), 
                tr("Bold"), 
                tr("Italic"), 
                tr("Bold, Italic")];
    f.optionValue = ["", "bold", "italic", "bold, italic"];
    f.value = this.value;
    ff.push(f);
  },
  
  set: function(value) {
    if (value != null) {
      this.value = value;
    }
  }
});

/* ------------------------------------------------------------------------ */

var FontSizeProp = BaseProp.extend({
  initialize: function() {
    this.parent("fontsize");
    this.value = 13;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Font Size:");
    f.type = "slider";
    f.ticks = 8;
    f.minLength = 9;
    f.maxLength = 26; // error in YWE401: limit ticks to (max-min) instead of (max-min+1)
    f.tickLabel = ["9", "12", "14", "16", "18", "20", "22", "25"];
    f.description = tr("Set font size in points.");
    f.value = this.value;
    ff.push(f);
  },
  
  set: function(value) {
    if (value) {
      this.value = parseInt(value);
      if (this.value > 25) this.value = 25; // error in YWE401, see above
    }
  }
});

/* ------------------------------------------------------------------------ */

var DockText = DockBase.extend({
  initialize: function() {
    this.parent();
    this.className = "DockText";
    
    this.textProp = new TextProp("text", "Title:", this.getTitle(),
                                 "Text to display in sensor");
    this.fontProp = new FontProp();
    this.fontStyleProp = new FontStyleProp();
    this.fontSizeProp = new FontSizeProp();
    this.shadowProp = new BoolProp("shadow", "Display shadow", false, "");
    this.props = [this.textProp, this.fontProp, this.fontStyleProp, 
                  this.fontSizeProp, this.shadowProp];
  },

  destroy: function() {
    if (this.parent()) {
      this.label = null;
      this.labelShadow = null;
    }
  },

  applyFont: function() {
    this.label.style.fontFamily = $font(this.fontProp.value);
    this.label.style.fontWeight = this.fontStyleProp.value.test("bold")?"bold":"normal";
    this.label.style.fontStyle = this.fontStyleProp.value.test("italic")?"italic":"normal";
    this.label.style.fontSize = this.fontSizeProp.value + "px";
  },

  alignText: function() {
    this.label.vOffset = 1 + Math.round((dockObj.h - this.label.height) / 2);
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    if (this.label) {
      this.setText();
      this.applyFont();
      this.alignText();
      dockObj.align();
    }
    this.setShadow();
  },

  getName: function() {
    return tr("Text");
  },

  getTitle: function() {
    return "???";
  },

  updateColor: function() {
    this.parent();
    if (this.label) { 
      this.label.style.color = this.color;
    }
    this.setShadow();
  },

  createControls: function() {
    if (this.parent()) {
      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.data = this.textProp.value;
      this.label.anchorStyle = "topLeft";
      this.label.hOffset = 1;
      this.label.vOffset = 0;
      
      this.applyFont();
      this.label.style.color = "#FFFFFF";

      this.labelShadow = new Shadow();
      this.labelShadow.hOffset = 1;
      this.labelShadow.vOffset = 1;
      this.setShadow();
      
      this.alignText();
    }
    return (this.frame != null);
  },

  setText: function() {
    this.label.data = this.textProp.value;
  },

  setShadow: function() {
    if (!this.label) return;
    
    if (this.shadowProp.value) {
      if (this.labelShadow) {
        this.labelShadow.color = Color.isLightColor(Color.HexToRGB(this.color)) 
                               ? "#000000" : "#FFFFFF";
      }
      this.label.shadow = this.labelShadow;
    } else {
      this.label.shadow = null;
    }
  }
});

dockFactory.register({name: tr("Text"),
                      className: "DockText",
                      classDef: DockText,
                      category: tr("General"),
                      description: tr("Text block")
                      });
