
/* ------------------------------------------------------------------------ */

var ChimeProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("Never"), "(-", tr("Every Hour"), 
                    tr("Every Half Hour"), tr("Every Fifteen Minutes")];
    var optValues = [0, 0, 60, 30, 15];
    this.parent("chime", "Chime:", 0, optTexts, optValues, 
              "If you would like a chime to happen at regular intervals, please choose how often from the popup above.");
    this.className = "ChimeProp";
  }
});

/* ------------------------------------------------------------------------ */

var DockTime = DockBaseTimer.extend({
  initialize: function() {
    this.parent(1.0);
    this.className = "DockTime";
    this.maxWidth = 0;
    
    this.euroTimeProp = new BoolProp("24hours", "Use 24 hour time", true,
                                     "Display the hours in 24 hour format instead of 12 hour am/pm.");
    this.showSecondsProp = new BoolProp("seconds", "Display seconds", false,
                                        "Display the seconds.");
    this.shadowProp = new BoolProp("shadow", "Display shadow", false,
                                   "Display the text with shadow.");
    this.chimeProp = new ChimeProp();
    
    this.props = [this.euroTimeProp, this.showSecondsProp, 
                  this.shadowProp, this.chimeProp];
  },

  getName: function() {
    return tr("Time");
  },

  updatePrefs: function() {
    this.parent();
    if (this.time) { $$font(this.time, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.second) { $$font(this.second, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.hour) { $$font(this.hour, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.minute) { $$font(this.minute, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.note) { $$font(this.note, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    this.updateProps();
  },

  updateColor: function() {
    this.parent();
    if (this.time) { 
      this.time.style.color   = this.color;
      this.second.style.color = this.color;
      this.hour.style.color   = this.color;
      this.minute.style.color = this.color;
      this.note.style.color   = this.color;
      this.updateShadow();
    }
  },

  createControls: function() {
    if (this.parent()) {
      this.time = new Text(mainWindow);
      this.time.data = "00:00";
      this.time.vOffset = 19;
      this.time.style.fontFamily = "Arial, Helvetica";
      this.time.style.fontSize = "15px";
      this.time.style.fontWeight = "bold";
      this.time.style.color = "#FFFFFF";
      
      this.second = new Text(mainWindow);
      this.second.data = "00";
      this.second.anchorStyle = "topLeft";
      this.second.style.fontFamily = "Arial, Helvetica";
      this.second.style.fontSize = "11px";
      this.second.style.color = "#FFFFFF";

      this.hour = new Text(mainWindow);
      this.hour.data = "0";
      this.hour.vOffset = 19;
      this.hour.style.fontFamily = "Arial, Helvetica";
      this.hour.style.fontSize = "15px";
      this.hour.style.fontWeight = "bold";
      this.hour.style.color = "#FFFFFF";

      this.minute = new Text(mainWindow);
      this.minute.data = "00";
      this.minute.style.fontFamily = "Arial, Helvetica";
      this.minute.style.fontSize = "11px";
      this.minute.style.color = "#FFFFFF";

      this.note = new Text(mainWindow);
      this.note.data = "am";
      this.note.vOffset = 14;
      this.note.anchorStyle = "topLeft";
      this.note.style.fontFamily = "Arial, Helvetica";
      this.note.style.fontSize = "9px";
      this.note.style.color = "#FFFFFF";

      this.updateProps();
    }
    return (this.frame != null);
  },

  updateProps: function(loaded) {
    if (!this.time) return;
    
    this.updateShadow();
    
    if (this.showSecondsProp.value) {
      this.hour.removeFromSuperview();
      this.minute.removeFromSuperview();
      this.note.removeFromSuperview();
      
      this.frame.addSubview(this.time);
      this.time.shadow = this.shadowObj;
      this.frame.addSubview(this.second);
      this.second.hOffset = this.time.width + 2;
      this.second.vOffset = this.euroTimeProp.value?10:4;
      this.second.shadow = this.shadowObj;
      this.frame.addSubview(this.note);
      this.note.hOffset = this.time.width + 2;
      this.note.shadow = this.shadowObj;
      this.note.visible = !this.euroTimeProp.value;
      
      this.second.data ="00";
      this.maxWidth = this.second.width;
      this.second.data = "";
      this.note.data = "am";
      if (this.maxWidth < this.note.width) {
        this.maxWidth = this.note.width;
      }
      this.note.data = "pm";
      if (this.maxWidth < this.note.width) {
        this.maxWidth = this.note.width;
      }
      this.note.data = "";
      
      this.updateSeconds();

      this.timer = new Timer();
      this.timer.interval = 0.5;
      this.timer.onTimerFired = this.updateSeconds.bind(this);
      this.timer.ticking = true;
    } else {
      this.time.removeFromSuperview();
      this.second.removeFromSuperview();
      this.note.removeFromSuperview();
      
      this.frame.addSubview(this.hour);
      this.hour.shadow = this.shadowObj;
      this.frame.addSubview(this.minute);
      this.minute.hOffset = this.hour.width + 2;
      this.minute.vOffset = this.euroTimeProp.value?16:14;
      this.minute.shadow = this.shadowObj;
      this.frame.addSubview(this.note);
      this.note.hOffset = this.hour.width + 2;
      this.note.shadow = this.shadowObj;
      this.note.visible = !this.euroTimeProp.value;

      this.minute.data ="00";
      this.maxWidth = this.minute.width;
      this.minute.data = "";
      this.note.data = "am";
      if (this.maxWidth < this.note.width) {
        this.maxWidth = this.note.width;
      }
      this.note.data = "pm";
      if (this.maxWidth < this.note.width) {
        this.maxWidth = this.note.width;
      }
      this.note.data = "";
      
      this.updateValue();

      this.timer = new Timer();
      this.timer.interval = 10.0;
      this.timer.onTimerFired = this.updateValue.bind(this);
      this.timer.ticking = true;
    }
    dockObj.align();
  },
  
  updateShadow: function() {
    if (this.shadowProp.value) {
      this.shadowObj = new Shadow();
      this.shadowObj.hOffset = 1;
      this.shadowObj.vOffset = 1;
      this.shadowObj.color = Color.isLightColor(Color.HexToRGB(this.color)) 
                           ? "#000000" : "#FFFFFF";
    } else {
      this.shadowObj = null;
    }
    this.time.shadow   = this.shadowObj;
    this.second.shadow = this.shadowObj;
    this.hour.shadow   = this.shadowObj;
    this.minute.shadow = this.shadowObj;
    this.note.shadow   = this.shadowObj;
  },
  
  chime: function() {
    var now = new Date();
    if (this.chimeProp.value != 0) {
      if (now.getMinutes() % this.chimeProp.value == 0) {
        play("Resources/Sensors/Time/chime.mp3", 1);
      }
    }
  },

  updateSeconds: function() {
    if (!this.time) return;
    var now = new Date();
    var s = now.toString('%S');
    if (this.second.data != s) {
      var oldTimeWidth = this.time.width; 
      var widthChanged = false;
      if (this.euroTimeProp.value) {
        var t = now.toString('%k:%M');
      } else {
        var t = now.toString('%l:%M');
        var oldNoteWidth = this.note.width;
        this.note.data = now.toString('%P');
        widthChanged = (oldNoteWidth != this.note.width);
      }
      this.second.data = s;
      this.second.width = this.maxWidth;
      if (this.time.data != t) {
        this.time.data = t;
        if (oldTimeWidth != this.time.width) {
          this.second.hOffset = this.time.width + 2;
          this.note.hOffset = this.time.width + 2;
          widthChanged = true;
        }
        if (widthChanged) {
          dockObj.align();
        }
        this.chime();
      }
    }
  },

  updateValue: function() {
    if (!this.hour) return;
    var now = new Date();
    var m = now.toString('%M');
    if (this.minute.data != m) {
      var oldHourWidth = this.hour.width;
      var widthChanged = false;
      if (this.euroTimeProp.value) {
        this.hour.data = now.toString('%k');
      } else {
        this.hour.data = now.toString('%l');
        var oldNoteWidth = this.note.width;
        this.note.data = now.toString('%P');
        widthChanged = (oldNoteWidth != this.note.width);
      }
      this.minute.data = m;
      this.minute.width = this.maxWidth;
      if (oldHourWidth != this.hour.width) {
        this.minute.hOffset = this.hour.width + 2;
        this.note.hOffset = this.hour.width + 2;
        widthChanged = true;
      }
      if (widthChanged) {
        dockObj.align();
      }
      this.chime();
    }
  },

  getHint: function() {
    var now = new Date();
    return now.toString(tr("%A, %B %e, %Y"));
  },

  doClick: function() {
    this.parent();
    if (system.platform == "windows") {
      runCommandInBg("rundll32.exe shell32.dll,Control_RunDLL timedate.cpl", "datetimeApplet");
    }
  }
});

dockFactory.register({name: tr("Time"),
                      className: "DockTime",
                      classDef: DockTime,
                      category: tr("Info"),
                      description: tr("Show current time")
                      });
