
var DockTrashMeter = DockBaseTimer.extend({
  initialize: function() {
    this.parent();
    this.className = "DockTrashMeter";

    this.captureDragDrop = true;
    this.image = null;
    
    this.titleProp = new TextProp("title", "Title:", "",
                                  "Leave blank to system name.");
    this.imgEmptyProp = new FileProp("image-empty", "Image (empty):",
                                     "Image to display empty state, maximum height 24 pix.\n\nLeave blank to default icon.");
    this.imgFullProp = new FileProp("image-full", "Image (full):",
                                    "Image to display full state, maximum height 24 pix.\n\nLeave blank to default icon.");
    this.opacityProp = new OpacityProp("opacity", "Opacity:", 255, 50);
    this.colorizeProp = new ColorizeProp("colorize", "Color:");

    this.props = [this.titleProp, this.imgEmptyProp, this.imgFullProp, 
                  this.opacityProp, this.colorizeProp];
  },

  destroy: function() {
    if (this.parent()) {
      this.label = null;
      this.image = null;
      this.value = null;
    }
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    if (this.label) {
      this.label.data = this.titleProp.value ? this.titleProp.value : this.getName();
    }
    this.updateImage();
    if (this.image) {
      this.image.opacity = this.opacityProp.value;
      this.image.colorize = this.colorizeProp.colorize();
    }
  },

  getName: function() {
    return tr((system.platform == "windows")?"Recycle Bin":"Trash");
  },

  updatePrefs: function() {
    this.parent();
    if (this.label) { $$font(this.label, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.value) { $$font(this.value, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    dockObj.align();
  },

  updateColor: function() {
    this.parent();
    if (this.label) { this.label.style.color = this.color; }
    if (this.value) { this.value.style.color = this.color; }
  },

  createControls: function() {
    if (this.parent()) {
      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.hOffset = 26;
      this.label.vOffset = 11;
      this.label.data = this.getName();
      this.label.style.fontFamily = "Arial, Helvetica";
      this.label.style.fontSize = "9px";
      this.label.style.fontWeight = "bold";
      this.label.style.color = this.color;

      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.missingSrc = "Resources/Sensors/Trash-Empty.png";
      this.image.hOffset = 0;

      this.value = new Text(mainWindow);
      this.frame.addSubview(this.value);
      this.value.hOffset = 26;
      this.value.vOffset = 21;
      this.value.data = "";
      this.value.style.fontFamily = "Arial, Helvetica";
      this.value.style.fontSize = "9px";
      this.value.style.fontWeight = "bold";
      this.value.style.color = this.color;

      this.updateProps();
      this.updateValue();
    }
    return (this.frame != null);
  },

  updateValue: function() {
    if (!this.value) return;
    var ri = filesystem.getRecycleBinInfo();
    if (ri.numItems <= 0) {
      this.value.data = tr("empty");
    } else {
      this.value.data = ri.numItems + tr(" item" + ((ri.numItems != 1)?"s":""));
    }
    this.updateImage();
    dockObj.align();
  },

  updateImage: function() {
    if (this.image) {
      var ri = filesystem.getRecycleBinInfo();
      if (ri.numItems > 0) {
        this.image.src = (this.imgFullProp.value != "")?this.imgFullProp.value:"Resources/Sensors/Trash-Full.png";
      } else {
        this.image.src = (this.imgEmptyProp.value != "")?this.imgEmptyProp.value:"Resources/Sensors/Trash-Empty.png";
      }
      this.image.vOffset = Math.round((25 - this.image.srcHeight) / 2);

      if (this.label) {
        this.label.hOffset = this.image.srcWidth + 2;
      }
      if (this.value) {
        this.value.hOffset = this.image.srcWidth + 2;
      }
    }
  },

  openTrash: function() {
    if (system.platform == "windows") {
      runCommand("start explorer ::{645FF040-5081-101B-9F08-00AA002F954E}"); // Recycle Bin
    } else {
      filesystem.openTrash();
    }
  },

  sensorMenuItems: function(items) {
    this.parent();

    var item = new MenuItem();
    item.title = (system.platform == "windows")?"-":"(-";
    items.push(item);

    var item = new MenuItem();
    item.title = tr((system.platform == "windows")?"Open Recycle Bin":"Open Trash");
    //item.onSelect = "filesystem.openTrash()";
    item.onSelect = this.openTrash.bind(this);
    items.push(item);

    var item = new MenuItem();
    item.title = tr((system.platform == "windows")?"Empty Recycle Bin":"Empty Trash");
    item.onSelect = function() {
      filesystem.emptyTrash();
      if (system.platform == 'macintosh') play('/System/Library/Components/CoreAudio.component/Contents/Resources/SystemSounds/finder/empty\ trash.aif');
      this.updateValue();
    }.bind(this);
    items.push(item);
  },

  getHint: function() {
    var ri = filesystem.getRecycleBinInfo();
    this.updateValue();
    return this.getName() +
      tr("\n   Items: ") + ri.numItems +
      tr("\n   Space used: ") + trBytesToUI(ri.size);
  },

  doDragDrop: function() {
    data = system.event.data;
    var type = data.shift();
    var item = "";
    if (type == "filenames") {
      for (var i = 0; i < data.length; i++) {
        item = data[i].substring( ((data[i].lastIndexOf("/") != -1)?data[i].lastIndexOf("/") + 1:0), data[i].length);
        if (filesystem.isDirectory(data[i])) item += "/*.*";
        if (this.trashed.length == this.maxItemsInHint) this.trashed.pop();
        this.trashed.unshift(item);
      }
      filesystem.moveToTrash(data);
      this.updateValue();
    } 
  },

  doClick: function() {
    this.parent();
    filesystem.openTrash();
  }
});

dockFactory.register({name: tr((system.platform == "windows")?"Recycle Bin":"Trash"),
                      className: "DockTrashMeter",
                      classDef: DockTrashMeter,
                      category: tr("System"),
                      description: tr("Show Recycle Bin/Trash Can and its capacity")
                      });
