
include("Sensors/TurboFolderPopup.js");

var TurboFolder = DockBase.extend({
  initialize: function() {
    this.parent();
    this.className = "TurboFolder";
    this.image = null;

    this.tip = null;

    this.folderProp = new FolderProp("folder", "Folder:");
    this.imageProp = new FileProp("image", "Image:", 
                                  "Image to display, maximum height 21 pix.\nLeave blank to application icon.",
                                  [".ico", ".png", ".gif", ".jpg", ".jpeg"]);
    this.textProp = new TextProp("text", "Title:", "", "Leave blank for none.");
    this.colorizeProp = new ColorizeProp("colorize", "Color:");
    this.opacityProp = new OpacityProp("opacity", "Opacity:", 255, 50);

    this.widthProp = new BaseProp("popup-width", 160);
    this.widthProp.defaultValue = 160;
    this.widthProp.hidden = true;
    this.heightProp = new BaseProp("popup-height", 250);
    this.heightProp.defaultValue = 250;
    this.heightProp.hidden = true;
    
    this.props = [this.folderProp, this.textProp, this.imageProp, this.colorizeProp,
                  this.opacityProp, this.widthProp, this.heightProp];

    this.popup = null;

    this.lockTimer = null;
    this.lockedPopup = false;

    //this.items = [];
  },

  destroy: function() {
    if (this.parent()) {
      this.image = null;

      delete this.lockTimer;
      this.lockTimer = null;
    }
  },

  createPopup: function() {
    if (this.folderProp.value == "") return null;
    return new TurboFolderPopup(this);
  },

  getFolderTitle: function() {
    return (this.textProp.value != "")?this.textProp.value:filesystem.getDisplayName(this.folderProp.value);
  },

  getFolderImage: function() {
    return ( (this.imageProp.value != "")?this.imageProp.value:"Resources/Sensors/Folder.png" );
  },

  updateProps: function(loaded) {
    if (!this.image) return;
    this.image.opacity = this.opacityProp.value;
    this.image.colorize = this.colorizeProp.colorize();
    this.image.src = this.getFolderImage();
  },

  getName: function() {
    return tr("TurboFolder");
  },

  setFolder: function(value) {
    this.folderProp.value = value;
  },

  /*getWidth: function() {
    if (this.frame) {
      return this.frame.width + 2;
    }
    return 0;
  },*/

  createControls: function() {
    if (this.parent()) {
      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.src = this.getFolderImage();
      this.image.hOffset = 4;
      this.image.vOffset = 3;
      this.image.width = 21;
      this.image.height = 21;
      this.image.tracking = "rectangle";
      this.image.useFileIcon = false;
      this.image.zOrder = 2;

      this.bk = new Image(mainWindow);
      this.frame.addSubview(this.bk);
      this.bk.src = "Resources/Hint/Fill.png";
      this.bk.hOffset = 0;
      this.bk.vOffset = 0;
      this.bk.width = 28;
      this.bk.height = dockObj.h;
      this.bk.colorize = "#000000";
      this.bk.opacity = 200;
      this.bk.zOrder = 1;
      this.bk.visible = false;

      /*this.shadow = new Image(mainWindow);
      this.frame.addSubview(this.shadow);
      this.shadow.src = "Resources/Hint/RightShadow.png";
      this.shadow.hOffset = this.bk.hOffset + this.bk.width;
      this.shadow.vOffset = 0;
      this.shadow.width = 9;
      this.shadow.height = dockObj.h;
      this.shadow.opacity = 100;
      this.shadow.zOrder = 1;
      this.shadow.visible = false;*/

      this.lockTimer = new Timer(mainWindow);
      this.lockTimer.interval = 0.5;
      this.lockTimer.onTimerFired = this.unlock.bind(this);
      this.lockTimer.ticking = false;

      this.frame.onDragEnter = this.beginDrop.bind(this);
      this.frame.onDragDrop = this.dragDrop.bind(this);
      this.frame.onDragExit = this.endDrop.bind(this);

      this.updateProps();
    }
    return (this.frame != null);
  },

  makeURLs: function(links) {
    //print('Link to: ' + links);
    for (var i = 0; i < links.length; i++) {
      dockObj.makeLink(links[i], this.folderProp.value);
    }
  },

  moveFiles: function(files) {
    //print('Move: ' + files);
    for (var i = 0; i < files.length; i++) {
      filesystem.move(files[i], this.folderProp.value);
    }
  },

  copyFiles: function(files) {
    //print('Copy: ' + files);
    for (var i = 0; i < files.length; i++) {
      filesystem.copy(files[i], this.folderProp.value);
    }
  },

  beginDrop: function() {
    //print('begin drop');
    this.bk.visible = true;

    if (!this.tip) {
      this.tip = new FloatingHint();
    }
    //var msg = ((system.event.modifiers.indexOf("option") == -1)?tr('Move'):tr('Copy')) + ' ';
    //var mode = (system.event.modifiers.indexOf("option") == -1)?tr('Move'):tr('Copy');
    var msg = (!system.event.altKey?tr('Move'):tr('Copy')) + ' ';
    var mode = !system.event.altKey?tr('Move'):tr('Copy');
    var src = "";
    var dest = this.getFolderTitle();
    var items = system.event.data;
    var type = items.shift();
    if (type == "urls") {
      if (items.length > 1) {
        msg = tr("Create shortcuts for %s URLs in '%s'").sprintf(items.length, dest);
      } else {
        src = items[0];
        if (src.length > 100) {
          src = src.substring(0, 97) + "...";
        }
        msg = tr("Create shortcut to '%s' in '%s'").sprintf(src, dest);
      }
    } else if (type == "filenames") {
      if (items.length > 1) {
        msg = tr("%s %s folders/files to '%s'").sprintf(mode, items.length, dest);
      } else {
        msg = tr("%s %s '%s' to '%s'").sprintf(mode, tr(filesystem.isDirectory(items[0])?'folder':'file'), filesystem.getDisplayName(items[0]), dest);
      }
    }

    this.tip.setText(msg);
    this.tip.show(system.event.screenX);
  },

  dragDrop: function() {
    var items = system.event.data;
    var type = items.shift();
    if (type == "urls") {
      this.makeURLs(items);
    } else if (type == "filenames") {
      //if (system.event.modifiers.indexOf("option") == -1) {
      if (!system.event.altKey) {
        this.moveFiles(items);
      } else {
        this.copyFiles(items);        
      }
    }
  },

  endDrop: function() {
    //print('end drop');
    this.bk.visible = false;
    if (this.tip) {
      this.tip.hideNow();
      delete this.tip;
      this.tip = null;
    }
  },

  getTitle: function() {
    return tr("TurboFolder");
  },

  getHint: function() {
    return ( ((this.popup == null) && !this.lockedPopup) ? ( (this.getFolderTitle()) ) : "" );
  },

  popupShow: function() {
    /*this.bk.visible = true;
    this.shadow.visible = true;*/
    this.image.visible = false;
  },

  popupHide: function() {
    /*this.bk.visible = false;
    this.shadow.visible = false;*/
    this.image.visible = true;

    delete this.popup;
    this.popup = null;

    this.lockedPopup = true;
    this.lockTimer.ticking = true;
  },

  hidePopup: function() {
    if (this.popup) {
      this.popup.hide();
    }
  },

  /*loseFocus: function() {
    this.parent();
    
    this.hidePopup();
  },*/

  doClick: function() {
    this.parent();

    if (this.lockedPopup) return;

    if (!this.popup) {
      this.popup = this.createPopup();
      if (this.popup == null) return;
      this.popup.onShow = this.popupShow.bind(this);
      this.popup.onHide = this.popupHide.bind(this);

      this.popup.show();
    } else {
      this.hidePopup();
    }
  },

  unlock: function() {
    this.lockTimer.ticking = false;
    this.lockedPopup = false;
  },

  sensorMenuItems: function(items) {
    this.parent(items);

    var item = new MenuItem();
    item.title = (system.platform == "windows")?"-":"(-";
    items.push(item);

    var item = new MenuItem();
    item.title = tr("Open folder");
    item.onSelect = this.doOpen.bind(this);
    items.push(item);
  },

  doOpen: function() {
    filesystem.open(this.folderProp.value);
  }
});

dockFactory.register({name: tr("TurboFolder"),
                      className: "TurboFolder",
                      classDef: TurboFolder,
                      category: tr("System"),
                      description: tr("Display list of folder's files and folders")
                      });
