
var Uptime = new Class({

  initialize: function(value) {
    this.value = {d: 0, h: 0, m: 0, s: 0};
    if (typeof(value) == "object") {
      this.value = value;
    }
    if (typeof(value) == "string") {
      this.fromString(value);
    }
  },

  toString: function() {
    return this.value.d + ":" + this.value.h + ":" + this.value.m + ":" + this.value.s;
  },

  fromString: function(str) {
    var temp = str.split(":");
    if (temp.length == 4) {
      this.value.d = parseInt(temp[0]);
      this.value.h = parseInt(temp[1]);
      this.value.m = parseInt(temp[2]);
      this.value.s = parseInt(temp[3]);
    }
  },

  fromSeconds: function(seconds) {
    this.value = {d: 0, h: 0, m: 0, s: 0};
    this.value.s = seconds % 60;
    this.value.m = Math.floor(seconds / 60);
    this.value.h = Math.floor(this.value.m / 60);
    this.value.d = Math.floor(this.value.h / 24);
    this.value.h %= 24;
    this.value.m %= 60;
  },

  empty: function() {
    return (this.value.d == 0) && (this.value.h == 0) && (this.value.m == 0) && (this.value.s == 0);
  },

  compare: function(value) {
    if (!value) return 1;
    if (this.d > value.d) return 1;
    if (this.d < value.d) return -1;
    
    if (this.h > value.h) return 1;
    if (this.h < value.h) return -1;
    
    if (this.m > value.m) return 1;
    if (this.m < value.m) return -1;
    
    if (this.s > value.s) return 1;
    if (this.s < value.s) return -1;

    return 0;
  },

  equal: function(value) {
    return (this.compare(value) == 0);
  },

  less: function(value) {
    return (this.compare(value) == 1);
  },

  greater: function(value) {
    return (this.compare(value) == -1);
  }

});

var DockUpTime = DockBaseText.extend({
  initialize: function() {
    this.updateTime = new Date();
    this.uptime = new Uptime();
    this.WBEM = null;
    this.wmi = null;
    this.parent(60.0);
    this.className = "DockUpTime";
    
    this.maxUptime = new BaseProp("max-uptime", "");
    this.textProp.defaultValue = "";
    this.fontStyleProp.value = "bold";
    this.fontSizeProp.value = 12;
    this.props = [this.textProp, this.fontProp, this.fontStyleProp, 
                  this.fontSizeProp, this.shadowProp, this.maxUptime];
  },

  getName: function() {
    return tr("Uptime");
  },

  getTitle: function() {
    return tr("Uptime") + ":";
  },

  createControls: function() {
    if (this.parent()) {
      if (system.platform == "windows") {
        this.WBEM = COM.createObject("WbemScripting.SWbemLocator.1");
        this.wmi = this.WBEM.ConnectServer(".", "root/cimv2");
      } else {
        // detect uptime under Mac OS X
        var upTimeData = runCommand("uptime");
        this.updateTime = new Date();
        //upTimeData = "0:39  up 11 mins, 2 users, load averages: 6.21 5.50 4.94";
        arrayBreaker = upTimeData.match(/(?:up).*?user(s)?/i);
        if (arrayBreaker) {
          upTime = arrayBreaker[0];
        }
        utArray = upTime.split(",");
        var u = 0;
        if (utArray[0].match(/day/)) {
          utArray[0] = utArray[0].replace(/up/g, '');
          utArray[0] = utArray[0].replace(/day/g, '');
          utArray[0] = utArray[0].replace(/s/g, '');
          utArray[0] = utArray[0].replace(/ /g, '');
          utArray[1] = utArray[1].replace(/ /g, '');
          utArray[1] = utArray[1].replace(/mins/g, '');

          if (utArray[1].match(/\:/))     {
            secs = utArray[1].split(":");
          } else {
            secs = new Array();
            secs[0] = 0;
            secs[1] = utArray[1];
          }
          u = (utArray[0] * 24 * 60 * 60) + (secs[0] * 60 * 60) + (secs[1] * 60);
        } else {
          print(utArray.join("\n"));
          utArray[0] = utArray[0].replace(/up/g, '');
          utArray[0] = utArray[0].replace(/ /g, '');
          utArray[0] = utArray[0].replace(/mins/g, '');

          if (utArray[0].match(/\:/))     {
            secs = utArray[0].split(":");
          } else {
            secs = new Array();
            secs[0] = 0;
            secs[1] = utArray[0];
          }
          u = (secs[0] * 60 * 60) + (secs[1] * 60);
        }
        this.uptime.fromSeconds(u);
      }

      this.updateValue();
    }
    return (this.frame != null);
  },

  incUptime: function() {
    var now = new Date();
    var diff_ms = now.getTime() - this.updateTime.getTime();
    var seconds = ((((this.uptime.value.d * 24 + this.uptime.value.h) * 60) + this.uptime.value.m) * 60) + this.uptime.value.s
                + Math.floor(diff_ms / 1000);
    this.uptime.fromSeconds(seconds);
    this.updateTime = new Date(now.getTime() - diff_ms % 1000);
  },

  date2String: function(uptime) {
    var longDate = false;
    if (arguments.length > 1) {
      longDate = arguments[1];
    }
    if (isNaN(uptime.d) || isNaN(uptime.h) || isNaN(uptime.m) || isNaN(uptime.s)) return "";
    if (!longDate) {
      return uptime.d + "d " + String(uptime.h).pad(2, "0") + "h " + String(uptime.m).pad(2, "0") + "m";
    }
    var s = "";
    if (uptime.d != 1) {
      s = uptime.d + tr(" days, ");
    } else {
      s = uptime.d + tr(" day, ");
    }
    if (uptime.h != 1) {
      s += uptime.h + tr(" hours, ");
    } else {
      s += uptime.h + tr(" hour, ");
    }
    if (uptime.m != 1) {
      s += uptime.m + tr(" minutes");
    } else {
      s += uptime.m + tr(" minute");
    }
    return s;
  },

  updateValue: function() {
    if (system.platform == "windows") {
      try {
        var tmp = this.wmi.Get('Win32_PerfFormattedData_PerfOS_System=@').Properties_.Item("SystemUpTime").Value;
        this.updateTime = new Date();
        this.uptime.fromSeconds(tmp);
      } catch (e) {
      } finally {
        this.incUptime();
      }
    } else {
      this.incUptime();
    }
    
    // Calculate remaining time until next minute
    var ms = this.uptime.value.m * 60 + this.uptime.value.s;
    this.timer.interval = this.intervalProp.value - 
                          (ms % this.intervalProp.value);
    
    this.setText();
  },

  setText: function() {
    if (this.uptime.greater(new Uptime(this.maxUptime.value))) {
      this.maxUptime.value = this.uptime.toString();
    }
    if (!this.label) return;
    var text = this.textProp.value.trim();
    this.label.data = text + ((text != '')?' ':'') + this.date2String(this.uptime.value);
    dockObj.align();
  },

  getHint: function() {
    var longest = new Uptime(this.maxUptime.value);
    var l = "";
    l = (((!longest.empty()) && (this.uptime.compare(longest) != 0))?
         ("\r\n" + tr("Longest uptime") + ": " + this.date2String(longest.value, true)):
         "");
    return this.date2String(this.uptime.value, true) + l;
  }
});


dockFactory.register({name: tr("Uptime"),
                      className: "DockUpTime",
                      classDef: DockUpTime,
                      category: tr("Info"),
                      description: tr("Show system uptime")
                      });
