
var wfmName    = 0; // Display name
var wfmPercent = 1; // Display signal percent
var wfmSignal  = 2; // Display signal value

/* ------------------------------------------------------------------------ */

var WiFiModeProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("Network Name"), tr("Signal Percentage"), 
                    tr("Signal Value")];
    var optValues = [wfmName, wfmPercent, wfmSignal];
    this.parent("display-mode", "Text Display:", wfmPercent, optTexts, optValues);
    this.className = "WiFiModeProp";
  }
});

/* ------------------------------------------------------------------------ */

var DockWiFi = DockBaseTimer.extend({
  initialize: function() {
    this.parent();
    this.className = "DockWiFi";
    
    this.titleProp = new TextProp("title", "Title:", "",
                                  "Leave blank for network name.");
    this.titleProp.value = this.getName();
    this.showModeProp = new WiFiModeProp();
    this.props = [this.titleProp, this.intervalProp, this.showModeProp];
  },

  destroy: function() {
    if (this.parent()) {
      this.label = null;
      this.image = null;
      this.value = null;
    }
  },

  updateProps: function(loaded) {
    this.parent(loaded);
    if ((this.titleProp.value == "") && (this.showModeProp.value == wfmName)) {
      this.showModeProp.value = wfmPercent;
    }
    this.updateValue();
    dockObj.align();
  },

  getName: function() {
    return tr("WiFi Signal");
  },

  minInterval: function() {
    return 1.0;
  },

  updatePrefs: function() {
    this.parent();
    if (this.label) { $$font(this.label, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    if (this.value) { $$font(this.value, preferences.sensorsFont.value, preferences.sensorsFontStyle.value); }
    dockObj.align();
  },

  updateColor: function() {
    this.parent();
    if (this.label) { this.label.style.color = this.color; }
    if (this.value) { this.value.style.color = this.color; }
    if (this.image) { this.image.colorize = this.color; }
  },

  createControls: function() {
    if (this.parent()) {
      this.label = new Text(mainWindow);
      this.frame.addSubview(this.label);
      this.label.hOffset = 38;
      this.label.vOffset = 11;
      this.label.data = tr("WiFi Signal");
      this.label.style.fontFamily = "Arial, Helvetica";
      this.label.style.fontSize = "9px";
      this.label.style.fontWeight = "bold";
      this.label.style.color = this.color;

      this.image = new Image(mainWindow);
      this.frame.addSubview(this.image);
      this.image.hOffset = 0;
      this.image.vOffset = 1;
      this.image.width = 36;
      this.image.height = 22;
      this.image.missingSrc = "Resources/Sensors/WiFi/0.png";
      this.image.src = "Resources/Sensors/WiFi/0.png";
      this.image.opacity = 190;
      this.image.colorize = "#FFFFFF";

      this.value = new Text(mainWindow);
      this.frame.addSubview(this.value);
      this.value.hOffset = 38;
      this.value.vOffset = 21;
      this.value.data = "";
      this.value.style.fontFamily = "Arial, Helvetica";
      this.value.style.fontSize = "9px";
      this.value.style.fontWeight = "bold";
      this.value.style.color = this.color;

      this.updateValue();
    }
    return (this.frame != null);
  },

  updateValue: function() {
    if (!this.image) return;
    if (!system.wireless.available || !system.wireless.powered ||
        (system.wireless.signal == 100)) {
      this.image.src = "Resources/Sensors/WiFi/0.png";
      if (this.label.data == "") {
        this.label.data = this.getName();
      }
      this.value.data = "offline";
      return;
    }

    var signal = system.wireless.signal;
    var percent = 0;

    if (system.platform == "macintosh") {
      if (signal > 70) signal = 70;
      percent = Math.round(signal * 1.43);
      signal = Math.ceil(signal / 14);
    } else {
      if (signal > 70) signal = 70;
      percent = Math.round(signal * 1.43);

      if (signal > 69) signal = 5;
      else if (signal > 59) signal = 4;
      else if (signal > 55) signal = 3;
      else if (signal > 46) signal = 2;
      else if (signal > 37) signal = 1;
      else signal = 0;
    }
    this.image.src = "Resources/Sensors/WiFi/" + signal + ".png";
    
    var title = this.titleProp.value ? this.titleProp.value : system.wireless.network;
    if (this.label.data != title) {
      this.label.data = title;
    }
    if (this.showModeProp.value == wfmPercent) {
      this.value.data = percent + "%";
    } else if (this.showModeProp.value == wfmName) {
      this.value.data = system.wireless.network;
    } else if (this.showModeProp.value == wfmSignal) {
      this.value.data = system.wireless.signal;
    }
  },

  getHint: function() {
    if (!system.wireless.available || !system.wireless.powered) {
      return tr("WiFi Signal ") + "\n" +
        "   " + tr("Offline");
    }

    var signal = system.wireless.signal;
    var percent = 0;
    if (system.platform == "macintosh") {
      if (signal > 70) signal = 70;
      percent = Math.round(signal * 1.43);
    } else {
      if (signal > 70) signal = 70;
      percent = Math.round(signal * 1.43);
    }
    return tr("WiFi Signal ") + "\n" +
      "   " + tr("Network: ") + system.wireless.network + "\n" +
      "   " + tr("Info: ") + system.wireless.info + "\n" +
      "   " + tr("Noise level: ") + system.wireless.noise + "\n" +
      "   " + tr("Signal level: ") + system.wireless.signal + "\n" +
      "   " + tr("Signal percentage: ") + percent + "%";
  }
});

dockFactory.register({name: tr("WiFi Signal"),
                      className: "DockWiFi",
                      classDef: DockWiFi,
                      category: tr("Network"),
                      description: tr("Show WiFi signal level")
                      });
